(nxml-define-char-name-set 'enclosed-alphanumerics
  '(("CIRCLED DIGIT ONE" #x2460)
    ("CIRCLED DIGIT TWO" #x2461)
    ("CIRCLED DIGIT THREE" #x2462)
    ("CIRCLED DIGIT FOUR" #x2463)
    ("CIRCLED DIGIT FIVE" #x2464)
    ("CIRCLED DIGIT SIX" #x2465)
    ("CIRCLED DIGIT SEVEN" #x2466)
    ("CIRCLED DIGIT EIGHT" #x2467)
    ("CIRCLED DIGIT NINE" #x2468)
    ("CIRCLED NUMBER TEN" #x2469)
    ("CIRCLED NUMBER ELEVEN" #x246A)
    ("CIRCLED NUMBER TWELVE" #x246B)
    ("CIRCLED NUMBER THIRTEEN" #x246C)
    ("CIRCLED NUMBER FOURTEEN" #x246D)
    ("CIRCLED NUMBER FIFTEEN" #x246E)
    ("CIRCLED NUMBER SIXTEEN" #x246F)
    ("CIRCLED NUMBER SEVENTEEN" #x2470)
    ("CIRCLED NUMBER EIGHTEEN" #x2471)
    ("CIRCLED NUMBER NINETEEN" #x2472)
    ("CIRCLED NUMBER TWENTY" #x2473)
    ("PARENTHESIZED DIGIT ONE" #x2474)
    ("PARENTHESIZED DIGIT TWO" #x2475)
    ("PARENTHESIZED DIGIT THREE" #x2476)
    ("PARENTHESIZED DIGIT FOUR" #x2477)
    ("PARENTHESIZED DIGIT FIVE" #x2478)
    ("PARENTHESIZED DIGIT SIX" #x2479)
    ("PARENTHESIZED DIGIT SEVEN" #x247A)
    ("PARENTHESIZED DIGIT EIGHT" #x247B)
    ("PARENTHESIZED DIGIT NINE" #x247C)
    ("PARENTHESIZED NUMBER TEN" #x247D)
    ("PARENTHESIZED NUMBER ELEVEN" #x247E)
    ("PARENTHESIZED NUMBER TWELVE" #x247F)
    ("PARENTHESIZED NUMBER THIRTEEN" #x2480)
    ("PARENTHESIZED NUMBER FOURTEEN" #x2481)
    ("PARENTHESIZED NUMBER FIFTEEN" #x2482)
    ("PARENTHESIZED NUMBER SIXTEEN" #x2483)
    ("PARENTHESIZED NUMBER SEVENTEEN" #x2484)
    ("PARENTHESIZED NUMBER EIGHTEEN" #x2485)
    ("PARENTHESIZED NUMBER NINETEEN" #x2486)
    ("PARENTHESIZED NUMBER TWENTY" #x2487)
    ("DIGIT ONE FULL STOP" #x2488)
    ("DIGIT TWO FULL STOP" #x2489)
    ("DIGIT THREE FULL STOP" #x248A)
    ("DIGIT FOUR FULL STOP" #x248B)
    ("DIGIT FIVE FULL STOP" #x248C)
    ("DIGIT SIX FULL STOP" #x248D)
    ("DIGIT SEVEN FULL STOP" #x248E)
    ("DIGIT EIGHT FULL STOP" #x248F)
    ("DIGIT NINE FULL STOP" #x2490)
    ("NUMBER TEN FULL STOP" #x2491)
    ("NUMBER ELEVEN FULL STOP" #x2492)
    ("NUMBER TWELVE FULL STOP" #x2493)
    ("NUMBER THIRTEEN FULL STOP" #x2494)
    ("NUMBER FOURTEEN FULL STOP" #x2495)
    ("NUMBER FIFTEEN FULL STOP" #x2496)
    ("NUMBER SIXTEEN FULL STOP" #x2497)
    ("NUMBER SEVENTEEN FULL STOP" #x2498)
    ("NUMBER EIGHTEEN FULL STOP" #x2499)
    ("NUMBER NINETEEN FULL STOP" #x249A)
    ("NUMBER TWENTY FULL STOP" #x249B)
    ("PARENTHESIZED LATIN SMALL LETTER A" #x249C)
    ("PARENTHESIZED LATIN SMALL LETTER B" #x249D)
    ("PARENTHESIZED LATIN SMALL LETTER C" #x249E)
    ("PARENTHESIZED LATIN SMALL LETTER D" #x249F)
    ("PARENTHESIZED LATIN SMALL LETTER E" #x24A0)
    ("PARENTHESIZED LATIN SMALL LETTER F" #x24A1)
    ("PARENTHESIZED LATIN SMALL LETTER G" #x24A2)
    ("PARENTHESIZED LATIN SMALL LETTER H" #x24A3)
    ("PARENTHESIZED LATIN SMALL LETTER I" #x24A4)
    ("PARENTHESIZED LATIN SMALL LETTER J" #x24A5)
    ("PARENTHESIZED LATIN SMALL LETTER K" #x24A6)
    ("PARENTHESIZED LATIN SMALL LETTER L" #x24A7)
    ("PARENTHESIZED LATIN SMALL LETTER M" #x24A8)
    ("PARENTHESIZED LATIN SMALL LETTER N" #x24A9)
    ("PARENTHESIZED LATIN SMALL LETTER O" #x24AA)
    ("PARENTHESIZED LATIN SMALL LETTER P" #x24AB)
    ("PARENTHESIZED LATIN SMALL LETTER Q" #x24AC)
    ("PARENTHESIZED LATIN SMALL LETTER R" #x24AD)
    ("PARENTHESIZED LATIN SMALL LETTER S" #x24AE)
    ("PARENTHESIZED LATIN SMALL LETTER T" #x24AF)
    ("PARENTHESIZED LATIN SMALL LETTER U" #x24B0)
    ("PARENTHESIZED LATIN SMALL LETTER V" #x24B1)
    ("PARENTHESIZED LATIN SMALL LETTER W" #x24B2)
    ("PARENTHESIZED LATIN SMALL LETTER X" #x24B3)
    ("PARENTHESIZED LATIN SMALL LETTER Y" #x24B4)
    ("PARENTHESIZED LATIN SMALL LETTER Z" #x24B5)
    ("CIRCLED LATIN CAPITAL LETTER A" #x24B6)
    ("CIRCLED LATIN CAPITAL LETTER B" #x24B7)
    ("CIRCLED LATIN CAPITAL LETTER C" #x24B8)
    ("CIRCLED LATIN CAPITAL LETTER D" #x24B9)
    ("CIRCLED LATIN CAPITAL LETTER E" #x24BA)
    ("CIRCLED LATIN CAPITAL LETTER F" #x24BB)
    ("CIRCLED LATIN CAPITAL LETTER G" #x24BC)
    ("CIRCLED LATIN CAPITAL LETTER H" #x24BD)
    ("CIRCLED LATIN CAPITAL LETTER I" #x24BE)
    ("CIRCLED LATIN CAPITAL LETTER J" #x24BF)
    ("CIRCLED LATIN CAPITAL LETTER K" #x24C0)
    ("CIRCLED LATIN CAPITAL LETTER L" #x24C1)
    ("CIRCLED LATIN CAPITAL LETTER M" #x24C2)
    ("CIRCLED LATIN CAPITAL LETTER N" #x24C3)
    ("CIRCLED LATIN CAPITAL LETTER O" #x24C4)
    ("CIRCLED LATIN CAPITAL LETTER P" #x24C5)
    ("CIRCLED LATIN CAPITAL LETTER Q" #x24C6)
    ("CIRCLED LATIN CAPITAL LETTER R" #x24C7)
    ("CIRCLED LATIN CAPITAL LETTER S" #x24C8)
    ("CIRCLED LATIN CAPITAL LETTER T" #x24C9)
    ("CIRCLED LATIN CAPITAL LETTER U" #x24CA)
    ("CIRCLED LATIN CAPITAL LETTER V" #x24CB)
    ("CIRCLED LATIN CAPITAL LETTER W" #x24CC)
    ("CIRCLED LATIN CAPITAL LETTER X" #x24CD)
    ("CIRCLED LATIN CAPITAL LETTER Y" #x24CE)
    ("CIRCLED LATIN CAPITAL LETTER Z" #x24CF)
    ("CIRCLED LATIN SMALL LETTER A" #x24D0)
    ("CIRCLED LATIN SMALL LETTER B" #x24D1)
    ("CIRCLED LATIN SMALL LETTER C" #x24D2)
    ("CIRCLED LATIN SMALL LETTER D" #x24D3)
    ("CIRCLED LATIN SMALL LETTER E" #x24D4)
    ("CIRCLED LATIN SMALL LETTER F" #x24D5)
    ("CIRCLED LATIN SMALL LETTER G" #x24D6)
    ("CIRCLED LATIN SMALL LETTER H" #x24D7)
    ("CIRCLED LATIN SMALL LETTER I" #x24D8)
    ("CIRCLED LATIN SMALL LETTER J" #x24D9)
    ("CIRCLED LATIN SMALL LETTER K" #x24DA)
    ("CIRCLED LATIN SMALL LETTER L" #x24DB)
    ("CIRCLED LATIN SMALL LETTER M" #x24DC)
    ("CIRCLED LATIN SMALL LETTER N" #x24DD)
    ("CIRCLED LATIN SMALL LETTER O" #x24DE)
    ("CIRCLED LATIN SMALL LETTER P" #x24DF)
    ("CIRCLED LATIN SMALL LETTER Q" #x24E0)
    ("CIRCLED LATIN SMALL LETTER R" #x24E1)
    ("CIRCLED LATIN SMALL LETTER S" #x24E2)
    ("CIRCLED LATIN SMALL LETTER T" #x24E3)
    ("CIRCLED LATIN SMALL LETTER U" #x24E4)
    ("CIRCLED LATIN SMALL LETTER V" #x24E5)
    ("CIRCLED LATIN SMALL LETTER W" #x24E6)
    ("CIRCLED LATIN SMALL LETTER X" #x24E7)
    ("CIRCLED LATIN SMALL LETTER Y" #x24E8)
    ("CIRCLED LATIN SMALL LETTER Z" #x24E9)
    ("CIRCLED DIGIT ZERO" #x24EA)
    ("NEGATIVE CIRCLED NUMBER ELEVEN" #x24EB)
    ("NEGATIVE CIRCLED NUMBER TWELVE" #x24EC)
    ("NEGATIVE CIRCLED NUMBER THIRTEEN" #x24ED)
    ("NEGATIVE CIRCLED NUMBER FOURTEEN" #x24EE)
    ("NEGATIVE CIRCLED NUMBER FIFTEEN" #x24EF)
    ("NEGATIVE CIRCLED NUMBER SIXTEEN" #x24F0)
    ("NEGATIVE CIRCLED NUMBER SEVENTEEN" #x24F1)
    ("NEGATIVE CIRCLED NUMBER EIGHTEEN" #x24F2)
    ("NEGATIVE CIRCLED NUMBER NINETEEN" #x24F3)
    ("NEGATIVE CIRCLED NUMBER TWENTY" #x24F4)
    ("DOUBLE CIRCLED DIGIT ONE" #x24F5)
    ("DOUBLE CIRCLED DIGIT TWO" #x24F6)
    ("DOUBLE CIRCLED DIGIT THREE" #x24F7)
    ("DOUBLE CIRCLED DIGIT FOUR" #x24F8)
    ("DOUBLE CIRCLED DIGIT FIVE" #x24F9)
    ("DOUBLE CIRCLED DIGIT SIX" #x24FA)
    ("DOUBLE CIRCLED DIGIT SEVEN" #x24FB)
    ("DOUBLE CIRCLED DIGIT EIGHT" #x24FC)
    ("DOUBLE CIRCLED DIGIT NINE" #x24FD)
    ("DOUBLE CIRCLED NUMBER TEN" #x24FE)
    ))

;; arch-tag: 2ec7ede5-ce9d-48b5-a725-dd3f177060f4
