(nxml-define-char-name-set 'mathematical-alphanumeric-symbols
  '(("MATHEMATICAL BOLD CAPITAL A" #x1D400)
    ("MATHEMATICAL BOLD CAPITAL B" #x1D401)
    ("MATHEMATICAL BOLD CAPITAL C" #x1D402)
    ("MATHEMATICAL BOLD CAPITAL D" #x1D403)
    ("MATHEMATICAL BOLD CAPITAL E" #x1D404)
    ("MATHEMATICAL BOLD CAPITAL F" #x1D405)
    ("MATHEMATICAL BOLD CAPITAL G" #x1D406)
    ("MATHEMATICAL BOLD CAPITAL H" #x1D407)
    ("MATHEMATICAL BOLD CAPITAL I" #x1D408)
    ("MATHEMATICAL BOLD CAPITAL J" #x1D409)
    ("MATHEMATICAL BOLD CAPITAL K" #x1D40A)
    ("MATHEMATICAL BOLD CAPITAL L" #x1D40B)
    ("MATHEMATICAL BOLD CAPITAL M" #x1D40C)
    ("MATHEMATICAL BOLD CAPITAL N" #x1D40D)
    ("MATHEMATICAL BOLD CAPITAL O" #x1D40E)
    ("MATHEMATICAL BOLD CAPITAL P" #x1D40F)
    ("MATHEMATICAL BOLD CAPITAL Q" #x1D410)
    ("MATHEMATICAL BOLD CAPITAL R" #x1D411)
    ("MATHEMATICAL BOLD CAPITAL S" #x1D412)
    ("MATHEMATICAL BOLD CAPITAL T" #x1D413)
    ("MATHEMATICAL BOLD CAPITAL U" #x1D414)
    ("MATHEMATICAL BOLD CAPITAL V" #x1D415)
    ("MATHEMATICAL BOLD CAPITAL W" #x1D416)
    ("MATHEMATICAL BOLD CAPITAL X" #x1D417)
    ("MATHEMATICAL BOLD CAPITAL Y" #x1D418)
    ("MATHEMATICAL BOLD CAPITAL Z" #x1D419)
    ("MATHEMATICAL BOLD SMALL A" #x1D41A)
    ("MATHEMATICAL BOLD SMALL B" #x1D41B)
    ("MATHEMATICAL BOLD SMALL C" #x1D41C)
    ("MATHEMATICAL BOLD SMALL D" #x1D41D)
    ("MATHEMATICAL BOLD SMALL E" #x1D41E)
    ("MATHEMATICAL BOLD SMALL F" #x1D41F)
    ("MATHEMATICAL BOLD SMALL G" #x1D420)
    ("MATHEMATICAL BOLD SMALL H" #x1D421)
    ("MATHEMATICAL BOLD SMALL I" #x1D422)
    ("MATHEMATICAL BOLD SMALL J" #x1D423)
    ("MATHEMATICAL BOLD SMALL K" #x1D424)
    ("MATHEMATICAL BOLD SMALL L" #x1D425)
    ("MATHEMATICAL BOLD SMALL M" #x1D426)
    ("MATHEMATICAL BOLD SMALL N" #x1D427)
    ("MATHEMATICAL BOLD SMALL O" #x1D428)
    ("MATHEMATICAL BOLD SMALL P" #x1D429)
    ("MATHEMATICAL BOLD SMALL Q" #x1D42A)
    ("MATHEMATICAL BOLD SMALL R" #x1D42B)
    ("MATHEMATICAL BOLD SMALL S" #x1D42C)
    ("MATHEMATICAL BOLD SMALL T" #x1D42D)
    ("MATHEMATICAL BOLD SMALL U" #x1D42E)
    ("MATHEMATICAL BOLD SMALL V" #x1D42F)
    ("MATHEMATICAL BOLD SMALL W" #x1D430)
    ("MATHEMATICAL BOLD SMALL X" #x1D431)
    ("MATHEMATICAL BOLD SMALL Y" #x1D432)
    ("MATHEMATICAL BOLD SMALL Z" #x1D433)
    ("MATHEMATICAL ITALIC CAPITAL A" #x1D434)
    ("MATHEMATICAL ITALIC CAPITAL B" #x1D435)
    ("MATHEMATICAL ITALIC CAPITAL C" #x1D436)
    ("MATHEMATICAL ITALIC CAPITAL D" #x1D437)
    ("MATHEMATICAL ITALIC CAPITAL E" #x1D438)
    ("MATHEMATICAL ITALIC CAPITAL F" #x1D439)
    ("MATHEMATICAL ITALIC CAPITAL G" #x1D43A)
    ("MATHEMATICAL ITALIC CAPITAL H" #x1D43B)
    ("MATHEMATICAL ITALIC CAPITAL I" #x1D43C)
    ("MATHEMATICAL ITALIC CAPITAL J" #x1D43D)
    ("MATHEMATICAL ITALIC CAPITAL K" #x1D43E)
    ("MATHEMATICAL ITALIC CAPITAL L" #x1D43F)
    ("MATHEMATICAL ITALIC CAPITAL M" #x1D440)
    ("MATHEMATICAL ITALIC CAPITAL N" #x1D441)
    ("MATHEMATICAL ITALIC CAPITAL O" #x1D442)
    ("MATHEMATICAL ITALIC CAPITAL P" #x1D443)
    ("MATHEMATICAL ITALIC CAPITAL Q" #x1D444)
    ("MATHEMATICAL ITALIC CAPITAL R" #x1D445)
    ("MATHEMATICAL ITALIC CAPITAL S" #x1D446)
    ("MATHEMATICAL ITALIC CAPITAL T" #x1D447)
    ("MATHEMATICAL ITALIC CAPITAL U" #x1D448)
    ("MATHEMATICAL ITALIC CAPITAL V" #x1D449)
    ("MATHEMATICAL ITALIC CAPITAL W" #x1D44A)
    ("MATHEMATICAL ITALIC CAPITAL X" #x1D44B)
    ("MATHEMATICAL ITALIC CAPITAL Y" #x1D44C)
    ("MATHEMATICAL ITALIC CAPITAL Z" #x1D44D)
    ("MATHEMATICAL ITALIC SMALL A" #x1D44E)
    ("MATHEMATICAL ITALIC SMALL B" #x1D44F)
    ("MATHEMATICAL ITALIC SMALL C" #x1D450)
    ("MATHEMATICAL ITALIC SMALL D" #x1D451)
    ("MATHEMATICAL ITALIC SMALL E" #x1D452)
    ("MATHEMATICAL ITALIC SMALL F" #x1D453)
    ("MATHEMATICAL ITALIC SMALL G" #x1D454)
    ("MATHEMATICAL ITALIC SMALL I" #x1D456)
    ("MATHEMATICAL ITALIC SMALL J" #x1D457)
    ("MATHEMATICAL ITALIC SMALL K" #x1D458)
    ("MATHEMATICAL ITALIC SMALL L" #x1D459)
    ("MATHEMATICAL ITALIC SMALL M" #x1D45A)
    ("MATHEMATICAL ITALIC SMALL N" #x1D45B)
    ("MATHEMATICAL ITALIC SMALL O" #x1D45C)
    ("MATHEMATICAL ITALIC SMALL P" #x1D45D)
    ("MATHEMATICAL ITALIC SMALL Q" #x1D45E)
    ("MATHEMATICAL ITALIC SMALL R" #x1D45F)
    ("MATHEMATICAL ITALIC SMALL S" #x1D460)
    ("MATHEMATICAL ITALIC SMALL T" #x1D461)
    ("MATHEMATICAL ITALIC SMALL U" #x1D462)
    ("MATHEMATICAL ITALIC SMALL V" #x1D463)
    ("MATHEMATICAL ITALIC SMALL W" #x1D464)
    ("MATHEMATICAL ITALIC SMALL X" #x1D465)
    ("MATHEMATICAL ITALIC SMALL Y" #x1D466)
    ("MATHEMATICAL ITALIC SMALL Z" #x1D467)
    ("MATHEMATICAL BOLD ITALIC CAPITAL A" #x1D468)
    ("MATHEMATICAL BOLD ITALIC CAPITAL B" #x1D469)
    ("MATHEMATICAL BOLD ITALIC CAPITAL C" #x1D46A)
    ("MATHEMATICAL BOLD ITALIC CAPITAL D" #x1D46B)
    ("MATHEMATICAL BOLD ITALIC CAPITAL E" #x1D46C)
    ("MATHEMATICAL BOLD ITALIC CAPITAL F" #x1D46D)
    ("MATHEMATICAL BOLD ITALIC CAPITAL G" #x1D46E)
    ("MATHEMATICAL BOLD ITALIC CAPITAL H" #x1D46F)
    ("MATHEMATICAL BOLD ITALIC CAPITAL I" #x1D470)
    ("MATHEMATICAL BOLD ITALIC CAPITAL J" #x1D471)
    ("MATHEMATICAL BOLD ITALIC CAPITAL K" #x1D472)
    ("MATHEMATICAL BOLD ITALIC CAPITAL L" #x1D473)
    ("MATHEMATICAL BOLD ITALIC CAPITAL M" #x1D474)
    ("MATHEMATICAL BOLD ITALIC CAPITAL N" #x1D475)
    ("MATHEMATICAL BOLD ITALIC CAPITAL O" #x1D476)
    ("MATHEMATICAL BOLD ITALIC CAPITAL P" #x1D477)
    ("MATHEMATICAL BOLD ITALIC CAPITAL Q" #x1D478)
    ("MATHEMATICAL BOLD ITALIC CAPITAL R" #x1D479)
    ("MATHEMATICAL BOLD ITALIC CAPITAL S" #x1D47A)
    ("MATHEMATICAL BOLD ITALIC CAPITAL T" #x1D47B)
    ("MATHEMATICAL BOLD ITALIC CAPITAL U" #x1D47C)
    ("MATHEMATICAL BOLD ITALIC CAPITAL V" #x1D47D)
    ("MATHEMATICAL BOLD ITALIC CAPITAL W" #x1D47E)
    ("MATHEMATICAL BOLD ITALIC CAPITAL X" #x1D47F)
    ("MATHEMATICAL BOLD ITALIC CAPITAL Y" #x1D480)
    ("MATHEMATICAL BOLD ITALIC CAPITAL Z" #x1D481)
    ("MATHEMATICAL BOLD ITALIC SMALL A" #x1D482)
    ("MATHEMATICAL BOLD ITALIC SMALL B" #x1D483)
    ("MATHEMATICAL BOLD ITALIC SMALL C" #x1D484)
    ("MATHEMATICAL BOLD ITALIC SMALL D" #x1D485)
    ("MATHEMATICAL BOLD ITALIC SMALL E" #x1D486)
    ("MATHEMATICAL BOLD ITALIC SMALL F" #x1D487)
    ("MATHEMATICAL BOLD ITALIC SMALL G" #x1D488)
    ("MATHEMATICAL BOLD ITALIC SMALL H" #x1D489)
    ("MATHEMATICAL BOLD ITALIC SMALL I" #x1D48A)
    ("MATHEMATICAL BOLD ITALIC SMALL J" #x1D48B)
    ("MATHEMATICAL BOLD ITALIC SMALL K" #x1D48C)
    ("MATHEMATICAL BOLD ITALIC SMALL L" #x1D48D)
    ("MATHEMATICAL BOLD ITALIC SMALL M" #x1D48E)
    ("MATHEMATICAL BOLD ITALIC SMALL N" #x1D48F)
    ("MATHEMATICAL BOLD ITALIC SMALL O" #x1D490)
    ("MATHEMATICAL BOLD ITALIC SMALL P" #x1D491)
    ("MATHEMATICAL BOLD ITALIC SMALL Q" #x1D492)
    ("MATHEMATICAL BOLD ITALIC SMALL R" #x1D493)
    ("MATHEMATICAL BOLD ITALIC SMALL S" #x1D494)
    ("MATHEMATICAL BOLD ITALIC SMALL T" #x1D495)
    ("MATHEMATICAL BOLD ITALIC SMALL U" #x1D496)
    ("MATHEMATICAL BOLD ITALIC SMALL V" #x1D497)
    ("MATHEMATICAL BOLD ITALIC SMALL W" #x1D498)
    ("MATHEMATICAL BOLD ITALIC SMALL X" #x1D499)
    ("MATHEMATICAL BOLD ITALIC SMALL Y" #x1D49A)
    ("MATHEMATICAL BOLD ITALIC SMALL Z" #x1D49B)
    ("MATHEMATICAL SCRIPT CAPITAL A" #x1D49C)
    ("MATHEMATICAL SCRIPT CAPITAL C" #x1D49E)
    ("MATHEMATICAL SCRIPT CAPITAL D" #x1D49F)
    ("MATHEMATICAL SCRIPT CAPITAL G" #x1D4A2)
    ("MATHEMATICAL SCRIPT CAPITAL J" #x1D4A5)
    ("MATHEMATICAL SCRIPT CAPITAL K" #x1D4A6)
    ("MATHEMATICAL SCRIPT CAPITAL N" #x1D4A9)
    ("MATHEMATICAL SCRIPT CAPITAL O" #x1D4AA)
    ("MATHEMATICAL SCRIPT CAPITAL P" #x1D4AB)
    ("MATHEMATICAL SCRIPT CAPITAL Q" #x1D4AC)
    ("MATHEMATICAL SCRIPT CAPITAL S" #x1D4AE)
    ("MATHEMATICAL SCRIPT CAPITAL T" #x1D4AF)
    ("MATHEMATICAL SCRIPT CAPITAL U" #x1D4B0)
    ("MATHEMATICAL SCRIPT CAPITAL V" #x1D4B1)
    ("MATHEMATICAL SCRIPT CAPITAL W" #x1D4B2)
    ("MATHEMATICAL SCRIPT CAPITAL X" #x1D4B3)
    ("MATHEMATICAL SCRIPT CAPITAL Y" #x1D4B4)
    ("MATHEMATICAL SCRIPT CAPITAL Z" #x1D4B5)
    ("MATHEMATICAL SCRIPT SMALL A" #x1D4B6)
    ("MATHEMATICAL SCRIPT SMALL B" #x1D4B7)
    ("MATHEMATICAL SCRIPT SMALL C" #x1D4B8)
    ("MATHEMATICAL SCRIPT SMALL D" #x1D4B9)
    ("MATHEMATICAL SCRIPT SMALL F" #x1D4BB)
    ("MATHEMATICAL SCRIPT SMALL H" #x1D4BD)
    ("MATHEMATICAL SCRIPT SMALL I" #x1D4BE)
    ("MATHEMATICAL SCRIPT SMALL J" #x1D4BF)
    ("MATHEMATICAL SCRIPT SMALL K" #x1D4C0)
    ("MATHEMATICAL SCRIPT SMALL M" #x1D4C2)
    ("MATHEMATICAL SCRIPT SMALL N" #x1D4C3)
    ("MATHEMATICAL SCRIPT SMALL P" #x1D4C5)
    ("MATHEMATICAL SCRIPT SMALL Q" #x1D4C6)
    ("MATHEMATICAL SCRIPT SMALL R" #x1D4C7)
    ("MATHEMATICAL SCRIPT SMALL S" #x1D4C8)
    ("MATHEMATICAL SCRIPT SMALL T" #x1D4C9)
    ("MATHEMATICAL SCRIPT SMALL U" #x1D4CA)
    ("MATHEMATICAL SCRIPT SMALL V" #x1D4CB)
    ("MATHEMATICAL SCRIPT SMALL W" #x1D4CC)
    ("MATHEMATICAL SCRIPT SMALL X" #x1D4CD)
    ("MATHEMATICAL SCRIPT SMALL Y" #x1D4CE)
    ("MATHEMATICAL SCRIPT SMALL Z" #x1D4CF)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL A" #x1D4D0)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL B" #x1D4D1)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL C" #x1D4D2)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL D" #x1D4D3)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL E" #x1D4D4)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL F" #x1D4D5)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL G" #x1D4D6)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL H" #x1D4D7)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL I" #x1D4D8)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL J" #x1D4D9)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL K" #x1D4DA)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL L" #x1D4DB)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL M" #x1D4DC)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL N" #x1D4DD)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL O" #x1D4DE)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL P" #x1D4DF)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL Q" #x1D4E0)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL R" #x1D4E1)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL S" #x1D4E2)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL T" #x1D4E3)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL U" #x1D4E4)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL V" #x1D4E5)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL W" #x1D4E6)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL X" #x1D4E7)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL Y" #x1D4E8)
    ("MATHEMATICAL BOLD SCRIPT CAPITAL Z" #x1D4E9)
    ("MATHEMATICAL BOLD SCRIPT SMALL A" #x1D4EA)
    ("MATHEMATICAL BOLD SCRIPT SMALL B" #x1D4EB)
    ("MATHEMATICAL BOLD SCRIPT SMALL C" #x1D4EC)
    ("MATHEMATICAL BOLD SCRIPT SMALL D" #x1D4ED)
    ("MATHEMATICAL BOLD SCRIPT SMALL E" #x1D4EE)
    ("MATHEMATICAL BOLD SCRIPT SMALL F" #x1D4EF)
    ("MATHEMATICAL BOLD SCRIPT SMALL G" #x1D4F0)
    ("MATHEMATICAL BOLD SCRIPT SMALL H" #x1D4F1)
    ("MATHEMATICAL BOLD SCRIPT SMALL I" #x1D4F2)
    ("MATHEMATICAL BOLD SCRIPT SMALL J" #x1D4F3)
    ("MATHEMATICAL BOLD SCRIPT SMALL K" #x1D4F4)
    ("MATHEMATICAL BOLD SCRIPT SMALL L" #x1D4F5)
    ("MATHEMATICAL BOLD SCRIPT SMALL M" #x1D4F6)
    ("MATHEMATICAL BOLD SCRIPT SMALL N" #x1D4F7)
    ("MATHEMATICAL BOLD SCRIPT SMALL O" #x1D4F8)
    ("MATHEMATICAL BOLD SCRIPT SMALL P" #x1D4F9)
    ("MATHEMATICAL BOLD SCRIPT SMALL Q" #x1D4FA)
    ("MATHEMATICAL BOLD SCRIPT SMALL R" #x1D4FB)
    ("MATHEMATICAL BOLD SCRIPT SMALL S" #x1D4FC)
    ("MATHEMATICAL BOLD SCRIPT SMALL T" #x1D4FD)
    ("MATHEMATICAL BOLD SCRIPT SMALL U" #x1D4FE)
    ("MATHEMATICAL BOLD SCRIPT SMALL V" #x1D4FF)
    ("MATHEMATICAL BOLD SCRIPT SMALL W" #x1D500)
    ("MATHEMATICAL BOLD SCRIPT SMALL X" #x1D501)
    ("MATHEMATICAL BOLD SCRIPT SMALL Y" #x1D502)
    ("MATHEMATICAL BOLD SCRIPT SMALL Z" #x1D503)
    ("MATHEMATICAL FRAKTUR CAPITAL A" #x1D504)
    ("MATHEMATICAL FRAKTUR CAPITAL B" #x1D505)
    ("MATHEMATICAL FRAKTUR CAPITAL D" #x1D507)
    ("MATHEMATICAL FRAKTUR CAPITAL E" #x1D508)
    ("MATHEMATICAL FRAKTUR CAPITAL F" #x1D509)
    ("MATHEMATICAL FRAKTUR CAPITAL G" #x1D50A)
    ("MATHEMATICAL FRAKTUR CAPITAL J" #x1D50D)
    ("MATHEMATICAL FRAKTUR CAPITAL K" #x1D50E)
    ("MATHEMATICAL FRAKTUR CAPITAL L" #x1D50F)
    ("MATHEMATICAL FRAKTUR CAPITAL M" #x1D510)
    ("MATHEMATICAL FRAKTUR CAPITAL N" #x1D511)
    ("MATHEMATICAL FRAKTUR CAPITAL O" #x1D512)
    ("MATHEMATICAL FRAKTUR CAPITAL P" #x1D513)
    ("MATHEMATICAL FRAKTUR CAPITAL Q" #x1D514)
    ("MATHEMATICAL FRAKTUR CAPITAL S" #x1D516)
    ("MATHEMATICAL FRAKTUR CAPITAL T" #x1D517)
    ("MATHEMATICAL FRAKTUR CAPITAL U" #x1D518)
    ("MATHEMATICAL FRAKTUR CAPITAL V" #x1D519)
    ("MATHEMATICAL FRAKTUR CAPITAL W" #x1D51A)
    ("MATHEMATICAL FRAKTUR CAPITAL X" #x1D51B)
    ("MATHEMATICAL FRAKTUR CAPITAL Y" #x1D51C)
    ("MATHEMATICAL FRAKTUR SMALL A" #x1D51E)
    ("MATHEMATICAL FRAKTUR SMALL B" #x1D51F)
    ("MATHEMATICAL FRAKTUR SMALL C" #x1D520)
    ("MATHEMATICAL FRAKTUR SMALL D" #x1D521)
    ("MATHEMATICAL FRAKTUR SMALL E" #x1D522)
    ("MATHEMATICAL FRAKTUR SMALL F" #x1D523)
    ("MATHEMATICAL FRAKTUR SMALL G" #x1D524)
    ("MATHEMATICAL FRAKTUR SMALL H" #x1D525)
    ("MATHEMATICAL FRAKTUR SMALL I" #x1D526)
    ("MATHEMATICAL FRAKTUR SMALL J" #x1D527)
    ("MATHEMATICAL FRAKTUR SMALL K" #x1D528)
    ("MATHEMATICAL FRAKTUR SMALL L" #x1D529)
    ("MATHEMATICAL FRAKTUR SMALL M" #x1D52A)
    ("MATHEMATICAL FRAKTUR SMALL N" #x1D52B)
    ("MATHEMATICAL FRAKTUR SMALL O" #x1D52C)
    ("MATHEMATICAL FRAKTUR SMALL P" #x1D52D)
    ("MATHEMATICAL FRAKTUR SMALL Q" #x1D52E)
    ("MATHEMATICAL FRAKTUR SMALL R" #x1D52F)
    ("MATHEMATICAL FRAKTUR SMALL S" #x1D530)
    ("MATHEMATICAL FRAKTUR SMALL T" #x1D531)
    ("MATHEMATICAL FRAKTUR SMALL U" #x1D532)
    ("MATHEMATICAL FRAKTUR SMALL V" #x1D533)
    ("MATHEMATICAL FRAKTUR SMALL W" #x1D534)
    ("MATHEMATICAL FRAKTUR SMALL X" #x1D535)
    ("MATHEMATICAL FRAKTUR SMALL Y" #x1D536)
    ("MATHEMATICAL FRAKTUR SMALL Z" #x1D537)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL A" #x1D538)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL B" #x1D539)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL D" #x1D53B)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL E" #x1D53C)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL F" #x1D53D)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL G" #x1D53E)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL I" #x1D540)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL J" #x1D541)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL K" #x1D542)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL L" #x1D543)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL M" #x1D544)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL O" #x1D546)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL S" #x1D54A)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL T" #x1D54B)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL U" #x1D54C)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL V" #x1D54D)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL W" #x1D54E)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL X" #x1D54F)
    ("MATHEMATICAL DOUBLE-STRUCK CAPITAL Y" #x1D550)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL A" #x1D552)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL B" #x1D553)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL C" #x1D554)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL D" #x1D555)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL E" #x1D556)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL F" #x1D557)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL G" #x1D558)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL H" #x1D559)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL I" #x1D55A)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL J" #x1D55B)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL K" #x1D55C)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL L" #x1D55D)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL M" #x1D55E)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL N" #x1D55F)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL O" #x1D560)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL P" #x1D561)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL Q" #x1D562)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL R" #x1D563)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL S" #x1D564)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL T" #x1D565)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL U" #x1D566)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL V" #x1D567)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL W" #x1D568)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL X" #x1D569)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL Y" #x1D56A)
    ("MATHEMATICAL DOUBLE-STRUCK SMALL Z" #x1D56B)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL A" #x1D56C)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL B" #x1D56D)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL C" #x1D56E)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL D" #x1D56F)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL E" #x1D570)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL F" #x1D571)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL G" #x1D572)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL H" #x1D573)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL I" #x1D574)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL J" #x1D575)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL K" #x1D576)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL L" #x1D577)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL M" #x1D578)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL N" #x1D579)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL O" #x1D57A)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL P" #x1D57B)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL Q" #x1D57C)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL R" #x1D57D)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL S" #x1D57E)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL T" #x1D57F)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL U" #x1D580)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL V" #x1D581)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL W" #x1D582)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL X" #x1D583)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL Y" #x1D584)
    ("MATHEMATICAL BOLD FRAKTUR CAPITAL Z" #x1D585)
    ("MATHEMATICAL BOLD FRAKTUR SMALL A" #x1D586)
    ("MATHEMATICAL BOLD FRAKTUR SMALL B" #x1D587)
    ("MATHEMATICAL BOLD FRAKTUR SMALL C" #x1D588)
    ("MATHEMATICAL BOLD FRAKTUR SMALL D" #x1D589)
    ("MATHEMATICAL BOLD FRAKTUR SMALL E" #x1D58A)
    ("MATHEMATICAL BOLD FRAKTUR SMALL F" #x1D58B)
    ("MATHEMATICAL BOLD FRAKTUR SMALL G" #x1D58C)
    ("MATHEMATICAL BOLD FRAKTUR SMALL H" #x1D58D)
    ("MATHEMATICAL BOLD FRAKTUR SMALL I" #x1D58E)
    ("MATHEMATICAL BOLD FRAKTUR SMALL J" #x1D58F)
    ("MATHEMATICAL BOLD FRAKTUR SMALL K" #x1D590)
    ("MATHEMATICAL BOLD FRAKTUR SMALL L" #x1D591)
    ("MATHEMATICAL BOLD FRAKTUR SMALL M" #x1D592)
    ("MATHEMATICAL BOLD FRAKTUR SMALL N" #x1D593)
    ("MATHEMATICAL BOLD FRAKTUR SMALL O" #x1D594)
    ("MATHEMATICAL BOLD FRAKTUR SMALL P" #x1D595)
    ("MATHEMATICAL BOLD FRAKTUR SMALL Q" #x1D596)
    ("MATHEMATICAL BOLD FRAKTUR SMALL R" #x1D597)
    ("MATHEMATICAL BOLD FRAKTUR SMALL S" #x1D598)
    ("MATHEMATICAL BOLD FRAKTUR SMALL T" #x1D599)
    ("MATHEMATICAL BOLD FRAKTUR SMALL U" #x1D59A)
    ("MATHEMATICAL BOLD FRAKTUR SMALL V" #x1D59B)
    ("MATHEMATICAL BOLD FRAKTUR SMALL W" #x1D59C)
    ("MATHEMATICAL BOLD FRAKTUR SMALL X" #x1D59D)
    ("MATHEMATICAL BOLD FRAKTUR SMALL Y" #x1D59E)
    ("MATHEMATICAL BOLD FRAKTUR SMALL Z" #x1D59F)
    ("MATHEMATICAL SANS-SERIF CAPITAL A" #x1D5A0)
    ("MATHEMATICAL SANS-SERIF CAPITAL B" #x1D5A1)
    ("MATHEMATICAL SANS-SERIF CAPITAL C" #x1D5A2)
    ("MATHEMATICAL SANS-SERIF CAPITAL D" #x1D5A3)
    ("MATHEMATICAL SANS-SERIF CAPITAL E" #x1D5A4)
    ("MATHEMATICAL SANS-SERIF CAPITAL F" #x1D5A5)
    ("MATHEMATICAL SANS-SERIF CAPITAL G" #x1D5A6)
    ("MATHEMATICAL SANS-SERIF CAPITAL H" #x1D5A7)
    ("MATHEMATICAL SANS-SERIF CAPITAL I" #x1D5A8)
    ("MATHEMATICAL SANS-SERIF CAPITAL J" #x1D5A9)
    ("MATHEMATICAL SANS-SERIF CAPITAL K" #x1D5AA)
    ("MATHEMATICAL SANS-SERIF CAPITAL L" #x1D5AB)
    ("MATHEMATICAL SANS-SERIF CAPITAL M" #x1D5AC)
    ("MATHEMATICAL SANS-SERIF CAPITAL N" #x1D5AD)
    ("MATHEMATICAL SANS-SERIF CAPITAL O" #x1D5AE)
    ("MATHEMATICAL SANS-SERIF CAPITAL P" #x1D5AF)
    ("MATHEMATICAL SANS-SERIF CAPITAL Q" #x1D5B0)
    ("MATHEMATICAL SANS-SERIF CAPITAL R" #x1D5B1)
    ("MATHEMATICAL SANS-SERIF CAPITAL S" #x1D5B2)
    ("MATHEMATICAL SANS-SERIF CAPITAL T" #x1D5B3)
    ("MATHEMATICAL SANS-SERIF CAPITAL U" #x1D5B4)
    ("MATHEMATICAL SANS-SERIF CAPITAL V" #x1D5B5)
    ("MATHEMATICAL SANS-SERIF CAPITAL W" #x1D5B6)
    ("MATHEMATICAL SANS-SERIF CAPITAL X" #x1D5B7)
    ("MATHEMATICAL SANS-SERIF CAPITAL Y" #x1D5B8)
    ("MATHEMATICAL SANS-SERIF CAPITAL Z" #x1D5B9)
    ("MATHEMATICAL SANS-SERIF SMALL A" #x1D5BA)
    ("MATHEMATICAL SANS-SERIF SMALL B" #x1D5BB)
    ("MATHEMATICAL SANS-SERIF SMALL C" #x1D5BC)
    ("MATHEMATICAL SANS-SERIF SMALL D" #x1D5BD)
    ("MATHEMATICAL SANS-SERIF SMALL E" #x1D5BE)
    ("MATHEMATICAL SANS-SERIF SMALL F" #x1D5BF)
    ("MATHEMATICAL SANS-SERIF SMALL G" #x1D5C0)
    ("MATHEMATICAL SANS-SERIF SMALL H" #x1D5C1)
    ("MATHEMATICAL SANS-SERIF SMALL I" #x1D5C2)
    ("MATHEMATICAL SANS-SERIF SMALL J" #x1D5C3)
    ("MATHEMATICAL SANS-SERIF SMALL K" #x1D5C4)
    ("MATHEMATICAL SANS-SERIF SMALL L" #x1D5C5)
    ("MATHEMATICAL SANS-SERIF SMALL M" #x1D5C6)
    ("MATHEMATICAL SANS-SERIF SMALL N" #x1D5C7)
    ("MATHEMATICAL SANS-SERIF SMALL O" #x1D5C8)
    ("MATHEMATICAL SANS-SERIF SMALL P" #x1D5C9)
    ("MATHEMATICAL SANS-SERIF SMALL Q" #x1D5CA)
    ("MATHEMATICAL SANS-SERIF SMALL R" #x1D5CB)
    ("MATHEMATICAL SANS-SERIF SMALL S" #x1D5CC)
    ("MATHEMATICAL SANS-SERIF SMALL T" #x1D5CD)
    ("MATHEMATICAL SANS-SERIF SMALL U" #x1D5CE)
    ("MATHEMATICAL SANS-SERIF SMALL V" #x1D5CF)
    ("MATHEMATICAL SANS-SERIF SMALL W" #x1D5D0)
    ("MATHEMATICAL SANS-SERIF SMALL X" #x1D5D1)
    ("MATHEMATICAL SANS-SERIF SMALL Y" #x1D5D2)
    ("MATHEMATICAL SANS-SERIF SMALL Z" #x1D5D3)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL A" #x1D5D4)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL B" #x1D5D5)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL C" #x1D5D6)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL D" #x1D5D7)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL E" #x1D5D8)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL F" #x1D5D9)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL G" #x1D5DA)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL H" #x1D5DB)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL I" #x1D5DC)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL J" #x1D5DD)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL K" #x1D5DE)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL L" #x1D5DF)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL M" #x1D5E0)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL N" #x1D5E1)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL O" #x1D5E2)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL P" #x1D5E3)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL Q" #x1D5E4)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL R" #x1D5E5)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL S" #x1D5E6)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL T" #x1D5E7)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL U" #x1D5E8)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL V" #x1D5E9)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL W" #x1D5EA)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL X" #x1D5EB)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL Y" #x1D5EC)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL Z" #x1D5ED)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL A" #x1D5EE)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL B" #x1D5EF)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL C" #x1D5F0)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL D" #x1D5F1)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL E" #x1D5F2)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL F" #x1D5F3)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL G" #x1D5F4)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL H" #x1D5F5)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL I" #x1D5F6)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL J" #x1D5F7)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL K" #x1D5F8)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL L" #x1D5F9)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL M" #x1D5FA)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL N" #x1D5FB)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL O" #x1D5FC)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL P" #x1D5FD)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL Q" #x1D5FE)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL R" #x1D5FF)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL S" #x1D600)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL T" #x1D601)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL U" #x1D602)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL V" #x1D603)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL W" #x1D604)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL X" #x1D605)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL Y" #x1D606)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL Z" #x1D607)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL A" #x1D608)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL B" #x1D609)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL C" #x1D60A)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL D" #x1D60B)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL E" #x1D60C)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL F" #x1D60D)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL G" #x1D60E)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL H" #x1D60F)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL I" #x1D610)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL J" #x1D611)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL K" #x1D612)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL L" #x1D613)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL M" #x1D614)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL N" #x1D615)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL O" #x1D616)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL P" #x1D617)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL Q" #x1D618)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL R" #x1D619)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL S" #x1D61A)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL T" #x1D61B)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL U" #x1D61C)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL V" #x1D61D)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL W" #x1D61E)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL X" #x1D61F)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL Y" #x1D620)
    ("MATHEMATICAL SANS-SERIF ITALIC CAPITAL Z" #x1D621)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL A" #x1D622)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL B" #x1D623)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL C" #x1D624)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL D" #x1D625)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL E" #x1D626)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL F" #x1D627)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL G" #x1D628)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL H" #x1D629)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL I" #x1D62A)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL J" #x1D62B)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL K" #x1D62C)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL L" #x1D62D)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL M" #x1D62E)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL N" #x1D62F)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL O" #x1D630)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL P" #x1D631)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL Q" #x1D632)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL R" #x1D633)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL S" #x1D634)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL T" #x1D635)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL U" #x1D636)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL V" #x1D637)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL W" #x1D638)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL X" #x1D639)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL Y" #x1D63A)
    ("MATHEMATICAL SANS-SERIF ITALIC SMALL Z" #x1D63B)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL A" #x1D63C)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL B" #x1D63D)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL C" #x1D63E)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL D" #x1D63F)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL E" #x1D640)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL F" #x1D641)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL G" #x1D642)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL H" #x1D643)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL I" #x1D644)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL J" #x1D645)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL K" #x1D646)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL L" #x1D647)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL M" #x1D648)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL N" #x1D649)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL O" #x1D64A)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL P" #x1D64B)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Q" #x1D64C)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL R" #x1D64D)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL S" #x1D64E)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL T" #x1D64F)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL U" #x1D650)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL V" #x1D651)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL W" #x1D652)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL X" #x1D653)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Y" #x1D654)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Z" #x1D655)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL A" #x1D656)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL B" #x1D657)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL C" #x1D658)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL D" #x1D659)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL E" #x1D65A)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL F" #x1D65B)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL G" #x1D65C)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL H" #x1D65D)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL I" #x1D65E)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL J" #x1D65F)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL K" #x1D660)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL L" #x1D661)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL M" #x1D662)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL N" #x1D663)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL O" #x1D664)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL P" #x1D665)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Q" #x1D666)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL R" #x1D667)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL S" #x1D668)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL T" #x1D669)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL U" #x1D66A)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL V" #x1D66B)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL W" #x1D66C)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL X" #x1D66D)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Y" #x1D66E)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Z" #x1D66F)
    ("MATHEMATICAL MONOSPACE CAPITAL A" #x1D670)
    ("MATHEMATICAL MONOSPACE CAPITAL B" #x1D671)
    ("MATHEMATICAL MONOSPACE CAPITAL C" #x1D672)
    ("MATHEMATICAL MONOSPACE CAPITAL D" #x1D673)
    ("MATHEMATICAL MONOSPACE CAPITAL E" #x1D674)
    ("MATHEMATICAL MONOSPACE CAPITAL F" #x1D675)
    ("MATHEMATICAL MONOSPACE CAPITAL G" #x1D676)
    ("MATHEMATICAL MONOSPACE CAPITAL H" #x1D677)
    ("MATHEMATICAL MONOSPACE CAPITAL I" #x1D678)
    ("MATHEMATICAL MONOSPACE CAPITAL J" #x1D679)
    ("MATHEMATICAL MONOSPACE CAPITAL K" #x1D67A)
    ("MATHEMATICAL MONOSPACE CAPITAL L" #x1D67B)
    ("MATHEMATICAL MONOSPACE CAPITAL M" #x1D67C)
    ("MATHEMATICAL MONOSPACE CAPITAL N" #x1D67D)
    ("MATHEMATICAL MONOSPACE CAPITAL O" #x1D67E)
    ("MATHEMATICAL MONOSPACE CAPITAL P" #x1D67F)
    ("MATHEMATICAL MONOSPACE CAPITAL Q" #x1D680)
    ("MATHEMATICAL MONOSPACE CAPITAL R" #x1D681)
    ("MATHEMATICAL MONOSPACE CAPITAL S" #x1D682)
    ("MATHEMATICAL MONOSPACE CAPITAL T" #x1D683)
    ("MATHEMATICAL MONOSPACE CAPITAL U" #x1D684)
    ("MATHEMATICAL MONOSPACE CAPITAL V" #x1D685)
    ("MATHEMATICAL MONOSPACE CAPITAL W" #x1D686)
    ("MATHEMATICAL MONOSPACE CAPITAL X" #x1D687)
    ("MATHEMATICAL MONOSPACE CAPITAL Y" #x1D688)
    ("MATHEMATICAL MONOSPACE CAPITAL Z" #x1D689)
    ("MATHEMATICAL MONOSPACE SMALL A" #x1D68A)
    ("MATHEMATICAL MONOSPACE SMALL B" #x1D68B)
    ("MATHEMATICAL MONOSPACE SMALL C" #x1D68C)
    ("MATHEMATICAL MONOSPACE SMALL D" #x1D68D)
    ("MATHEMATICAL MONOSPACE SMALL E" #x1D68E)
    ("MATHEMATICAL MONOSPACE SMALL F" #x1D68F)
    ("MATHEMATICAL MONOSPACE SMALL G" #x1D690)
    ("MATHEMATICAL MONOSPACE SMALL H" #x1D691)
    ("MATHEMATICAL MONOSPACE SMALL I" #x1D692)
    ("MATHEMATICAL MONOSPACE SMALL J" #x1D693)
    ("MATHEMATICAL MONOSPACE SMALL K" #x1D694)
    ("MATHEMATICAL MONOSPACE SMALL L" #x1D695)
    ("MATHEMATICAL MONOSPACE SMALL M" #x1D696)
    ("MATHEMATICAL MONOSPACE SMALL N" #x1D697)
    ("MATHEMATICAL MONOSPACE SMALL O" #x1D698)
    ("MATHEMATICAL MONOSPACE SMALL P" #x1D699)
    ("MATHEMATICAL MONOSPACE SMALL Q" #x1D69A)
    ("MATHEMATICAL MONOSPACE SMALL R" #x1D69B)
    ("MATHEMATICAL MONOSPACE SMALL S" #x1D69C)
    ("MATHEMATICAL MONOSPACE SMALL T" #x1D69D)
    ("MATHEMATICAL MONOSPACE SMALL U" #x1D69E)
    ("MATHEMATICAL MONOSPACE SMALL V" #x1D69F)
    ("MATHEMATICAL MONOSPACE SMALL W" #x1D6A0)
    ("MATHEMATICAL MONOSPACE SMALL X" #x1D6A1)
    ("MATHEMATICAL MONOSPACE SMALL Y" #x1D6A2)
    ("MATHEMATICAL MONOSPACE SMALL Z" #x1D6A3)
    ("MATHEMATICAL BOLD CAPITAL ALPHA" #x1D6A8)
    ("MATHEMATICAL BOLD CAPITAL BETA" #x1D6A9)
    ("MATHEMATICAL BOLD CAPITAL GAMMA" #x1D6AA)
    ("MATHEMATICAL BOLD CAPITAL DELTA" #x1D6AB)
    ("MATHEMATICAL BOLD CAPITAL EPSILON" #x1D6AC)
    ("MATHEMATICAL BOLD CAPITAL ZETA" #x1D6AD)
    ("MATHEMATICAL BOLD CAPITAL ETA" #x1D6AE)
    ("MATHEMATICAL BOLD CAPITAL THETA" #x1D6AF)
    ("MATHEMATICAL BOLD CAPITAL IOTA" #x1D6B0)
    ("MATHEMATICAL BOLD CAPITAL KAPPA" #x1D6B1)
    ("MATHEMATICAL BOLD CAPITAL LAMDA" #x1D6B2)
    ("MATHEMATICAL BOLD CAPITAL MU" #x1D6B3)
    ("MATHEMATICAL BOLD CAPITAL NU" #x1D6B4)
    ("MATHEMATICAL BOLD CAPITAL XI" #x1D6B5)
    ("MATHEMATICAL BOLD CAPITAL OMICRON" #x1D6B6)
    ("MATHEMATICAL BOLD CAPITAL PI" #x1D6B7)
    ("MATHEMATICAL BOLD CAPITAL RHO" #x1D6B8)
    ("MATHEMATICAL BOLD CAPITAL THETA SYMBOL" #x1D6B9)
    ("MATHEMATICAL BOLD CAPITAL SIGMA" #x1D6BA)
    ("MATHEMATICAL BOLD CAPITAL TAU" #x1D6BB)
    ("MATHEMATICAL BOLD CAPITAL UPSILON" #x1D6BC)
    ("MATHEMATICAL BOLD CAPITAL PHI" #x1D6BD)
    ("MATHEMATICAL BOLD CAPITAL CHI" #x1D6BE)
    ("MATHEMATICAL BOLD CAPITAL PSI" #x1D6BF)
    ("MATHEMATICAL BOLD CAPITAL OMEGA" #x1D6C0)
    ("MATHEMATICAL BOLD NABLA" #x1D6C1)
    ("MATHEMATICAL BOLD SMALL ALPHA" #x1D6C2)
    ("MATHEMATICAL BOLD SMALL BETA" #x1D6C3)
    ("MATHEMATICAL BOLD SMALL GAMMA" #x1D6C4)
    ("MATHEMATICAL BOLD SMALL DELTA" #x1D6C5)
    ("MATHEMATICAL BOLD SMALL EPSILON" #x1D6C6)
    ("MATHEMATICAL BOLD SMALL ZETA" #x1D6C7)
    ("MATHEMATICAL BOLD SMALL ETA" #x1D6C8)
    ("MATHEMATICAL BOLD SMALL THETA" #x1D6C9)
    ("MATHEMATICAL BOLD SMALL IOTA" #x1D6CA)
    ("MATHEMATICAL BOLD SMALL KAPPA" #x1D6CB)
    ("MATHEMATICAL BOLD SMALL LAMDA" #x1D6CC)
    ("MATHEMATICAL BOLD SMALL MU" #x1D6CD)
    ("MATHEMATICAL BOLD SMALL NU" #x1D6CE)
    ("MATHEMATICAL BOLD SMALL XI" #x1D6CF)
    ("MATHEMATICAL BOLD SMALL OMICRON" #x1D6D0)
    ("MATHEMATICAL BOLD SMALL PI" #x1D6D1)
    ("MATHEMATICAL BOLD SMALL RHO" #x1D6D2)
    ("MATHEMATICAL BOLD SMALL FINAL SIGMA" #x1D6D3)
    ("MATHEMATICAL BOLD SMALL SIGMA" #x1D6D4)
    ("MATHEMATICAL BOLD SMALL TAU" #x1D6D5)
    ("MATHEMATICAL BOLD SMALL UPSILON" #x1D6D6)
    ("MATHEMATICAL BOLD SMALL PHI" #x1D6D7)
    ("MATHEMATICAL BOLD SMALL CHI" #x1D6D8)
    ("MATHEMATICAL BOLD SMALL PSI" #x1D6D9)
    ("MATHEMATICAL BOLD SMALL OMEGA" #x1D6DA)
    ("MATHEMATICAL BOLD PARTIAL DIFFERENTIAL" #x1D6DB)
    ("MATHEMATICAL BOLD EPSILON SYMBOL" #x1D6DC)
    ("MATHEMATICAL BOLD THETA SYMBOL" #x1D6DD)
    ("MATHEMATICAL BOLD KAPPA SYMBOL" #x1D6DE)
    ("MATHEMATICAL BOLD PHI SYMBOL" #x1D6DF)
    ("MATHEMATICAL BOLD RHO SYMBOL" #x1D6E0)
    ("MATHEMATICAL BOLD PI SYMBOL" #x1D6E1)
    ("MATHEMATICAL ITALIC CAPITAL ALPHA" #x1D6E2)
    ("MATHEMATICAL ITALIC CAPITAL BETA" #x1D6E3)
    ("MATHEMATICAL ITALIC CAPITAL GAMMA" #x1D6E4)
    ("MATHEMATICAL ITALIC CAPITAL DELTA" #x1D6E5)
    ("MATHEMATICAL ITALIC CAPITAL EPSILON" #x1D6E6)
    ("MATHEMATICAL ITALIC CAPITAL ZETA" #x1D6E7)
    ("MATHEMATICAL ITALIC CAPITAL ETA" #x1D6E8)
    ("MATHEMATICAL ITALIC CAPITAL THETA" #x1D6E9)
    ("MATHEMATICAL ITALIC CAPITAL IOTA" #x1D6EA)
    ("MATHEMATICAL ITALIC CAPITAL KAPPA" #x1D6EB)
    ("MATHEMATICAL ITALIC CAPITAL LAMDA" #x1D6EC)
    ("MATHEMATICAL ITALIC CAPITAL MU" #x1D6ED)
    ("MATHEMATICAL ITALIC CAPITAL NU" #x1D6EE)
    ("MATHEMATICAL ITALIC CAPITAL XI" #x1D6EF)
    ("MATHEMATICAL ITALIC CAPITAL OMICRON" #x1D6F0)
    ("MATHEMATICAL ITALIC CAPITAL PI" #x1D6F1)
    ("MATHEMATICAL ITALIC CAPITAL RHO" #x1D6F2)
    ("MATHEMATICAL ITALIC CAPITAL THETA SYMBOL" #x1D6F3)
    ("MATHEMATICAL ITALIC CAPITAL SIGMA" #x1D6F4)
    ("MATHEMATICAL ITALIC CAPITAL TAU" #x1D6F5)
    ("MATHEMATICAL ITALIC CAPITAL UPSILON" #x1D6F6)
    ("MATHEMATICAL ITALIC CAPITAL PHI" #x1D6F7)
    ("MATHEMATICAL ITALIC CAPITAL CHI" #x1D6F8)
    ("MATHEMATICAL ITALIC CAPITAL PSI" #x1D6F9)
    ("MATHEMATICAL ITALIC CAPITAL OMEGA" #x1D6FA)
    ("MATHEMATICAL ITALIC NABLA" #x1D6FB)
    ("MATHEMATICAL ITALIC SMALL ALPHA" #x1D6FC)
    ("MATHEMATICAL ITALIC SMALL BETA" #x1D6FD)
    ("MATHEMATICAL ITALIC SMALL GAMMA" #x1D6FE)
    ("MATHEMATICAL ITALIC SMALL DELTA" #x1D6FF)
    ("MATHEMATICAL ITALIC SMALL EPSILON" #x1D700)
    ("MATHEMATICAL ITALIC SMALL ZETA" #x1D701)
    ("MATHEMATICAL ITALIC SMALL ETA" #x1D702)
    ("MATHEMATICAL ITALIC SMALL THETA" #x1D703)
    ("MATHEMATICAL ITALIC SMALL IOTA" #x1D704)
    ("MATHEMATICAL ITALIC SMALL KAPPA" #x1D705)
    ("MATHEMATICAL ITALIC SMALL LAMDA" #x1D706)
    ("MATHEMATICAL ITALIC SMALL MU" #x1D707)
    ("MATHEMATICAL ITALIC SMALL NU" #x1D708)
    ("MATHEMATICAL ITALIC SMALL XI" #x1D709)
    ("MATHEMATICAL ITALIC SMALL OMICRON" #x1D70A)
    ("MATHEMATICAL ITALIC SMALL PI" #x1D70B)
    ("MATHEMATICAL ITALIC SMALL RHO" #x1D70C)
    ("MATHEMATICAL ITALIC SMALL FINAL SIGMA" #x1D70D)
    ("MATHEMATICAL ITALIC SMALL SIGMA" #x1D70E)
    ("MATHEMATICAL ITALIC SMALL TAU" #x1D70F)
    ("MATHEMATICAL ITALIC SMALL UPSILON" #x1D710)
    ("MATHEMATICAL ITALIC SMALL PHI" #x1D711)
    ("MATHEMATICAL ITALIC SMALL CHI" #x1D712)
    ("MATHEMATICAL ITALIC SMALL PSI" #x1D713)
    ("MATHEMATICAL ITALIC SMALL OMEGA" #x1D714)
    ("MATHEMATICAL ITALIC PARTIAL DIFFERENTIAL" #x1D715)
    ("MATHEMATICAL ITALIC EPSILON SYMBOL" #x1D716)
    ("MATHEMATICAL ITALIC THETA SYMBOL" #x1D717)
    ("MATHEMATICAL ITALIC KAPPA SYMBOL" #x1D718)
    ("MATHEMATICAL ITALIC PHI SYMBOL" #x1D719)
    ("MATHEMATICAL ITALIC RHO SYMBOL" #x1D71A)
    ("MATHEMATICAL ITALIC PI SYMBOL" #x1D71B)
    ("MATHEMATICAL BOLD ITALIC CAPITAL ALPHA" #x1D71C)
    ("MATHEMATICAL BOLD ITALIC CAPITAL BETA" #x1D71D)
    ("MATHEMATICAL BOLD ITALIC CAPITAL GAMMA" #x1D71E)
    ("MATHEMATICAL BOLD ITALIC CAPITAL DELTA" #x1D71F)
    ("MATHEMATICAL BOLD ITALIC CAPITAL EPSILON" #x1D720)
    ("MATHEMATICAL BOLD ITALIC CAPITAL ZETA" #x1D721)
    ("MATHEMATICAL BOLD ITALIC CAPITAL ETA" #x1D722)
    ("MATHEMATICAL BOLD ITALIC CAPITAL THETA" #x1D723)
    ("MATHEMATICAL BOLD ITALIC CAPITAL IOTA" #x1D724)
    ("MATHEMATICAL BOLD ITALIC CAPITAL KAPPA" #x1D725)
    ("MATHEMATICAL BOLD ITALIC CAPITAL LAMDA" #x1D726)
    ("MATHEMATICAL BOLD ITALIC CAPITAL MU" #x1D727)
    ("MATHEMATICAL BOLD ITALIC CAPITAL NU" #x1D728)
    ("MATHEMATICAL BOLD ITALIC CAPITAL XI" #x1D729)
    ("MATHEMATICAL BOLD ITALIC CAPITAL OMICRON" #x1D72A)
    ("MATHEMATICAL BOLD ITALIC CAPITAL PI" #x1D72B)
    ("MATHEMATICAL BOLD ITALIC CAPITAL RHO" #x1D72C)
    ("MATHEMATICAL BOLD ITALIC CAPITAL THETA SYMBOL" #x1D72D)
    ("MATHEMATICAL BOLD ITALIC CAPITAL SIGMA" #x1D72E)
    ("MATHEMATICAL BOLD ITALIC CAPITAL TAU" #x1D72F)
    ("MATHEMATICAL BOLD ITALIC CAPITAL UPSILON" #x1D730)
    ("MATHEMATICAL BOLD ITALIC CAPITAL PHI" #x1D731)
    ("MATHEMATICAL BOLD ITALIC CAPITAL CHI" #x1D732)
    ("MATHEMATICAL BOLD ITALIC CAPITAL PSI" #x1D733)
    ("MATHEMATICAL BOLD ITALIC CAPITAL OMEGA" #x1D734)
    ("MATHEMATICAL BOLD ITALIC NABLA" #x1D735)
    ("MATHEMATICAL BOLD ITALIC SMALL ALPHA" #x1D736)
    ("MATHEMATICAL BOLD ITALIC SMALL BETA" #x1D737)
    ("MATHEMATICAL BOLD ITALIC SMALL GAMMA" #x1D738)
    ("MATHEMATICAL BOLD ITALIC SMALL DELTA" #x1D739)
    ("MATHEMATICAL BOLD ITALIC SMALL EPSILON" #x1D73A)
    ("MATHEMATICAL BOLD ITALIC SMALL ZETA" #x1D73B)
    ("MATHEMATICAL BOLD ITALIC SMALL ETA" #x1D73C)
    ("MATHEMATICAL BOLD ITALIC SMALL THETA" #x1D73D)
    ("MATHEMATICAL BOLD ITALIC SMALL IOTA" #x1D73E)
    ("MATHEMATICAL BOLD ITALIC SMALL KAPPA" #x1D73F)
    ("MATHEMATICAL BOLD ITALIC SMALL LAMDA" #x1D740)
    ("MATHEMATICAL BOLD ITALIC SMALL MU" #x1D741)
    ("MATHEMATICAL BOLD ITALIC SMALL NU" #x1D742)
    ("MATHEMATICAL BOLD ITALIC SMALL XI" #x1D743)
    ("MATHEMATICAL BOLD ITALIC SMALL OMICRON" #x1D744)
    ("MATHEMATICAL BOLD ITALIC SMALL PI" #x1D745)
    ("MATHEMATICAL BOLD ITALIC SMALL RHO" #x1D746)
    ("MATHEMATICAL BOLD ITALIC SMALL FINAL SIGMA" #x1D747)
    ("MATHEMATICAL BOLD ITALIC SMALL SIGMA" #x1D748)
    ("MATHEMATICAL BOLD ITALIC SMALL TAU" #x1D749)
    ("MATHEMATICAL BOLD ITALIC SMALL UPSILON" #x1D74A)
    ("MATHEMATICAL BOLD ITALIC SMALL PHI" #x1D74B)
    ("MATHEMATICAL BOLD ITALIC SMALL CHI" #x1D74C)
    ("MATHEMATICAL BOLD ITALIC SMALL PSI" #x1D74D)
    ("MATHEMATICAL BOLD ITALIC SMALL OMEGA" #x1D74E)
    ("MATHEMATICAL BOLD ITALIC PARTIAL DIFFERENTIAL" #x1D74F)
    ("MATHEMATICAL BOLD ITALIC EPSILON SYMBOL" #x1D750)
    ("MATHEMATICAL BOLD ITALIC THETA SYMBOL" #x1D751)
    ("MATHEMATICAL BOLD ITALIC KAPPA SYMBOL" #x1D752)
    ("MATHEMATICAL BOLD ITALIC PHI SYMBOL" #x1D753)
    ("MATHEMATICAL BOLD ITALIC RHO SYMBOL" #x1D754)
    ("MATHEMATICAL BOLD ITALIC PI SYMBOL" #x1D755)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL ALPHA" #x1D756)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL BETA" #x1D757)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL GAMMA" #x1D758)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL DELTA" #x1D759)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL EPSILON" #x1D75A)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL ZETA" #x1D75B)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL ETA" #x1D75C)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL THETA" #x1D75D)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL IOTA" #x1D75E)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL KAPPA" #x1D75F)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL LAMDA" #x1D760)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL MU" #x1D761)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL NU" #x1D762)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL XI" #x1D763)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL OMICRON" #x1D764)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL PI" #x1D765)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL RHO" #x1D766)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL THETA SYMBOL" #x1D767)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL SIGMA" #x1D768)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL TAU" #x1D769)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL UPSILON" #x1D76A)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL PHI" #x1D76B)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL CHI" #x1D76C)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL PSI" #x1D76D)
    ("MATHEMATICAL SANS-SERIF BOLD CAPITAL OMEGA" #x1D76E)
    ("MATHEMATICAL SANS-SERIF BOLD NABLA" #x1D76F)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL ALPHA" #x1D770)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL BETA" #x1D771)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL GAMMA" #x1D772)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL DELTA" #x1D773)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL EPSILON" #x1D774)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL ZETA" #x1D775)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL ETA" #x1D776)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL THETA" #x1D777)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL IOTA" #x1D778)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL KAPPA" #x1D779)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL LAMDA" #x1D77A)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL MU" #x1D77B)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL NU" #x1D77C)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL XI" #x1D77D)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL OMICRON" #x1D77E)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL PI" #x1D77F)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL RHO" #x1D780)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL FINAL SIGMA" #x1D781)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL SIGMA" #x1D782)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL TAU" #x1D783)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL UPSILON" #x1D784)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL PHI" #x1D785)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL CHI" #x1D786)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL PSI" #x1D787)
    ("MATHEMATICAL SANS-SERIF BOLD SMALL OMEGA" #x1D788)
    ("MATHEMATICAL SANS-SERIF BOLD PARTIAL DIFFERENTIAL" #x1D789)
    ("MATHEMATICAL SANS-SERIF BOLD EPSILON SYMBOL" #x1D78A)
    ("MATHEMATICAL SANS-SERIF BOLD THETA SYMBOL" #x1D78B)
    ("MATHEMATICAL SANS-SERIF BOLD KAPPA SYMBOL" #x1D78C)
    ("MATHEMATICAL SANS-SERIF BOLD PHI SYMBOL" #x1D78D)
    ("MATHEMATICAL SANS-SERIF BOLD RHO SYMBOL" #x1D78E)
    ("MATHEMATICAL SANS-SERIF BOLD PI SYMBOL" #x1D78F)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ALPHA" #x1D790)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL BETA" #x1D791)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL GAMMA" #x1D792)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL DELTA" #x1D793)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL EPSILON" #x1D794)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ZETA" #x1D795)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ETA" #x1D796)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL THETA" #x1D797)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL IOTA" #x1D798)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL KAPPA" #x1D799)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL LAMDA" #x1D79A)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL MU" #x1D79B)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL NU" #x1D79C)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL XI" #x1D79D)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL OMICRON" #x1D79E)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PI" #x1D79F)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL RHO" #x1D7A0)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL THETA SYMBOL" #x1D7A1)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL SIGMA" #x1D7A2)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL TAU" #x1D7A3)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL UPSILON" #x1D7A4)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PHI" #x1D7A5)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL CHI" #x1D7A6)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PSI" #x1D7A7)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL OMEGA" #x1D7A8)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC NABLA" #x1D7A9)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ALPHA" #x1D7AA)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL BETA" #x1D7AB)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL GAMMA" #x1D7AC)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL DELTA" #x1D7AD)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL EPSILON" #x1D7AE)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ZETA" #x1D7AF)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ETA" #x1D7B0)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL THETA" #x1D7B1)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL IOTA" #x1D7B2)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL KAPPA" #x1D7B3)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL LAMDA" #x1D7B4)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL MU" #x1D7B5)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL NU" #x1D7B6)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL XI" #x1D7B7)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL OMICRON" #x1D7B8)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PI" #x1D7B9)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL RHO" #x1D7BA)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL FINAL SIGMA" #x1D7BB)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL SIGMA" #x1D7BC)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL TAU" #x1D7BD)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL UPSILON" #x1D7BE)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PHI" #x1D7BF)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL CHI" #x1D7C0)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PSI" #x1D7C1)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL OMEGA" #x1D7C2)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC PARTIAL DIFFERENTIAL" #x1D7C3)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC EPSILON SYMBOL" #x1D7C4)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC THETA SYMBOL" #x1D7C5)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC KAPPA SYMBOL" #x1D7C6)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC PHI SYMBOL" #x1D7C7)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC RHO SYMBOL" #x1D7C8)
    ("MATHEMATICAL SANS-SERIF BOLD ITALIC PI SYMBOL" #x1D7C9)
    ("MATHEMATICAL BOLD DIGIT ZERO" #x1D7CE)
    ("MATHEMATICAL BOLD DIGIT ONE" #x1D7CF)
    ("MATHEMATICAL BOLD DIGIT TWO" #x1D7D0)
    ("MATHEMATICAL BOLD DIGIT THREE" #x1D7D1)
    ("MATHEMATICAL BOLD DIGIT FOUR" #x1D7D2)
    ("MATHEMATICAL BOLD DIGIT FIVE" #x1D7D3)
    ("MATHEMATICAL BOLD DIGIT SIX" #x1D7D4)
    ("MATHEMATICAL BOLD DIGIT SEVEN" #x1D7D5)
    ("MATHEMATICAL BOLD DIGIT EIGHT" #x1D7D6)
    ("MATHEMATICAL BOLD DIGIT NINE" #x1D7D7)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT ZERO" #x1D7D8)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT ONE" #x1D7D9)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT TWO" #x1D7DA)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT THREE" #x1D7DB)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT FOUR" #x1D7DC)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT FIVE" #x1D7DD)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT SIX" #x1D7DE)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT SEVEN" #x1D7DF)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT EIGHT" #x1D7E0)
    ("MATHEMATICAL DOUBLE-STRUCK DIGIT NINE" #x1D7E1)
    ("MATHEMATICAL SANS-SERIF DIGIT ZERO" #x1D7E2)
    ("MATHEMATICAL SANS-SERIF DIGIT ONE" #x1D7E3)
    ("MATHEMATICAL SANS-SERIF DIGIT TWO" #x1D7E4)
    ("MATHEMATICAL SANS-SERIF DIGIT THREE" #x1D7E5)
    ("MATHEMATICAL SANS-SERIF DIGIT FOUR" #x1D7E6)
    ("MATHEMATICAL SANS-SERIF DIGIT FIVE" #x1D7E7)
    ("MATHEMATICAL SANS-SERIF DIGIT SIX" #x1D7E8)
    ("MATHEMATICAL SANS-SERIF DIGIT SEVEN" #x1D7E9)
    ("MATHEMATICAL SANS-SERIF DIGIT EIGHT" #x1D7EA)
    ("MATHEMATICAL SANS-SERIF DIGIT NINE" #x1D7EB)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT ZERO" #x1D7EC)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT ONE" #x1D7ED)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT TWO" #x1D7EE)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT THREE" #x1D7EF)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT FOUR" #x1D7F0)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT FIVE" #x1D7F1)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT SIX" #x1D7F2)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT SEVEN" #x1D7F3)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT EIGHT" #x1D7F4)
    ("MATHEMATICAL SANS-SERIF BOLD DIGIT NINE" #x1D7F5)
    ("MATHEMATICAL MONOSPACE DIGIT ZERO" #x1D7F6)
    ("MATHEMATICAL MONOSPACE DIGIT ONE" #x1D7F7)
    ("MATHEMATICAL MONOSPACE DIGIT TWO" #x1D7F8)
    ("MATHEMATICAL MONOSPACE DIGIT THREE" #x1D7F9)
    ("MATHEMATICAL MONOSPACE DIGIT FOUR" #x1D7FA)
    ("MATHEMATICAL MONOSPACE DIGIT FIVE" #x1D7FB)
    ("MATHEMATICAL MONOSPACE DIGIT SIX" #x1D7FC)
    ("MATHEMATICAL MONOSPACE DIGIT SEVEN" #x1D7FD)
    ("MATHEMATICAL MONOSPACE DIGIT EIGHT" #x1D7FE)
    ("MATHEMATICAL MONOSPACE DIGIT NINE" #x1D7FF)
    ))

;; arch-tag: af730f58-d05e-4cd5-8a94-f914444a5351
