(nxml-define-char-name-set 'block-elements
  '(("UPPER HALF BLOCK" #x2580)
    ("LOWER ONE EIGHTH BLOCK" #x2581)
    ("LOWER ONE QUARTER BLOCK" #x2582)
    ("LOWER THREE EIGHTHS BLOCK" #x2583)
    ("LOWER HALF BLOCK" #x2584)
    ("LOWER FIVE EIGHTHS BLOCK" #x2585)
    ("LOWER THREE QUARTERS BLOCK" #x2586)
    ("LOWER SEVEN EIGHTHS BLOCK" #x2587)
    ("FULL BLOCK" #x2588)
    ("LEFT SEVEN EIGHTHS BLOCK" #x2589)
    ("LEFT THREE QUARTERS BLOCK" #x258A)
    ("LEFT FIVE EIGHTHS BLOCK" #x258B)
    ("LEFT HALF BLOCK" #x258C)
    ("LEFT THREE EIGHTHS BLOCK" #x258D)
    ("LEFT ONE QUARTER BLOCK" #x258E)
    ("LEFT ONE EIGHTH BLOCK" #x258F)
    ("RIGHT HALF BLOCK" #x2590)
    ("LIGHT SHADE" #x2591)
    ("MEDIUM SHADE" #x2592)
    ("DARK SHADE" #x2593)
    ("UPPER ONE EIGHTH BLOCK" #x2594)
    ("RIGHT ONE EIGHTH BLOCK" #x2595)
    ("QUADRANT LOWER LEFT" #x2596)
    ("QUADRANT LOWER RIGHT" #x2597)
    ("QUADRANT UPPER LEFT" #x2598)
    ("QUADRANT UPPER LEFT AND LOWER LEFT AND LOWER RIGHT" #x2599)
    ("QUADRANT UPPER LEFT AND LOWER RIGHT" #x259A)
    ("QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER LEFT" #x259B)
    ("QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER RIGHT" #x259C)
    ("QUADRANT UPPER RIGHT" #x259D)
    ("QUADRANT UPPER RIGHT AND LOWER LEFT" #x259E)
    ("QUADRANT UPPER RIGHT AND LOWER LEFT AND LOWER RIGHT" #x259F)
    ))

;; arch-tag: 51c971a7-cd90-42a5-bf18-dd9683bcab41
