(nxml-define-char-name-set 'greek-extended
  '(("GREEK SMALL LETTER ALPHA WITH PSILI" #x1F00)
    ("GREEK SMALL LETTER ALPHA WITH DASIA" #x1F01)
    ("GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA" #x1F02)
    ("GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA" #x1F03)
    ("GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA" #x1F04)
    ("GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA" #x1F05)
    ("GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI" #x1F06)
    ("GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI" #x1F07)
    ("GREEK CAPITAL LETTER ALPHA WITH PSILI" #x1F08)
    ("GREEK CAPITAL LETTER ALPHA WITH DASIA" #x1F09)
    ("GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA" #x1F0A)
    ("GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA" #x1F0B)
    ("GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA" #x1F0C)
    ("GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA" #x1F0D)
    ("GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI" #x1F0E)
    ("GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI" #x1F0F)
    ("GREEK SMALL LETTER EPSILON WITH PSILI" #x1F10)
    ("GREEK SMALL LETTER EPSILON WITH DASIA" #x1F11)
    ("GREEK SMALL LETTER EPSILON WITH PSILI AND VARIA" #x1F12)
    ("GREEK SMALL LETTER EPSILON WITH DASIA AND VARIA" #x1F13)
    ("GREEK SMALL LETTER EPSILON WITH PSILI AND OXIA" #x1F14)
    ("GREEK SMALL LETTER EPSILON WITH DASIA AND OXIA" #x1F15)
    ("GREEK CAPITAL LETTER EPSILON WITH PSILI" #x1F18)
    ("GREEK CAPITAL LETTER EPSILON WITH DASIA" #x1F19)
    ("GREEK CAPITAL LETTER EPSILON WITH PSILI AND VARIA" #x1F1A)
    ("GREEK CAPITAL LETTER EPSILON WITH DASIA AND VARIA" #x1F1B)
    ("GREEK CAPITAL LETTER EPSILON WITH PSILI AND OXIA" #x1F1C)
    ("GREEK CAPITAL LETTER EPSILON WITH DASIA AND OXIA" #x1F1D)
    ("GREEK SMALL LETTER ETA WITH PSILI" #x1F20)
    ("GREEK SMALL LETTER ETA WITH DASIA" #x1F21)
    ("GREEK SMALL LETTER ETA WITH PSILI AND VARIA" #x1F22)
    ("GREEK SMALL LETTER ETA WITH DASIA AND VARIA" #x1F23)
    ("GREEK SMALL LETTER ETA WITH PSILI AND OXIA" #x1F24)
    ("GREEK SMALL LETTER ETA WITH DASIA AND OXIA" #x1F25)
    ("GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI" #x1F26)
    ("GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI" #x1F27)
    ("GREEK CAPITAL LETTER ETA WITH PSILI" #x1F28)
    ("GREEK CAPITAL LETTER ETA WITH DASIA" #x1F29)
    ("GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA" #x1F2A)
    ("GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA" #x1F2B)
    ("GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA" #x1F2C)
    ("GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA" #x1F2D)
    ("GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI" #x1F2E)
    ("GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI" #x1F2F)
    ("GREEK SMALL LETTER IOTA WITH PSILI" #x1F30)
    ("GREEK SMALL LETTER IOTA WITH DASIA" #x1F31)
    ("GREEK SMALL LETTER IOTA WITH PSILI AND VARIA" #x1F32)
    ("GREEK SMALL LETTER IOTA WITH DASIA AND VARIA" #x1F33)
    ("GREEK SMALL LETTER IOTA WITH PSILI AND OXIA" #x1F34)
    ("GREEK SMALL LETTER IOTA WITH DASIA AND OXIA" #x1F35)
    ("GREEK SMALL LETTER IOTA WITH PSILI AND PERISPOMENI" #x1F36)
    ("GREEK SMALL LETTER IOTA WITH DASIA AND PERISPOMENI" #x1F37)
    ("GREEK CAPITAL LETTER IOTA WITH PSILI" #x1F38)
    ("GREEK CAPITAL LETTER IOTA WITH DASIA" #x1F39)
    ("GREEK CAPITAL LETTER IOTA WITH PSILI AND VARIA" #x1F3A)
    ("GREEK CAPITAL LETTER IOTA WITH DASIA AND VARIA" #x1F3B)
    ("GREEK CAPITAL LETTER IOTA WITH PSILI AND OXIA" #x1F3C)
    ("GREEK CAPITAL LETTER IOTA WITH DASIA AND OXIA" #x1F3D)
    ("GREEK CAPITAL LETTER IOTA WITH PSILI AND PERISPOMENI" #x1F3E)
    ("GREEK CAPITAL LETTER IOTA WITH DASIA AND PERISPOMENI" #x1F3F)
    ("GREEK SMALL LETTER OMICRON WITH PSILI" #x1F40)
    ("GREEK SMALL LETTER OMICRON WITH DASIA" #x1F41)
    ("GREEK SMALL LETTER OMICRON WITH PSILI AND VARIA" #x1F42)
    ("GREEK SMALL LETTER OMICRON WITH DASIA AND VARIA" #x1F43)
    ("GREEK SMALL LETTER OMICRON WITH PSILI AND OXIA" #x1F44)
    ("GREEK SMALL LETTER OMICRON WITH DASIA AND OXIA" #x1F45)
    ("GREEK CAPITAL LETTER OMICRON WITH PSILI" #x1F48)
    ("GREEK CAPITAL LETTER OMICRON WITH DASIA" #x1F49)
    ("GREEK CAPITAL LETTER OMICRON WITH PSILI AND VARIA" #x1F4A)
    ("GREEK CAPITAL LETTER OMICRON WITH DASIA AND VARIA" #x1F4B)
    ("GREEK CAPITAL LETTER OMICRON WITH PSILI AND OXIA" #x1F4C)
    ("GREEK CAPITAL LETTER OMICRON WITH DASIA AND OXIA" #x1F4D)
    ("GREEK SMALL LETTER UPSILON WITH PSILI" #x1F50)
    ("GREEK SMALL LETTER UPSILON WITH DASIA" #x1F51)
    ("GREEK SMALL LETTER UPSILON WITH PSILI AND VARIA" #x1F52)
    ("GREEK SMALL LETTER UPSILON WITH DASIA AND VARIA" #x1F53)
    ("GREEK SMALL LETTER UPSILON WITH PSILI AND OXIA" #x1F54)
    ("GREEK SMALL LETTER UPSILON WITH DASIA AND OXIA" #x1F55)
    ("GREEK SMALL LETTER UPSILON WITH PSILI AND PERISPOMENI" #x1F56)
    ("GREEK SMALL LETTER UPSILON WITH DASIA AND PERISPOMENI" #x1F57)
    ("GREEK CAPITAL LETTER UPSILON WITH DASIA" #x1F59)
    ("GREEK CAPITAL LETTER UPSILON WITH DASIA AND VARIA" #x1F5B)
    ("GREEK CAPITAL LETTER UPSILON WITH DASIA AND OXIA" #x1F5D)
    ("GREEK CAPITAL LETTER UPSILON WITH DASIA AND PERISPOMENI" #x1F5F)
    ("GREEK SMALL LETTER OMEGA WITH PSILI" #x1F60)
    ("GREEK SMALL LETTER OMEGA WITH DASIA" #x1F61)
    ("GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA" #x1F62)
    ("GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA" #x1F63)
    ("GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA" #x1F64)
    ("GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA" #x1F65)
    ("GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI" #x1F66)
    ("GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI" #x1F67)
    ("GREEK CAPITAL LETTER OMEGA WITH PSILI" #x1F68)
    ("GREEK CAPITAL LETTER OMEGA WITH DASIA" #x1F69)
    ("GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA" #x1F6A)
    ("GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA" #x1F6B)
    ("GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA" #x1F6C)
    ("GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA" #x1F6D)
    ("GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI" #x1F6E)
    ("GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI" #x1F6F)
    ("GREEK SMALL LETTER ALPHA WITH VARIA" #x1F70)
    ("GREEK SMALL LETTER ALPHA WITH OXIA" #x1F71)
    ("GREEK SMALL LETTER EPSILON WITH VARIA" #x1F72)
    ("GREEK SMALL LETTER EPSILON WITH OXIA" #x1F73)
    ("GREEK SMALL LETTER ETA WITH VARIA" #x1F74)
    ("GREEK SMALL LETTER ETA WITH OXIA" #x1F75)
    ("GREEK SMALL LETTER IOTA WITH VARIA" #x1F76)
    ("GREEK SMALL LETTER IOTA WITH OXIA" #x1F77)
    ("GREEK SMALL LETTER OMICRON WITH VARIA" #x1F78)
    ("GREEK SMALL LETTER OMICRON WITH OXIA" #x1F79)
    ("GREEK SMALL LETTER UPSILON WITH VARIA" #x1F7A)
    ("GREEK SMALL LETTER UPSILON WITH OXIA" #x1F7B)
    ("GREEK SMALL LETTER OMEGA WITH VARIA" #x1F7C)
    ("GREEK SMALL LETTER OMEGA WITH OXIA" #x1F7D)
    ("GREEK SMALL LETTER ALPHA WITH PSILI AND YPOGEGRAMMENI" #x1F80)
    ("GREEK SMALL LETTER ALPHA WITH DASIA AND YPOGEGRAMMENI" #x1F81)
    ("GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA AND YPOGEGRAMMENI" #x1F82)
    ("GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA AND YPOGEGRAMMENI" #x1F83)
    ("GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA AND YPOGEGRAMMENI" #x1F84)
    ("GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA AND YPOGEGRAMMENI" #x1F85)
    ("GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI" #x1F86)
    ("GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI" #x1F87)
    ("GREEK CAPITAL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI" #x1F88)
    ("GREEK CAPITAL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI" #x1F89)
    ("GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA AND PROSGEGRAMMENI" #x1F8A)
    ("GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA AND PROSGEGRAMMENI" #x1F8B)
    ("GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA AND PROSGEGRAMMENI" #x1F8C)
    ("GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA AND PROSGEGRAMMENI" #x1F8D)
    ("GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI" #x1F8E)
    ("GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI" #x1F8F)
    ("GREEK SMALL LETTER ETA WITH PSILI AND YPOGEGRAMMENI" #x1F90)
    ("GREEK SMALL LETTER ETA WITH DASIA AND YPOGEGRAMMENI" #x1F91)
    ("GREEK SMALL LETTER ETA WITH PSILI AND VARIA AND YPOGEGRAMMENI" #x1F92)
    ("GREEK SMALL LETTER ETA WITH DASIA AND VARIA AND YPOGEGRAMMENI" #x1F93)
    ("GREEK SMALL LETTER ETA WITH PSILI AND OXIA AND YPOGEGRAMMENI" #x1F94)
    ("GREEK SMALL LETTER ETA WITH DASIA AND OXIA AND YPOGEGRAMMENI" #x1F95)
    ("GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI" #x1F96)
    ("GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI" #x1F97)
    ("GREEK CAPITAL LETTER ETA WITH PSILI AND PROSGEGRAMMENI" #x1F98)
    ("GREEK CAPITAL LETTER ETA WITH DASIA AND PROSGEGRAMMENI" #x1F99)
    ("GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA AND PROSGEGRAMMENI" #x1F9A)
    ("GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA AND PROSGEGRAMMENI" #x1F9B)
    ("GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA AND PROSGEGRAMMENI" #x1F9C)
    ("GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA AND PROSGEGRAMMENI" #x1F9D)
    ("GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI" #x1F9E)
    ("GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI" #x1F9F)
    ("GREEK SMALL LETTER OMEGA WITH PSILI AND YPOGEGRAMMENI" #x1FA0)
    ("GREEK SMALL LETTER OMEGA WITH DASIA AND YPOGEGRAMMENI" #x1FA1)
    ("GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA AND YPOGEGRAMMENI" #x1FA2)
    ("GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA AND YPOGEGRAMMENI" #x1FA3)
    ("GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA AND YPOGEGRAMMENI" #x1FA4)
    ("GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA AND YPOGEGRAMMENI" #x1FA5)
    ("GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI" #x1FA6)
    ("GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI" #x1FA7)
    ("GREEK CAPITAL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI" #x1FA8)
    ("GREEK CAPITAL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI" #x1FA9)
    ("GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA AND PROSGEGRAMMENI" #x1FAA)
    ("GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA AND PROSGEGRAMMENI" #x1FAB)
    ("GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA AND PROSGEGRAMMENI" #x1FAC)
    ("GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA AND PROSGEGRAMMENI" #x1FAD)
    ("GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI" #x1FAE)
    ("GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI" #x1FAF)
    ("GREEK SMALL LETTER ALPHA WITH VRACHY" #x1FB0)
    ("GREEK SMALL LETTER ALPHA WITH MACRON" #x1FB1)
    ("GREEK SMALL LETTER ALPHA WITH VARIA AND YPOGEGRAMMENI" #x1FB2)
    ("GREEK SMALL LETTER ALPHA WITH YPOGEGRAMMENI" #x1FB3)
    ("GREEK SMALL LETTER ALPHA WITH OXIA AND YPOGEGRAMMENI" #x1FB4)
    ("GREEK SMALL LETTER ALPHA WITH PERISPOMENI" #x1FB6)
    ("GREEK SMALL LETTER ALPHA WITH PERISPOMENI AND YPOGEGRAMMENI" #x1FB7)
    ("GREEK CAPITAL LETTER ALPHA WITH VRACHY" #x1FB8)
    ("GREEK CAPITAL LETTER ALPHA WITH MACRON" #x1FB9)
    ("GREEK CAPITAL LETTER ALPHA WITH VARIA" #x1FBA)
    ("GREEK CAPITAL LETTER ALPHA WITH OXIA" #x1FBB)
    ("GREEK CAPITAL LETTER ALPHA WITH PROSGEGRAMMENI" #x1FBC)
    ("GREEK KORONIS" #x1FBD)
    ("GREEK PROSGEGRAMMENI" #x1FBE)
    ("GREEK PSILI" #x1FBF)
    ("GREEK PERISPOMENI" #x1FC0)
    ("GREEK DIALYTIKA AND PERISPOMENI" #x1FC1)
    ("GREEK SMALL LETTER ETA WITH VARIA AND YPOGEGRAMMENI" #x1FC2)
    ("GREEK SMALL LETTER ETA WITH YPOGEGRAMMENI" #x1FC3)
    ("GREEK SMALL LETTER ETA WITH OXIA AND YPOGEGRAMMENI" #x1FC4)
    ("GREEK SMALL LETTER ETA WITH PERISPOMENI" #x1FC6)
    ("GREEK SMALL LETTER ETA WITH PERISPOMENI AND YPOGEGRAMMENI" #x1FC7)
    ("GREEK CAPITAL LETTER EPSILON WITH VARIA" #x1FC8)
    ("GREEK CAPITAL LETTER EPSILON WITH OXIA" #x1FC9)
    ("GREEK CAPITAL LETTER ETA WITH VARIA" #x1FCA)
    ("GREEK CAPITAL LETTER ETA WITH OXIA" #x1FCB)
    ("GREEK CAPITAL LETTER ETA WITH PROSGEGRAMMENI" #x1FCC)
    ("GREEK PSILI AND VARIA" #x1FCD)
    ("GREEK PSILI AND OXIA" #x1FCE)
    ("GREEK PSILI AND PERISPOMENI" #x1FCF)
    ("GREEK SMALL LETTER IOTA WITH VRACHY" #x1FD0)
    ("GREEK SMALL LETTER IOTA WITH MACRON" #x1FD1)
    ("GREEK SMALL LETTER IOTA WITH DIALYTIKA AND VARIA" #x1FD2)
    ("GREEK SMALL LETTER IOTA WITH DIALYTIKA AND OXIA" #x1FD3)
    ("GREEK SMALL LETTER IOTA WITH PERISPOMENI" #x1FD6)
    ("GREEK SMALL LETTER IOTA WITH DIALYTIKA AND PERISPOMENI" #x1FD7)
    ("GREEK CAPITAL LETTER IOTA WITH VRACHY" #x1FD8)
    ("GREEK CAPITAL LETTER IOTA WITH MACRON" #x1FD9)
    ("GREEK CAPITAL LETTER IOTA WITH VARIA" #x1FDA)
    ("GREEK CAPITAL LETTER IOTA WITH OXIA" #x1FDB)
    ("GREEK DASIA AND VARIA" #x1FDD)
    ("GREEK DASIA AND OXIA" #x1FDE)
    ("GREEK DASIA AND PERISPOMENI" #x1FDF)
    ("GREEK SMALL LETTER UPSILON WITH VRACHY" #x1FE0)
    ("GREEK SMALL LETTER UPSILON WITH MACRON" #x1FE1)
    ("GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND VARIA" #x1FE2)
    ("GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND OXIA" #x1FE3)
    ("GREEK SMALL LETTER RHO WITH PSILI" #x1FE4)
    ("GREEK SMALL LETTER RHO WITH DASIA" #x1FE5)
    ("GREEK SMALL LETTER UPSILON WITH PERISPOMENI" #x1FE6)
    ("GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND PERISPOMENI" #x1FE7)
    ("GREEK CAPITAL LETTER UPSILON WITH VRACHY" #x1FE8)
    ("GREEK CAPITAL LETTER UPSILON WITH MACRON" #x1FE9)
    ("GREEK CAPITAL LETTER UPSILON WITH VARIA" #x1FEA)
    ("GREEK CAPITAL LETTER UPSILON WITH OXIA" #x1FEB)
    ("GREEK CAPITAL LETTER RHO WITH DASIA" #x1FEC)
    ("GREEK DIALYTIKA AND VARIA" #x1FED)
    ("GREEK DIALYTIKA AND OXIA" #x1FEE)
    ("GREEK VARIA" #x1FEF)
    ("GREEK SMALL LETTER OMEGA WITH VARIA AND YPOGEGRAMMENI" #x1FF2)
    ("GREEK SMALL LETTER OMEGA WITH YPOGEGRAMMENI" #x1FF3)
    ("GREEK SMALL LETTER OMEGA WITH OXIA AND YPOGEGRAMMENI" #x1FF4)
    ("GREEK SMALL LETTER OMEGA WITH PERISPOMENI" #x1FF6)
    ("GREEK SMALL LETTER OMEGA WITH PERISPOMENI AND YPOGEGRAMMENI" #x1FF7)
    ("GREEK CAPITAL LETTER OMICRON WITH VARIA" #x1FF8)
    ("GREEK CAPITAL LETTER OMICRON WITH OXIA" #x1FF9)
    ("GREEK CAPITAL LETTER OMEGA WITH VARIA" #x1FFA)
    ("GREEK CAPITAL LETTER OMEGA WITH OXIA" #x1FFB)
    ("GREEK CAPITAL LETTER OMEGA WITH PROSGEGRAMMENI" #x1FFC)
    ("GREEK OXIA" #x1FFD)
    ("GREEK DASIA" #x1FFE)
    ))

;; arch-tag: ace2193e-e5be-4800-898e-efad7ba5d6da
