(nxml-define-char-name-set 'yi-radicals
  '(("YI RADICAL QOT" #xA490)
    ("YI RADICAL LI" #xA491)
    ("YI RADICAL KIT" #xA492)
    ("YI RADICAL NYIP" #xA493)
    ("YI RADICAL CYP" #xA494)
    ("YI RADICAL SSI" #xA495)
    ("YI RADICAL GGOP" #xA496)
    ("YI RADICAL GEP" #xA497)
    ("YI RADICAL MI" #xA498)
    ("YI RADICAL HXIT" #xA499)
    ("YI RADICAL LYR" #xA49A)
    ("YI RADICAL BBUT" #xA49B)
    ("YI RADICAL MOP" #xA49C)
    ("YI RADICAL YO" #xA49D)
    ("YI RADICAL PUT" #xA49E)
    ("YI RADICAL HXUO" #xA49F)
    ("YI RADICAL TAT" #xA4A0)
    ("YI RADICAL GA" #xA4A1)
    ("YI RADICAL ZUP" #xA4A2)
    ("YI RADICAL CYT" #xA4A3)
    ("YI RADICAL DDUR" #xA4A4)
    ("YI RADICAL BUR" #xA4A5)
    ("YI RADICAL GGUO" #xA4A6)
    ("YI RADICAL NYOP" #xA4A7)
    ("YI RADICAL TU" #xA4A8)
    ("YI RADICAL OP" #xA4A9)
    ("YI RADICAL JJUT" #xA4AA)
    ("YI RADICAL ZOT" #xA4AB)
    ("YI RADICAL PYT" #xA4AC)
    ("YI RADICAL HMO" #xA4AD)
    ("YI RADICAL YIT" #xA4AE)
    ("YI RADICAL VUR" #xA4AF)
    ("YI RADICAL SHY" #xA4B0)
    ("YI RADICAL VEP" #xA4B1)
    ("YI RADICAL ZA" #xA4B2)
    ("YI RADICAL JO" #xA4B3)
    ("YI RADICAL NZUP" #xA4B4)
    ("YI RADICAL JJY" #xA4B5)
    ("YI RADICAL GOT" #xA4B6)
    ("YI RADICAL JJIE" #xA4B7)
    ("YI RADICAL WO" #xA4B8)
    ("YI RADICAL DU" #xA4B9)
    ("YI RADICAL SHUR" #xA4BA)
    ("YI RADICAL LIE" #xA4BB)
    ("YI RADICAL CY" #xA4BC)
    ("YI RADICAL CUOP" #xA4BD)
    ("YI RADICAL CIP" #xA4BE)
    ("YI RADICAL HXOP" #xA4BF)
    ("YI RADICAL SHAT" #xA4C0)
    ("YI RADICAL ZUR" #xA4C1)
    ("YI RADICAL SHOP" #xA4C2)
    ("YI RADICAL CHE" #xA4C3)
    ("YI RADICAL ZZIET" #xA4C4)
    ("YI RADICAL NBIE" #xA4C5)
    ("YI RADICAL KE" #xA4C6)
    ))

;; arch-tag: 2c93cdf1-e6f2-4a91-a798-3e1d7c094cf4
