(nxml-define-char-name-set 'kangxi-radicals
  '(("KANGXI RADICAL ONE" #x2F00)
    ("KANGXI RADICAL LINE" #x2F01)
    ("KANGXI RADICAL DOT" #x2F02)
    ("KANGXI RADICAL SLASH" #x2F03)
    ("KANGXI RADICAL SECOND" #x2F04)
    ("KANGXI RADICAL HOOK" #x2F05)
    ("KANGXI RADICAL TWO" #x2F06)
    ("KANGXI RADICAL LID" #x2F07)
    ("KANGXI RADICAL MAN" #x2F08)
    ("KANGXI RADICAL LEGS" #x2F09)
    ("KANGXI RADICAL ENTER" #x2F0A)
    ("KANGXI RADICAL EIGHT" #x2F0B)
    ("KANGXI RADICAL DOWN BOX" #x2F0C)
    ("KANGXI RADICAL COVER" #x2F0D)
    ("KANGXI RADICAL ICE" #x2F0E)
    ("KANGXI RADICAL TABLE" #x2F0F)
    ("KANGXI RADICAL OPEN BOX" #x2F10)
    ("KANGXI RADICAL KNIFE" #x2F11)
    ("KANGXI RADICAL POWER" #x2F12)
    ("KANGXI RADICAL WRAP" #x2F13)
    ("KANGXI RADICAL SPOON" #x2F14)
    ("KANGXI RADICAL RIGHT OPEN BOX" #x2F15)
    ("KANGXI RADICAL HIDING ENCLOSURE" #x2F16)
    ("KANGXI RADICAL TEN" #x2F17)
    ("KANGXI RADICAL DIVINATION" #x2F18)
    ("KANGXI RADICAL SEAL" #x2F19)
    ("KANGXI RADICAL CLIFF" #x2F1A)
    ("KANGXI RADICAL PRIVATE" #x2F1B)
    ("KANGXI RADICAL AGAIN" #x2F1C)
    ("KANGXI RADICAL MOUTH" #x2F1D)
    ("KANGXI RADICAL ENCLOSURE" #x2F1E)
    ("KANGXI RADICAL EARTH" #x2F1F)
    ("KANGXI RADICAL SCHOLAR" #x2F20)
    ("KANGXI RADICAL GO" #x2F21)
    ("KANGXI RADICAL GO SLOWLY" #x2F22)
    ("KANGXI RADICAL EVENING" #x2F23)
    ("KANGXI RADICAL BIG" #x2F24)
    ("KANGXI RADICAL WOMAN" #x2F25)
    ("KANGXI RADICAL CHILD" #x2F26)
    ("KANGXI RADICAL ROOF" #x2F27)
    ("KANGXI RADICAL INCH" #x2F28)
    ("KANGXI RADICAL SMALL" #x2F29)
    ("KANGXI RADICAL LAME" #x2F2A)
    ("KANGXI RADICAL CORPSE" #x2F2B)
    ("KANGXI RADICAL SPROUT" #x2F2C)
    ("KANGXI RADICAL MOUNTAIN" #x2F2D)
    ("KANGXI RADICAL RIVER" #x2F2E)
    ("KANGXI RADICAL WORK" #x2F2F)
    ("KANGXI RADICAL ONESELF" #x2F30)
    ("KANGXI RADICAL TURBAN" #x2F31)
    ("KANGXI RADICAL DRY" #x2F32)
    ("KANGXI RADICAL SHORT THREAD" #x2F33)
    ("KANGXI RADICAL DOTTED CLIFF" #x2F34)
    ("KANGXI RADICAL LONG STRIDE" #x2F35)
    ("KANGXI RADICAL TWO HANDS" #x2F36)
    ("KANGXI RADICAL SHOOT" #x2F37)
    ("KANGXI RADICAL BOW" #x2F38)
    ("KANGXI RADICAL SNOUT" #x2F39)
    ("KANGXI RADICAL BRISTLE" #x2F3A)
    ("KANGXI RADICAL STEP" #x2F3B)
    ("KANGXI RADICAL HEART" #x2F3C)
    ("KANGXI RADICAL HALBERD" #x2F3D)
    ("KANGXI RADICAL DOOR" #x2F3E)
    ("KANGXI RADICAL HAND" #x2F3F)
    ("KANGXI RADICAL BRANCH" #x2F40)
    ("KANGXI RADICAL RAP" #x2F41)
    ("KANGXI RADICAL SCRIPT" #x2F42)
    ("KANGXI RADICAL DIPPER" #x2F43)
    ("KANGXI RADICAL AXE" #x2F44)
    ("KANGXI RADICAL SQUARE" #x2F45)
    ("KANGXI RADICAL NOT" #x2F46)
    ("KANGXI RADICAL SUN" #x2F47)
    ("KANGXI RADICAL SAY" #x2F48)
    ("KANGXI RADICAL MOON" #x2F49)
    ("KANGXI RADICAL TREE" #x2F4A)
    ("KANGXI RADICAL LACK" #x2F4B)
    ("KANGXI RADICAL STOP" #x2F4C)
    ("KANGXI RADICAL DEATH" #x2F4D)
    ("KANGXI RADICAL WEAPON" #x2F4E)
    ("KANGXI RADICAL DO NOT" #x2F4F)
    ("KANGXI RADICAL COMPARE" #x2F50)
    ("KANGXI RADICAL FUR" #x2F51)
    ("KANGXI RADICAL CLAN" #x2F52)
    ("KANGXI RADICAL STEAM" #x2F53)
    ("KANGXI RADICAL WATER" #x2F54)
    ("KANGXI RADICAL FIRE" #x2F55)
    ("KANGXI RADICAL CLAW" #x2F56)
    ("KANGXI RADICAL FATHER" #x2F57)
    ("KANGXI RADICAL DOUBLE X" #x2F58)
    ("KANGXI RADICAL HALF TREE TRUNK" #x2F59)
    ("KANGXI RADICAL SLICE" #x2F5A)
    ("KANGXI RADICAL FANG" #x2F5B)
    ("KANGXI RADICAL COW" #x2F5C)
    ("KANGXI RADICAL DOG" #x2F5D)
    ("KANGXI RADICAL PROFOUND" #x2F5E)
    ("KANGXI RADICAL JADE" #x2F5F)
    ("KANGXI RADICAL MELON" #x2F60)
    ("KANGXI RADICAL TILE" #x2F61)
    ("KANGXI RADICAL SWEET" #x2F62)
    ("KANGXI RADICAL LIFE" #x2F63)
    ("KANGXI RADICAL USE" #x2F64)
    ("KANGXI RADICAL FIELD" #x2F65)
    ("KANGXI RADICAL BOLT OF CLOTH" #x2F66)
    ("KANGXI RADICAL SICKNESS" #x2F67)
    ("KANGXI RADICAL DOTTED TENT" #x2F68)
    ("KANGXI RADICAL WHITE" #x2F69)
    ("KANGXI RADICAL SKIN" #x2F6A)
    ("KANGXI RADICAL DISH" #x2F6B)
    ("KANGXI RADICAL EYE" #x2F6C)
    ("KANGXI RADICAL SPEAR" #x2F6D)
    ("KANGXI RADICAL ARROW" #x2F6E)
    ("KANGXI RADICAL STONE" #x2F6F)
    ("KANGXI RADICAL SPIRIT" #x2F70)
    ("KANGXI RADICAL TRACK" #x2F71)
    ("KANGXI RADICAL GRAIN" #x2F72)
    ("KANGXI RADICAL CAVE" #x2F73)
    ("KANGXI RADICAL STAND" #x2F74)
    ("KANGXI RADICAL BAMBOO" #x2F75)
    ("KANGXI RADICAL RICE" #x2F76)
    ("KANGXI RADICAL SILK" #x2F77)
    ("KANGXI RADICAL JAR" #x2F78)
    ("KANGXI RADICAL NET" #x2F79)
    ("KANGXI RADICAL SHEEP" #x2F7A)
    ("KANGXI RADICAL FEATHER" #x2F7B)
    ("KANGXI RADICAL OLD" #x2F7C)
    ("KANGXI RADICAL AND" #x2F7D)
    ("KANGXI RADICAL PLOW" #x2F7E)
    ("KANGXI RADICAL EAR" #x2F7F)
    ("KANGXI RADICAL BRUSH" #x2F80)
    ("KANGXI RADICAL MEAT" #x2F81)
    ("KANGXI RADICAL MINISTER" #x2F82)
    ("KANGXI RADICAL SELF" #x2F83)
    ("KANGXI RADICAL ARRIVE" #x2F84)
    ("KANGXI RADICAL MORTAR" #x2F85)
    ("KANGXI RADICAL TONGUE" #x2F86)
    ("KANGXI RADICAL OPPOSE" #x2F87)
    ("KANGXI RADICAL BOAT" #x2F88)
    ("KANGXI RADICAL STOPPING" #x2F89)
    ("KANGXI RADICAL COLOR" #x2F8A)
    ("KANGXI RADICAL GRASS" #x2F8B)
    ("KANGXI RADICAL TIGER" #x2F8C)
    ("KANGXI RADICAL INSECT" #x2F8D)
    ("KANGXI RADICAL BLOOD" #x2F8E)
    ("KANGXI RADICAL WALK ENCLOSURE" #x2F8F)
    ("KANGXI RADICAL CLOTHES" #x2F90)
    ("KANGXI RADICAL WEST" #x2F91)
    ("KANGXI RADICAL SEE" #x2F92)
    ("KANGXI RADICAL HORN" #x2F93)
    ("KANGXI RADICAL SPEECH" #x2F94)
    ("KANGXI RADICAL VALLEY" #x2F95)
    ("KANGXI RADICAL BEAN" #x2F96)
    ("KANGXI RADICAL PIG" #x2F97)
    ("KANGXI RADICAL BADGER" #x2F98)
    ("KANGXI RADICAL SHELL" #x2F99)
    ("KANGXI RADICAL RED" #x2F9A)
    ("KANGXI RADICAL RUN" #x2F9B)
    ("KANGXI RADICAL FOOT" #x2F9C)
    ("KANGXI RADICAL BODY" #x2F9D)
    ("KANGXI RADICAL CART" #x2F9E)
    ("KANGXI RADICAL BITTER" #x2F9F)
    ("KANGXI RADICAL MORNING" #x2FA0)
    ("KANGXI RADICAL WALK" #x2FA1)
    ("KANGXI RADICAL CITY" #x2FA2)
    ("KANGXI RADICAL WINE" #x2FA3)
    ("KANGXI RADICAL DISTINGUISH" #x2FA4)
    ("KANGXI RADICAL VILLAGE" #x2FA5)
    ("KANGXI RADICAL GOLD" #x2FA6)
    ("KANGXI RADICAL LONG" #x2FA7)
    ("KANGXI RADICAL GATE" #x2FA8)
    ("KANGXI RADICAL MOUND" #x2FA9)
    ("KANGXI RADICAL SLAVE" #x2FAA)
    ("KANGXI RADICAL SHORT TAILED BIRD" #x2FAB)
    ("KANGXI RADICAL RAIN" #x2FAC)
    ("KANGXI RADICAL BLUE" #x2FAD)
    ("KANGXI RADICAL WRONG" #x2FAE)
    ("KANGXI RADICAL FACE" #x2FAF)
    ("KANGXI RADICAL LEATHER" #x2FB0)
    ("KANGXI RADICAL TANNED LEATHER" #x2FB1)
    ("KANGXI RADICAL LEEK" #x2FB2)
    ("KANGXI RADICAL SOUND" #x2FB3)
    ("KANGXI RADICAL LEAF" #x2FB4)
    ("KANGXI RADICAL WIND" #x2FB5)
    ("KANGXI RADICAL FLY" #x2FB6)
    ("KANGXI RADICAL EAT" #x2FB7)
    ("KANGXI RADICAL HEAD" #x2FB8)
    ("KANGXI RADICAL FRAGRANT" #x2FB9)
    ("KANGXI RADICAL HORSE" #x2FBA)
    ("KANGXI RADICAL BONE" #x2FBB)
    ("KANGXI RADICAL TALL" #x2FBC)
    ("KANGXI RADICAL HAIR" #x2FBD)
    ("KANGXI RADICAL FIGHT" #x2FBE)
    ("KANGXI RADICAL SACRIFICIAL WINE" #x2FBF)
    ("KANGXI RADICAL CAULDRON" #x2FC0)
    ("KANGXI RADICAL GHOST" #x2FC1)
    ("KANGXI RADICAL FISH" #x2FC2)
    ("KANGXI RADICAL BIRD" #x2FC3)
    ("KANGXI RADICAL SALT" #x2FC4)
    ("KANGXI RADICAL DEER" #x2FC5)
    ("KANGXI RADICAL WHEAT" #x2FC6)
    ("KANGXI RADICAL HEMP" #x2FC7)
    ("KANGXI RADICAL YELLOW" #x2FC8)
    ("KANGXI RADICAL MILLET" #x2FC9)
    ("KANGXI RADICAL BLACK" #x2FCA)
    ("KANGXI RADICAL EMBROIDERY" #x2FCB)
    ("KANGXI RADICAL FROG" #x2FCC)
    ("KANGXI RADICAL TRIPOD" #x2FCD)
    ("KANGXI RADICAL DRUM" #x2FCE)
    ("KANGXI RADICAL RAT" #x2FCF)
    ("KANGXI RADICAL NOSE" #x2FD0)
    ("KANGXI RADICAL EVEN" #x2FD1)
    ("KANGXI RADICAL TOOTH" #x2FD2)
    ("KANGXI RADICAL DRAGON" #x2FD3)
    ("KANGXI RADICAL TURTLE" #x2FD4)
    ("KANGXI RADICAL FLUTE" #x2FD5)
    ))

;; arch-tag: e7210935-b74b-4a42-88c6-de146c755742
