(nxml-define-char-name-set 'halfwidth-and-fullwidth-forms
  '(("FULLWIDTH EXCLAMATION MARK" #xFF01)
    ("FULLWIDTH QUOTATION MARK" #xFF02)
    ("FULLWIDTH NUMBER SIGN" #xFF03)
    ("FULLWIDTH DOLLAR SIGN" #xFF04)
    ("FULLWIDTH PERCENT SIGN" #xFF05)
    ("FULLWIDTH AMPERSAND" #xFF06)
    ("FULLWIDTH APOSTROPHE" #xFF07)
    ("FULLWIDTH LEFT PARENTHESIS" #xFF08)
    ("FULLWIDTH RIGHT PARENTHESIS" #xFF09)
    ("FULLWIDTH ASTERISK" #xFF0A)
    ("FULLWIDTH PLUS SIGN" #xFF0B)
    ("FULLWIDTH COMMA" #xFF0C)
    ("FULLWIDTH HYPHEN-MINUS" #xFF0D)
    ("FULLWIDTH FULL STOP" #xFF0E)
    ("FULLWIDTH SOLIDUS" #xFF0F)
    ("FULLWIDTH DIGIT ZERO" #xFF10)
    ("FULLWIDTH DIGIT ONE" #xFF11)
    ("FULLWIDTH DIGIT TWO" #xFF12)
    ("FULLWIDTH DIGIT THREE" #xFF13)
    ("FULLWIDTH DIGIT FOUR" #xFF14)
    ("FULLWIDTH DIGIT FIVE" #xFF15)
    ("FULLWIDTH DIGIT SIX" #xFF16)
    ("FULLWIDTH DIGIT SEVEN" #xFF17)
    ("FULLWIDTH DIGIT EIGHT" #xFF18)
    ("FULLWIDTH DIGIT NINE" #xFF19)
    ("FULLWIDTH COLON" #xFF1A)
    ("FULLWIDTH SEMICOLON" #xFF1B)
    ("FULLWIDTH LESS-THAN SIGN" #xFF1C)
    ("FULLWIDTH EQUALS SIGN" #xFF1D)
    ("FULLWIDTH GREATER-THAN SIGN" #xFF1E)
    ("FULLWIDTH QUESTION MARK" #xFF1F)
    ("FULLWIDTH COMMERCIAL AT" #xFF20)
    ("FULLWIDTH LATIN CAPITAL LETTER A" #xFF21)
    ("FULLWIDTH LATIN CAPITAL LETTER B" #xFF22)
    ("FULLWIDTH LATIN CAPITAL LETTER C" #xFF23)
    ("FULLWIDTH LATIN CAPITAL LETTER D" #xFF24)
    ("FULLWIDTH LATIN CAPITAL LETTER E" #xFF25)
    ("FULLWIDTH LATIN CAPITAL LETTER F" #xFF26)
    ("FULLWIDTH LATIN CAPITAL LETTER G" #xFF27)
    ("FULLWIDTH LATIN CAPITAL LETTER H" #xFF28)
    ("FULLWIDTH LATIN CAPITAL LETTER I" #xFF29)
    ("FULLWIDTH LATIN CAPITAL LETTER J" #xFF2A)
    ("FULLWIDTH LATIN CAPITAL LETTER K" #xFF2B)
    ("FULLWIDTH LATIN CAPITAL LETTER L" #xFF2C)
    ("FULLWIDTH LATIN CAPITAL LETTER M" #xFF2D)
    ("FULLWIDTH LATIN CAPITAL LETTER N" #xFF2E)
    ("FULLWIDTH LATIN CAPITAL LETTER O" #xFF2F)
    ("FULLWIDTH LATIN CAPITAL LETTER P" #xFF30)
    ("FULLWIDTH LATIN CAPITAL LETTER Q" #xFF31)
    ("FULLWIDTH LATIN CAPITAL LETTER R" #xFF32)
    ("FULLWIDTH LATIN CAPITAL LETTER S" #xFF33)
    ("FULLWIDTH LATIN CAPITAL LETTER T" #xFF34)
    ("FULLWIDTH LATIN CAPITAL LETTER U" #xFF35)
    ("FULLWIDTH LATIN CAPITAL LETTER V" #xFF36)
    ("FULLWIDTH LATIN CAPITAL LETTER W" #xFF37)
    ("FULLWIDTH LATIN CAPITAL LETTER X" #xFF38)
    ("FULLWIDTH LATIN CAPITAL LETTER Y" #xFF39)
    ("FULLWIDTH LATIN CAPITAL LETTER Z" #xFF3A)
    ("FULLWIDTH LEFT SQUARE BRACKET" #xFF3B)
    ("FULLWIDTH REVERSE SOLIDUS" #xFF3C)
    ("FULLWIDTH RIGHT SQUARE BRACKET" #xFF3D)
    ("FULLWIDTH CIRCUMFLEX ACCENT" #xFF3E)
    ("FULLWIDTH LOW LINE" #xFF3F)
    ("FULLWIDTH GRAVE ACCENT" #xFF40)
    ("FULLWIDTH LATIN SMALL LETTER A" #xFF41)
    ("FULLWIDTH LATIN SMALL LETTER B" #xFF42)
    ("FULLWIDTH LATIN SMALL LETTER C" #xFF43)
    ("FULLWIDTH LATIN SMALL LETTER D" #xFF44)
    ("FULLWIDTH LATIN SMALL LETTER E" #xFF45)
    ("FULLWIDTH LATIN SMALL LETTER F" #xFF46)
    ("FULLWIDTH LATIN SMALL LETTER G" #xFF47)
    ("FULLWIDTH LATIN SMALL LETTER H" #xFF48)
    ("FULLWIDTH LATIN SMALL LETTER I" #xFF49)
    ("FULLWIDTH LATIN SMALL LETTER J" #xFF4A)
    ("FULLWIDTH LATIN SMALL LETTER K" #xFF4B)
    ("FULLWIDTH LATIN SMALL LETTER L" #xFF4C)
    ("FULLWIDTH LATIN SMALL LETTER M" #xFF4D)
    ("FULLWIDTH LATIN SMALL LETTER N" #xFF4E)
    ("FULLWIDTH LATIN SMALL LETTER O" #xFF4F)
    ("FULLWIDTH LATIN SMALL LETTER P" #xFF50)
    ("FULLWIDTH LATIN SMALL LETTER Q" #xFF51)
    ("FULLWIDTH LATIN SMALL LETTER R" #xFF52)
    ("FULLWIDTH LATIN SMALL LETTER S" #xFF53)
    ("FULLWIDTH LATIN SMALL LETTER T" #xFF54)
    ("FULLWIDTH LATIN SMALL LETTER U" #xFF55)
    ("FULLWIDTH LATIN SMALL LETTER V" #xFF56)
    ("FULLWIDTH LATIN SMALL LETTER W" #xFF57)
    ("FULLWIDTH LATIN SMALL LETTER X" #xFF58)
    ("FULLWIDTH LATIN SMALL LETTER Y" #xFF59)
    ("FULLWIDTH LATIN SMALL LETTER Z" #xFF5A)
    ("FULLWIDTH LEFT CURLY BRACKET" #xFF5B)
    ("FULLWIDTH VERTICAL LINE" #xFF5C)
    ("FULLWIDTH RIGHT CURLY BRACKET" #xFF5D)
    ("FULLWIDTH TILDE" #xFF5E)
    ("FULLWIDTH LEFT WHITE PARENTHESIS" #xFF5F)
    ("FULLWIDTH RIGHT WHITE PARENTHESIS" #xFF60)
    ("HALFWIDTH IDEOGRAPHIC FULL STOP" #xFF61)
    ("HALFWIDTH LEFT CORNER BRACKET" #xFF62)
    ("HALFWIDTH RIGHT CORNER BRACKET" #xFF63)
    ("HALFWIDTH IDEOGRAPHIC COMMA" #xFF64)
    ("HALFWIDTH KATAKANA MIDDLE DOT" #xFF65)
    ("HALFWIDTH KATAKANA LETTER WO" #xFF66)
    ("HALFWIDTH KATAKANA LETTER SMALL A" #xFF67)
    ("HALFWIDTH KATAKANA LETTER SMALL I" #xFF68)
    ("HALFWIDTH KATAKANA LETTER SMALL U" #xFF69)
    ("HALFWIDTH KATAKANA LETTER SMALL E" #xFF6A)
    ("HALFWIDTH KATAKANA LETTER SMALL O" #xFF6B)
    ("HALFWIDTH KATAKANA LETTER SMALL YA" #xFF6C)
    ("HALFWIDTH KATAKANA LETTER SMALL YU" #xFF6D)
    ("HALFWIDTH KATAKANA LETTER SMALL YO" #xFF6E)
    ("HALFWIDTH KATAKANA LETTER SMALL TU" #xFF6F)
    ("HALFWIDTH KATAKANA-HIRAGANA PROLONGED SOUND MARK" #xFF70)
    ("HALFWIDTH KATAKANA LETTER A" #xFF71)
    ("HALFWIDTH KATAKANA LETTER I" #xFF72)
    ("HALFWIDTH KATAKANA LETTER U" #xFF73)
    ("HALFWIDTH KATAKANA LETTER E" #xFF74)
    ("HALFWIDTH KATAKANA LETTER O" #xFF75)
    ("HALFWIDTH KATAKANA LETTER KA" #xFF76)
    ("HALFWIDTH KATAKANA LETTER KI" #xFF77)
    ("HALFWIDTH KATAKANA LETTER KU" #xFF78)
    ("HALFWIDTH KATAKANA LETTER KE" #xFF79)
    ("HALFWIDTH KATAKANA LETTER KO" #xFF7A)
    ("HALFWIDTH KATAKANA LETTER SA" #xFF7B)
    ("HALFWIDTH KATAKANA LETTER SI" #xFF7C)
    ("HALFWIDTH KATAKANA LETTER SU" #xFF7D)
    ("HALFWIDTH KATAKANA LETTER SE" #xFF7E)
    ("HALFWIDTH KATAKANA LETTER SO" #xFF7F)
    ("HALFWIDTH KATAKANA LETTER TA" #xFF80)
    ("HALFWIDTH KATAKANA LETTER TI" #xFF81)
    ("HALFWIDTH KATAKANA LETTER TU" #xFF82)
    ("HALFWIDTH KATAKANA LETTER TE" #xFF83)
    ("HALFWIDTH KATAKANA LETTER TO" #xFF84)
    ("HALFWIDTH KATAKANA LETTER NA" #xFF85)
    ("HALFWIDTH KATAKANA LETTER NI" #xFF86)
    ("HALFWIDTH KATAKANA LETTER NU" #xFF87)
    ("HALFWIDTH KATAKANA LETTER NE" #xFF88)
    ("HALFWIDTH KATAKANA LETTER NO" #xFF89)
    ("HALFWIDTH KATAKANA LETTER HA" #xFF8A)
    ("HALFWIDTH KATAKANA LETTER HI" #xFF8B)
    ("HALFWIDTH KATAKANA LETTER HU" #xFF8C)
    ("HALFWIDTH KATAKANA LETTER HE" #xFF8D)
    ("HALFWIDTH KATAKANA LETTER HO" #xFF8E)
    ("HALFWIDTH KATAKANA LETTER MA" #xFF8F)
    ("HALFWIDTH KATAKANA LETTER MI" #xFF90)
    ("HALFWIDTH KATAKANA LETTER MU" #xFF91)
    ("HALFWIDTH KATAKANA LETTER ME" #xFF92)
    ("HALFWIDTH KATAKANA LETTER MO" #xFF93)
    ("HALFWIDTH KATAKANA LETTER YA" #xFF94)
    ("HALFWIDTH KATAKANA LETTER YU" #xFF95)
    ("HALFWIDTH KATAKANA LETTER YO" #xFF96)
    ("HALFWIDTH KATAKANA LETTER RA" #xFF97)
    ("HALFWIDTH KATAKANA LETTER RI" #xFF98)
    ("HALFWIDTH KATAKANA LETTER RU" #xFF99)
    ("HALFWIDTH KATAKANA LETTER RE" #xFF9A)
    ("HALFWIDTH KATAKANA LETTER RO" #xFF9B)
    ("HALFWIDTH KATAKANA LETTER WA" #xFF9C)
    ("HALFWIDTH KATAKANA LETTER N" #xFF9D)
    ("HALFWIDTH KATAKANA VOICED SOUND MARK" #xFF9E)
    ("HALFWIDTH KATAKANA SEMI-VOICED SOUND MARK" #xFF9F)
    ("HALFWIDTH HANGUL FILLER" #xFFA0)
    ("HALFWIDTH HANGUL LETTER KIYEOK" #xFFA1)
    ("HALFWIDTH HANGUL LETTER SSANGKIYEOK" #xFFA2)
    ("HALFWIDTH HANGUL LETTER KIYEOK-SIOS" #xFFA3)
    ("HALFWIDTH HANGUL LETTER NIEUN" #xFFA4)
    ("HALFWIDTH HANGUL LETTER NIEUN-CIEUC" #xFFA5)
    ("HALFWIDTH HANGUL LETTER NIEUN-HIEUH" #xFFA6)
    ("HALFWIDTH HANGUL LETTER TIKEUT" #xFFA7)
    ("HALFWIDTH HANGUL LETTER SSANGTIKEUT" #xFFA8)
    ("HALFWIDTH HANGUL LETTER RIEUL" #xFFA9)
    ("HALFWIDTH HANGUL LETTER RIEUL-KIYEOK" #xFFAA)
    ("HALFWIDTH HANGUL LETTER RIEUL-MIEUM" #xFFAB)
    ("HALFWIDTH HANGUL LETTER RIEUL-PIEUP" #xFFAC)
    ("HALFWIDTH HANGUL LETTER RIEUL-SIOS" #xFFAD)
    ("HALFWIDTH HANGUL LETTER RIEUL-THIEUTH" #xFFAE)
    ("HALFWIDTH HANGUL LETTER RIEUL-PHIEUPH" #xFFAF)
    ("HALFWIDTH HANGUL LETTER RIEUL-HIEUH" #xFFB0)
    ("HALFWIDTH HANGUL LETTER MIEUM" #xFFB1)
    ("HALFWIDTH HANGUL LETTER PIEUP" #xFFB2)
    ("HALFWIDTH HANGUL LETTER SSANGPIEUP" #xFFB3)
    ("HALFWIDTH HANGUL LETTER PIEUP-SIOS" #xFFB4)
    ("HALFWIDTH HANGUL LETTER SIOS" #xFFB5)
    ("HALFWIDTH HANGUL LETTER SSANGSIOS" #xFFB6)
    ("HALFWIDTH HANGUL LETTER IEUNG" #xFFB7)
    ("HALFWIDTH HANGUL LETTER CIEUC" #xFFB8)
    ("HALFWIDTH HANGUL LETTER SSANGCIEUC" #xFFB9)
    ("HALFWIDTH HANGUL LETTER CHIEUCH" #xFFBA)
    ("HALFWIDTH HANGUL LETTER KHIEUKH" #xFFBB)
    ("HALFWIDTH HANGUL LETTER THIEUTH" #xFFBC)
    ("HALFWIDTH HANGUL LETTER PHIEUPH" #xFFBD)
    ("HALFWIDTH HANGUL LETTER HIEUH" #xFFBE)
    ("HALFWIDTH HANGUL LETTER A" #xFFC2)
    ("HALFWIDTH HANGUL LETTER AE" #xFFC3)
    ("HALFWIDTH HANGUL LETTER YA" #xFFC4)
    ("HALFWIDTH HANGUL LETTER YAE" #xFFC5)
    ("HALFWIDTH HANGUL LETTER EO" #xFFC6)
    ("HALFWIDTH HANGUL LETTER E" #xFFC7)
    ("HALFWIDTH HANGUL LETTER YEO" #xFFCA)
    ("HALFWIDTH HANGUL LETTER YE" #xFFCB)
    ("HALFWIDTH HANGUL LETTER O" #xFFCC)
    ("HALFWIDTH HANGUL LETTER WA" #xFFCD)
    ("HALFWIDTH HANGUL LETTER WAE" #xFFCE)
    ("HALFWIDTH HANGUL LETTER OE" #xFFCF)
    ("HALFWIDTH HANGUL LETTER YO" #xFFD2)
    ("HALFWIDTH HANGUL LETTER U" #xFFD3)
    ("HALFWIDTH HANGUL LETTER WEO" #xFFD4)
    ("HALFWIDTH HANGUL LETTER WE" #xFFD5)
    ("HALFWIDTH HANGUL LETTER WI" #xFFD6)
    ("HALFWIDTH HANGUL LETTER YU" #xFFD7)
    ("HALFWIDTH HANGUL LETTER EU" #xFFDA)
    ("HALFWIDTH HANGUL LETTER YI" #xFFDB)
    ("HALFWIDTH HANGUL LETTER I" #xFFDC)
    ("FULLWIDTH CENT SIGN" #xFFE0)
    ("FULLWIDTH POUND SIGN" #xFFE1)
    ("FULLWIDTH NOT SIGN" #xFFE2)
    ("FULLWIDTH MACRON" #xFFE3)
    ("FULLWIDTH BROKEN BAR" #xFFE4)
    ("FULLWIDTH YEN SIGN" #xFFE5)
    ("FULLWIDTH WON SIGN" #xFFE6)
    ("HALFWIDTH FORMS LIGHT VERTICAL" #xFFE8)
    ("HALFWIDTH LEFTWARDS ARROW" #xFFE9)
    ("HALFWIDTH UPWARDS ARROW" #xFFEA)
    ("HALFWIDTH RIGHTWARDS ARROW" #xFFEB)
    ("HALFWIDTH DOWNWARDS ARROW" #xFFEC)
    ("HALFWIDTH BLACK SQUARE" #xFFED)
    ("HALFWIDTH WHITE CIRCLE" #xFFEE)
    ))

;; arch-tag: 8af7a95d-6e71-47da-9679-b516b6e2f01a
