(nxml-define-char-name-set 'arabic-presentation-forms-a
  '(("ARABIC LETTER ALEF WASLA ISOLATED FORM" #xFB50)
    ("ARABIC LETTER ALEF WASLA FINAL FORM" #xFB51)
    ("ARABIC LETTER BEEH ISOLATED FORM" #xFB52)
    ("ARABIC LETTER BEEH FINAL FORM" #xFB53)
    ("ARABIC LETTER BEEH INITIAL FORM" #xFB54)
    ("ARABIC LETTER BEEH MEDIAL FORM" #xFB55)
    ("ARABIC LETTER PEH ISOLATED FORM" #xFB56)
    ("ARABIC LETTER PEH FINAL FORM" #xFB57)
    ("ARABIC LETTER PEH INITIAL FORM" #xFB58)
    ("ARABIC LETTER PEH MEDIAL FORM" #xFB59)
    ("ARABIC LETTER BEHEH ISOLATED FORM" #xFB5A)
    ("ARABIC LETTER BEHEH FINAL FORM" #xFB5B)
    ("ARABIC LETTER BEHEH INITIAL FORM" #xFB5C)
    ("ARABIC LETTER BEHEH MEDIAL FORM" #xFB5D)
    ("ARABIC LETTER TTEHEH ISOLATED FORM" #xFB5E)
    ("ARABIC LETTER TTEHEH FINAL FORM" #xFB5F)
    ("ARABIC LETTER TTEHEH INITIAL FORM" #xFB60)
    ("ARABIC LETTER TTEHEH MEDIAL FORM" #xFB61)
    ("ARABIC LETTER TEHEH ISOLATED FORM" #xFB62)
    ("ARABIC LETTER TEHEH FINAL FORM" #xFB63)
    ("ARABIC LETTER TEHEH INITIAL FORM" #xFB64)
    ("ARABIC LETTER TEHEH MEDIAL FORM" #xFB65)
    ("ARABIC LETTER TTEH ISOLATED FORM" #xFB66)
    ("ARABIC LETTER TTEH FINAL FORM" #xFB67)
    ("ARABIC LETTER TTEH INITIAL FORM" #xFB68)
    ("ARABIC LETTER TTEH MEDIAL FORM" #xFB69)
    ("ARABIC LETTER VEH ISOLATED FORM" #xFB6A)
    ("ARABIC LETTER VEH FINAL FORM" #xFB6B)
    ("ARABIC LETTER VEH INITIAL FORM" #xFB6C)
    ("ARABIC LETTER VEH MEDIAL FORM" #xFB6D)
    ("ARABIC LETTER PEHEH ISOLATED FORM" #xFB6E)
    ("ARABIC LETTER PEHEH FINAL FORM" #xFB6F)
    ("ARABIC LETTER PEHEH INITIAL FORM" #xFB70)
    ("ARABIC LETTER PEHEH MEDIAL FORM" #xFB71)
    ("ARABIC LETTER DYEH ISOLATED FORM" #xFB72)
    ("ARABIC LETTER DYEH FINAL FORM" #xFB73)
    ("ARABIC LETTER DYEH INITIAL FORM" #xFB74)
    ("ARABIC LETTER DYEH MEDIAL FORM" #xFB75)
    ("ARABIC LETTER NYEH ISOLATED FORM" #xFB76)
    ("ARABIC LETTER NYEH FINAL FORM" #xFB77)
    ("ARABIC LETTER NYEH INITIAL FORM" #xFB78)
    ("ARABIC LETTER NYEH MEDIAL FORM" #xFB79)
    ("ARABIC LETTER TCHEH ISOLATED FORM" #xFB7A)
    ("ARABIC LETTER TCHEH FINAL FORM" #xFB7B)
    ("ARABIC LETTER TCHEH INITIAL FORM" #xFB7C)
    ("ARABIC LETTER TCHEH MEDIAL FORM" #xFB7D)
    ("ARABIC LETTER TCHEHEH ISOLATED FORM" #xFB7E)
    ("ARABIC LETTER TCHEHEH FINAL FORM" #xFB7F)
    ("ARABIC LETTER TCHEHEH INITIAL FORM" #xFB80)
    ("ARABIC LETTER TCHEHEH MEDIAL FORM" #xFB81)
    ("ARABIC LETTER DDAHAL ISOLATED FORM" #xFB82)
    ("ARABIC LETTER DDAHAL FINAL FORM" #xFB83)
    ("ARABIC LETTER DAHAL ISOLATED FORM" #xFB84)
    ("ARABIC LETTER DAHAL FINAL FORM" #xFB85)
    ("ARABIC LETTER DUL ISOLATED FORM" #xFB86)
    ("ARABIC LETTER DUL FINAL FORM" #xFB87)
    ("ARABIC LETTER DDAL ISOLATED FORM" #xFB88)
    ("ARABIC LETTER DDAL FINAL FORM" #xFB89)
    ("ARABIC LETTER JEH ISOLATED FORM" #xFB8A)
    ("ARABIC LETTER JEH FINAL FORM" #xFB8B)
    ("ARABIC LETTER RREH ISOLATED FORM" #xFB8C)
    ("ARABIC LETTER RREH FINAL FORM" #xFB8D)
    ("ARABIC LETTER KEHEH ISOLATED FORM" #xFB8E)
    ("ARABIC LETTER KEHEH FINAL FORM" #xFB8F)
    ("ARABIC LETTER KEHEH INITIAL FORM" #xFB90)
    ("ARABIC LETTER KEHEH MEDIAL FORM" #xFB91)
    ("ARABIC LETTER GAF ISOLATED FORM" #xFB92)
    ("ARABIC LETTER GAF FINAL FORM" #xFB93)
    ("ARABIC LETTER GAF INITIAL FORM" #xFB94)
    ("ARABIC LETTER GAF MEDIAL FORM" #xFB95)
    ("ARABIC LETTER GUEH ISOLATED FORM" #xFB96)
    ("ARABIC LETTER GUEH FINAL FORM" #xFB97)
    ("ARABIC LETTER GUEH INITIAL FORM" #xFB98)
    ("ARABIC LETTER GUEH MEDIAL FORM" #xFB99)
    ("ARABIC LETTER NGOEH ISOLATED FORM" #xFB9A)
    ("ARABIC LETTER NGOEH FINAL FORM" #xFB9B)
    ("ARABIC LETTER NGOEH INITIAL FORM" #xFB9C)
    ("ARABIC LETTER NGOEH MEDIAL FORM" #xFB9D)
    ("ARABIC LETTER NOON GHUNNA ISOLATED FORM" #xFB9E)
    ("ARABIC LETTER NOON GHUNNA FINAL FORM" #xFB9F)
    ("ARABIC LETTER RNOON ISOLATED FORM" #xFBA0)
    ("ARABIC LETTER RNOON FINAL FORM" #xFBA1)
    ("ARABIC LETTER RNOON INITIAL FORM" #xFBA2)
    ("ARABIC LETTER RNOON MEDIAL FORM" #xFBA3)
    ("ARABIC LETTER HEH WITH YEH ABOVE ISOLATED FORM" #xFBA4)
    ("ARABIC LETTER HEH WITH YEH ABOVE FINAL FORM" #xFBA5)
    ("ARABIC LETTER HEH GOAL ISOLATED FORM" #xFBA6)
    ("ARABIC LETTER HEH GOAL FINAL FORM" #xFBA7)
    ("ARABIC LETTER HEH GOAL INITIAL FORM" #xFBA8)
    ("ARABIC LETTER HEH GOAL MEDIAL FORM" #xFBA9)
    ("ARABIC LETTER HEH DOACHASHMEE ISOLATED FORM" #xFBAA)
    ("ARABIC LETTER HEH DOACHASHMEE FINAL FORM" #xFBAB)
    ("ARABIC LETTER HEH DOACHASHMEE INITIAL FORM" #xFBAC)
    ("ARABIC LETTER HEH DOACHASHMEE MEDIAL FORM" #xFBAD)
    ("ARABIC LETTER YEH BARREE ISOLATED FORM" #xFBAE)
    ("ARABIC LETTER YEH BARREE FINAL FORM" #xFBAF)
    ("ARABIC LETTER YEH BARREE WITH HAMZA ABOVE ISOLATED FORM" #xFBB0)
    ("ARABIC LETTER YEH BARREE WITH HAMZA ABOVE FINAL FORM" #xFBB1)
    ("ARABIC LETTER NG ISOLATED FORM" #xFBD3)
    ("ARABIC LETTER NG FINAL FORM" #xFBD4)
    ("ARABIC LETTER NG INITIAL FORM" #xFBD5)
    ("ARABIC LETTER NG MEDIAL FORM" #xFBD6)
    ("ARABIC LETTER U ISOLATED FORM" #xFBD7)
    ("ARABIC LETTER U FINAL FORM" #xFBD8)
    ("ARABIC LETTER OE ISOLATED FORM" #xFBD9)
    ("ARABIC LETTER OE FINAL FORM" #xFBDA)
    ("ARABIC LETTER YU ISOLATED FORM" #xFBDB)
    ("ARABIC LETTER YU FINAL FORM" #xFBDC)
    ("ARABIC LETTER U WITH HAMZA ABOVE ISOLATED FORM" #xFBDD)
    ("ARABIC LETTER VE ISOLATED FORM" #xFBDE)
    ("ARABIC LETTER VE FINAL FORM" #xFBDF)
    ("ARABIC LETTER KIRGHIZ OE ISOLATED FORM" #xFBE0)
    ("ARABIC LETTER KIRGHIZ OE FINAL FORM" #xFBE1)
    ("ARABIC LETTER KIRGHIZ YU ISOLATED FORM" #xFBE2)
    ("ARABIC LETTER KIRGHIZ YU FINAL FORM" #xFBE3)
    ("ARABIC LETTER E ISOLATED FORM" #xFBE4)
    ("ARABIC LETTER E FINAL FORM" #xFBE5)
    ("ARABIC LETTER E INITIAL FORM" #xFBE6)
    ("ARABIC LETTER E MEDIAL FORM" #xFBE7)
    ("ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA INITIAL FORM" #xFBE8)
    ("ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA MEDIAL FORM" #xFBE9)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF ISOLATED FORM" #xFBEA)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF FINAL FORM" #xFBEB)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE ISOLATED FORM" #xFBEC)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE FINAL FORM" #xFBED)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW ISOLATED FORM" #xFBEE)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW FINAL FORM" #xFBEF)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U ISOLATED FORM" #xFBF0)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U FINAL FORM" #xFBF1)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE ISOLATED FORM" #xFBF2)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE FINAL FORM" #xFBF3)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU ISOLATED FORM" #xFBF4)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU FINAL FORM" #xFBF5)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E ISOLATED FORM" #xFBF6)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E FINAL FORM" #xFBF7)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E INITIAL FORM" #xFBF8)
    ("ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM" #xFBF9)
    ("ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM" #xFBFA)
    ("ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA INITIAL FORM" #xFBFB)
    ("ARABIC LETTER FARSI YEH ISOLATED FORM" #xFBFC)
    ("ARABIC LETTER FARSI YEH FINAL FORM" #xFBFD)
    ("ARABIC LETTER FARSI YEH INITIAL FORM" #xFBFE)
    ("ARABIC LETTER FARSI YEH MEDIAL FORM" #xFBFF)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM ISOLATED FORM" #xFC00)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH ISOLATED FORM" #xFC01)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM ISOLATED FORM" #xFC02)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM" #xFC03)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH ISOLATED FORM" #xFC04)
    ("ARABIC LIGATURE BEH WITH JEEM ISOLATED FORM" #xFC05)
    ("ARABIC LIGATURE BEH WITH HAH ISOLATED FORM" #xFC06)
    ("ARABIC LIGATURE BEH WITH KHAH ISOLATED FORM" #xFC07)
    ("ARABIC LIGATURE BEH WITH MEEM ISOLATED FORM" #xFC08)
    ("ARABIC LIGATURE BEH WITH ALEF MAKSURA ISOLATED FORM" #xFC09)
    ("ARABIC LIGATURE BEH WITH YEH ISOLATED FORM" #xFC0A)
    ("ARABIC LIGATURE TEH WITH JEEM ISOLATED FORM" #xFC0B)
    ("ARABIC LIGATURE TEH WITH HAH ISOLATED FORM" #xFC0C)
    ("ARABIC LIGATURE TEH WITH KHAH ISOLATED FORM" #xFC0D)
    ("ARABIC LIGATURE TEH WITH MEEM ISOLATED FORM" #xFC0E)
    ("ARABIC LIGATURE TEH WITH ALEF MAKSURA ISOLATED FORM" #xFC0F)
    ("ARABIC LIGATURE TEH WITH YEH ISOLATED FORM" #xFC10)
    ("ARABIC LIGATURE THEH WITH JEEM ISOLATED FORM" #xFC11)
    ("ARABIC LIGATURE THEH WITH MEEM ISOLATED FORM" #xFC12)
    ("ARABIC LIGATURE THEH WITH ALEF MAKSURA ISOLATED FORM" #xFC13)
    ("ARABIC LIGATURE THEH WITH YEH ISOLATED FORM" #xFC14)
    ("ARABIC LIGATURE JEEM WITH HAH ISOLATED FORM" #xFC15)
    ("ARABIC LIGATURE JEEM WITH MEEM ISOLATED FORM" #xFC16)
    ("ARABIC LIGATURE HAH WITH JEEM ISOLATED FORM" #xFC17)
    ("ARABIC LIGATURE HAH WITH MEEM ISOLATED FORM" #xFC18)
    ("ARABIC LIGATURE KHAH WITH JEEM ISOLATED FORM" #xFC19)
    ("ARABIC LIGATURE KHAH WITH HAH ISOLATED FORM" #xFC1A)
    ("ARABIC LIGATURE KHAH WITH MEEM ISOLATED FORM" #xFC1B)
    ("ARABIC LIGATURE SEEN WITH JEEM ISOLATED FORM" #xFC1C)
    ("ARABIC LIGATURE SEEN WITH HAH ISOLATED FORM" #xFC1D)
    ("ARABIC LIGATURE SEEN WITH KHAH ISOLATED FORM" #xFC1E)
    ("ARABIC LIGATURE SEEN WITH MEEM ISOLATED FORM" #xFC1F)
    ("ARABIC LIGATURE SAD WITH HAH ISOLATED FORM" #xFC20)
    ("ARABIC LIGATURE SAD WITH MEEM ISOLATED FORM" #xFC21)
    ("ARABIC LIGATURE DAD WITH JEEM ISOLATED FORM" #xFC22)
    ("ARABIC LIGATURE DAD WITH HAH ISOLATED FORM" #xFC23)
    ("ARABIC LIGATURE DAD WITH KHAH ISOLATED FORM" #xFC24)
    ("ARABIC LIGATURE DAD WITH MEEM ISOLATED FORM" #xFC25)
    ("ARABIC LIGATURE TAH WITH HAH ISOLATED FORM" #xFC26)
    ("ARABIC LIGATURE TAH WITH MEEM ISOLATED FORM" #xFC27)
    ("ARABIC LIGATURE ZAH WITH MEEM ISOLATED FORM" #xFC28)
    ("ARABIC LIGATURE AIN WITH JEEM ISOLATED FORM" #xFC29)
    ("ARABIC LIGATURE AIN WITH MEEM ISOLATED FORM" #xFC2A)
    ("ARABIC LIGATURE GHAIN WITH JEEM ISOLATED FORM" #xFC2B)
    ("ARABIC LIGATURE GHAIN WITH MEEM ISOLATED FORM" #xFC2C)
    ("ARABIC LIGATURE FEH WITH JEEM ISOLATED FORM" #xFC2D)
    ("ARABIC LIGATURE FEH WITH HAH ISOLATED FORM" #xFC2E)
    ("ARABIC LIGATURE FEH WITH KHAH ISOLATED FORM" #xFC2F)
    ("ARABIC LIGATURE FEH WITH MEEM ISOLATED FORM" #xFC30)
    ("ARABIC LIGATURE FEH WITH ALEF MAKSURA ISOLATED FORM" #xFC31)
    ("ARABIC LIGATURE FEH WITH YEH ISOLATED FORM" #xFC32)
    ("ARABIC LIGATURE QAF WITH HAH ISOLATED FORM" #xFC33)
    ("ARABIC LIGATURE QAF WITH MEEM ISOLATED FORM" #xFC34)
    ("ARABIC LIGATURE QAF WITH ALEF MAKSURA ISOLATED FORM" #xFC35)
    ("ARABIC LIGATURE QAF WITH YEH ISOLATED FORM" #xFC36)
    ("ARABIC LIGATURE KAF WITH ALEF ISOLATED FORM" #xFC37)
    ("ARABIC LIGATURE KAF WITH JEEM ISOLATED FORM" #xFC38)
    ("ARABIC LIGATURE KAF WITH HAH ISOLATED FORM" #xFC39)
    ("ARABIC LIGATURE KAF WITH KHAH ISOLATED FORM" #xFC3A)
    ("ARABIC LIGATURE KAF WITH LAM ISOLATED FORM" #xFC3B)
    ("ARABIC LIGATURE KAF WITH MEEM ISOLATED FORM" #xFC3C)
    ("ARABIC LIGATURE KAF WITH ALEF MAKSURA ISOLATED FORM" #xFC3D)
    ("ARABIC LIGATURE KAF WITH YEH ISOLATED FORM" #xFC3E)
    ("ARABIC LIGATURE LAM WITH JEEM ISOLATED FORM" #xFC3F)
    ("ARABIC LIGATURE LAM WITH HAH ISOLATED FORM" #xFC40)
    ("ARABIC LIGATURE LAM WITH KHAH ISOLATED FORM" #xFC41)
    ("ARABIC LIGATURE LAM WITH MEEM ISOLATED FORM" #xFC42)
    ("ARABIC LIGATURE LAM WITH ALEF MAKSURA ISOLATED FORM" #xFC43)
    ("ARABIC LIGATURE LAM WITH YEH ISOLATED FORM" #xFC44)
    ("ARABIC LIGATURE MEEM WITH JEEM ISOLATED FORM" #xFC45)
    ("ARABIC LIGATURE MEEM WITH HAH ISOLATED FORM" #xFC46)
    ("ARABIC LIGATURE MEEM WITH KHAH ISOLATED FORM" #xFC47)
    ("ARABIC LIGATURE MEEM WITH MEEM ISOLATED FORM" #xFC48)
    ("ARABIC LIGATURE MEEM WITH ALEF MAKSURA ISOLATED FORM" #xFC49)
    ("ARABIC LIGATURE MEEM WITH YEH ISOLATED FORM" #xFC4A)
    ("ARABIC LIGATURE NOON WITH JEEM ISOLATED FORM" #xFC4B)
    ("ARABIC LIGATURE NOON WITH HAH ISOLATED FORM" #xFC4C)
    ("ARABIC LIGATURE NOON WITH KHAH ISOLATED FORM" #xFC4D)
    ("ARABIC LIGATURE NOON WITH MEEM ISOLATED FORM" #xFC4E)
    ("ARABIC LIGATURE NOON WITH ALEF MAKSURA ISOLATED FORM" #xFC4F)
    ("ARABIC LIGATURE NOON WITH YEH ISOLATED FORM" #xFC50)
    ("ARABIC LIGATURE HEH WITH JEEM ISOLATED FORM" #xFC51)
    ("ARABIC LIGATURE HEH WITH MEEM ISOLATED FORM" #xFC52)
    ("ARABIC LIGATURE HEH WITH ALEF MAKSURA ISOLATED FORM" #xFC53)
    ("ARABIC LIGATURE HEH WITH YEH ISOLATED FORM" #xFC54)
    ("ARABIC LIGATURE YEH WITH JEEM ISOLATED FORM" #xFC55)
    ("ARABIC LIGATURE YEH WITH HAH ISOLATED FORM" #xFC56)
    ("ARABIC LIGATURE YEH WITH KHAH ISOLATED FORM" #xFC57)
    ("ARABIC LIGATURE YEH WITH MEEM ISOLATED FORM" #xFC58)
    ("ARABIC LIGATURE YEH WITH ALEF MAKSURA ISOLATED FORM" #xFC59)
    ("ARABIC LIGATURE YEH WITH YEH ISOLATED FORM" #xFC5A)
    ("ARABIC LIGATURE THAL WITH SUPERSCRIPT ALEF ISOLATED FORM" #xFC5B)
    ("ARABIC LIGATURE REH WITH SUPERSCRIPT ALEF ISOLATED FORM" #xFC5C)
    ("ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF ISOLATED FORM" #xFC5D)
    ("ARABIC LIGATURE SHADDA WITH DAMMATAN ISOLATED FORM" #xFC5E)
    ("ARABIC LIGATURE SHADDA WITH KASRATAN ISOLATED FORM" #xFC5F)
    ("ARABIC LIGATURE SHADDA WITH FATHA ISOLATED FORM" #xFC60)
    ("ARABIC LIGATURE SHADDA WITH DAMMA ISOLATED FORM" #xFC61)
    ("ARABIC LIGATURE SHADDA WITH KASRA ISOLATED FORM" #xFC62)
    ("ARABIC LIGATURE SHADDA WITH SUPERSCRIPT ALEF ISOLATED FORM" #xFC63)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH REH FINAL FORM" #xFC64)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ZAIN FINAL FORM" #xFC65)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM FINAL FORM" #xFC66)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH NOON FINAL FORM" #xFC67)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM" #xFC68)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH FINAL FORM" #xFC69)
    ("ARABIC LIGATURE BEH WITH REH FINAL FORM" #xFC6A)
    ("ARABIC LIGATURE BEH WITH ZAIN FINAL FORM" #xFC6B)
    ("ARABIC LIGATURE BEH WITH MEEM FINAL FORM" #xFC6C)
    ("ARABIC LIGATURE BEH WITH NOON FINAL FORM" #xFC6D)
    ("ARABIC LIGATURE BEH WITH ALEF MAKSURA FINAL FORM" #xFC6E)
    ("ARABIC LIGATURE BEH WITH YEH FINAL FORM" #xFC6F)
    ("ARABIC LIGATURE TEH WITH REH FINAL FORM" #xFC70)
    ("ARABIC LIGATURE TEH WITH ZAIN FINAL FORM" #xFC71)
    ("ARABIC LIGATURE TEH WITH MEEM FINAL FORM" #xFC72)
    ("ARABIC LIGATURE TEH WITH NOON FINAL FORM" #xFC73)
    ("ARABIC LIGATURE TEH WITH ALEF MAKSURA FINAL FORM" #xFC74)
    ("ARABIC LIGATURE TEH WITH YEH FINAL FORM" #xFC75)
    ("ARABIC LIGATURE THEH WITH REH FINAL FORM" #xFC76)
    ("ARABIC LIGATURE THEH WITH ZAIN FINAL FORM" #xFC77)
    ("ARABIC LIGATURE THEH WITH MEEM FINAL FORM" #xFC78)
    ("ARABIC LIGATURE THEH WITH NOON FINAL FORM" #xFC79)
    ("ARABIC LIGATURE THEH WITH ALEF MAKSURA FINAL FORM" #xFC7A)
    ("ARABIC LIGATURE THEH WITH YEH FINAL FORM" #xFC7B)
    ("ARABIC LIGATURE FEH WITH ALEF MAKSURA FINAL FORM" #xFC7C)
    ("ARABIC LIGATURE FEH WITH YEH FINAL FORM" #xFC7D)
    ("ARABIC LIGATURE QAF WITH ALEF MAKSURA FINAL FORM" #xFC7E)
    ("ARABIC LIGATURE QAF WITH YEH FINAL FORM" #xFC7F)
    ("ARABIC LIGATURE KAF WITH ALEF FINAL FORM" #xFC80)
    ("ARABIC LIGATURE KAF WITH LAM FINAL FORM" #xFC81)
    ("ARABIC LIGATURE KAF WITH MEEM FINAL FORM" #xFC82)
    ("ARABIC LIGATURE KAF WITH ALEF MAKSURA FINAL FORM" #xFC83)
    ("ARABIC LIGATURE KAF WITH YEH FINAL FORM" #xFC84)
    ("ARABIC LIGATURE LAM WITH MEEM FINAL FORM" #xFC85)
    ("ARABIC LIGATURE LAM WITH ALEF MAKSURA FINAL FORM" #xFC86)
    ("ARABIC LIGATURE LAM WITH YEH FINAL FORM" #xFC87)
    ("ARABIC LIGATURE MEEM WITH ALEF FINAL FORM" #xFC88)
    ("ARABIC LIGATURE MEEM WITH MEEM FINAL FORM" #xFC89)
    ("ARABIC LIGATURE NOON WITH REH FINAL FORM" #xFC8A)
    ("ARABIC LIGATURE NOON WITH ZAIN FINAL FORM" #xFC8B)
    ("ARABIC LIGATURE NOON WITH MEEM FINAL FORM" #xFC8C)
    ("ARABIC LIGATURE NOON WITH NOON FINAL FORM" #xFC8D)
    ("ARABIC LIGATURE NOON WITH ALEF MAKSURA FINAL FORM" #xFC8E)
    ("ARABIC LIGATURE NOON WITH YEH FINAL FORM" #xFC8F)
    ("ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF FINAL FORM" #xFC90)
    ("ARABIC LIGATURE YEH WITH REH FINAL FORM" #xFC91)
    ("ARABIC LIGATURE YEH WITH ZAIN FINAL FORM" #xFC92)
    ("ARABIC LIGATURE YEH WITH MEEM FINAL FORM" #xFC93)
    ("ARABIC LIGATURE YEH WITH NOON FINAL FORM" #xFC94)
    ("ARABIC LIGATURE YEH WITH ALEF MAKSURA FINAL FORM" #xFC95)
    ("ARABIC LIGATURE YEH WITH YEH FINAL FORM" #xFC96)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM INITIAL FORM" #xFC97)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH INITIAL FORM" #xFC98)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH KHAH INITIAL FORM" #xFC99)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM INITIAL FORM" #xFC9A)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH INITIAL FORM" #xFC9B)
    ("ARABIC LIGATURE BEH WITH JEEM INITIAL FORM" #xFC9C)
    ("ARABIC LIGATURE BEH WITH HAH INITIAL FORM" #xFC9D)
    ("ARABIC LIGATURE BEH WITH KHAH INITIAL FORM" #xFC9E)
    ("ARABIC LIGATURE BEH WITH MEEM INITIAL FORM" #xFC9F)
    ("ARABIC LIGATURE BEH WITH HEH INITIAL FORM" #xFCA0)
    ("ARABIC LIGATURE TEH WITH JEEM INITIAL FORM" #xFCA1)
    ("ARABIC LIGATURE TEH WITH HAH INITIAL FORM" #xFCA2)
    ("ARABIC LIGATURE TEH WITH KHAH INITIAL FORM" #xFCA3)
    ("ARABIC LIGATURE TEH WITH MEEM INITIAL FORM" #xFCA4)
    ("ARABIC LIGATURE TEH WITH HEH INITIAL FORM" #xFCA5)
    ("ARABIC LIGATURE THEH WITH MEEM INITIAL FORM" #xFCA6)
    ("ARABIC LIGATURE JEEM WITH HAH INITIAL FORM" #xFCA7)
    ("ARABIC LIGATURE JEEM WITH MEEM INITIAL FORM" #xFCA8)
    ("ARABIC LIGATURE HAH WITH JEEM INITIAL FORM" #xFCA9)
    ("ARABIC LIGATURE HAH WITH MEEM INITIAL FORM" #xFCAA)
    ("ARABIC LIGATURE KHAH WITH JEEM INITIAL FORM" #xFCAB)
    ("ARABIC LIGATURE KHAH WITH MEEM INITIAL FORM" #xFCAC)
    ("ARABIC LIGATURE SEEN WITH JEEM INITIAL FORM" #xFCAD)
    ("ARABIC LIGATURE SEEN WITH HAH INITIAL FORM" #xFCAE)
    ("ARABIC LIGATURE SEEN WITH KHAH INITIAL FORM" #xFCAF)
    ("ARABIC LIGATURE SEEN WITH MEEM INITIAL FORM" #xFCB0)
    ("ARABIC LIGATURE SAD WITH HAH INITIAL FORM" #xFCB1)
    ("ARABIC LIGATURE SAD WITH KHAH INITIAL FORM" #xFCB2)
    ("ARABIC LIGATURE SAD WITH MEEM INITIAL FORM" #xFCB3)
    ("ARABIC LIGATURE DAD WITH JEEM INITIAL FORM" #xFCB4)
    ("ARABIC LIGATURE DAD WITH HAH INITIAL FORM" #xFCB5)
    ("ARABIC LIGATURE DAD WITH KHAH INITIAL FORM" #xFCB6)
    ("ARABIC LIGATURE DAD WITH MEEM INITIAL FORM" #xFCB7)
    ("ARABIC LIGATURE TAH WITH HAH INITIAL FORM" #xFCB8)
    ("ARABIC LIGATURE ZAH WITH MEEM INITIAL FORM" #xFCB9)
    ("ARABIC LIGATURE AIN WITH JEEM INITIAL FORM" #xFCBA)
    ("ARABIC LIGATURE AIN WITH MEEM INITIAL FORM" #xFCBB)
    ("ARABIC LIGATURE GHAIN WITH JEEM INITIAL FORM" #xFCBC)
    ("ARABIC LIGATURE GHAIN WITH MEEM INITIAL FORM" #xFCBD)
    ("ARABIC LIGATURE FEH WITH JEEM INITIAL FORM" #xFCBE)
    ("ARABIC LIGATURE FEH WITH HAH INITIAL FORM" #xFCBF)
    ("ARABIC LIGATURE FEH WITH KHAH INITIAL FORM" #xFCC0)
    ("ARABIC LIGATURE FEH WITH MEEM INITIAL FORM" #xFCC1)
    ("ARABIC LIGATURE QAF WITH HAH INITIAL FORM" #xFCC2)
    ("ARABIC LIGATURE QAF WITH MEEM INITIAL FORM" #xFCC3)
    ("ARABIC LIGATURE KAF WITH JEEM INITIAL FORM" #xFCC4)
    ("ARABIC LIGATURE KAF WITH HAH INITIAL FORM" #xFCC5)
    ("ARABIC LIGATURE KAF WITH KHAH INITIAL FORM" #xFCC6)
    ("ARABIC LIGATURE KAF WITH LAM INITIAL FORM" #xFCC7)
    ("ARABIC LIGATURE KAF WITH MEEM INITIAL FORM" #xFCC8)
    ("ARABIC LIGATURE LAM WITH JEEM INITIAL FORM" #xFCC9)
    ("ARABIC LIGATURE LAM WITH HAH INITIAL FORM" #xFCCA)
    ("ARABIC LIGATURE LAM WITH KHAH INITIAL FORM" #xFCCB)
    ("ARABIC LIGATURE LAM WITH MEEM INITIAL FORM" #xFCCC)
    ("ARABIC LIGATURE LAM WITH HEH INITIAL FORM" #xFCCD)
    ("ARABIC LIGATURE MEEM WITH JEEM INITIAL FORM" #xFCCE)
    ("ARABIC LIGATURE MEEM WITH HAH INITIAL FORM" #xFCCF)
    ("ARABIC LIGATURE MEEM WITH KHAH INITIAL FORM" #xFCD0)
    ("ARABIC LIGATURE MEEM WITH MEEM INITIAL FORM" #xFCD1)
    ("ARABIC LIGATURE NOON WITH JEEM INITIAL FORM" #xFCD2)
    ("ARABIC LIGATURE NOON WITH HAH INITIAL FORM" #xFCD3)
    ("ARABIC LIGATURE NOON WITH KHAH INITIAL FORM" #xFCD4)
    ("ARABIC LIGATURE NOON WITH MEEM INITIAL FORM" #xFCD5)
    ("ARABIC LIGATURE NOON WITH HEH INITIAL FORM" #xFCD6)
    ("ARABIC LIGATURE HEH WITH JEEM INITIAL FORM" #xFCD7)
    ("ARABIC LIGATURE HEH WITH MEEM INITIAL FORM" #xFCD8)
    ("ARABIC LIGATURE HEH WITH SUPERSCRIPT ALEF INITIAL FORM" #xFCD9)
    ("ARABIC LIGATURE YEH WITH JEEM INITIAL FORM" #xFCDA)
    ("ARABIC LIGATURE YEH WITH HAH INITIAL FORM" #xFCDB)
    ("ARABIC LIGATURE YEH WITH KHAH INITIAL FORM" #xFCDC)
    ("ARABIC LIGATURE YEH WITH MEEM INITIAL FORM" #xFCDD)
    ("ARABIC LIGATURE YEH WITH HEH INITIAL FORM" #xFCDE)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM MEDIAL FORM" #xFCDF)
    ("ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH MEDIAL FORM" #xFCE0)
    ("ARABIC LIGATURE BEH WITH MEEM MEDIAL FORM" #xFCE1)
    ("ARABIC LIGATURE BEH WITH HEH MEDIAL FORM" #xFCE2)
    ("ARABIC LIGATURE TEH WITH MEEM MEDIAL FORM" #xFCE3)
    ("ARABIC LIGATURE TEH WITH HEH MEDIAL FORM" #xFCE4)
    ("ARABIC LIGATURE THEH WITH MEEM MEDIAL FORM" #xFCE5)
    ("ARABIC LIGATURE THEH WITH HEH MEDIAL FORM" #xFCE6)
    ("ARABIC LIGATURE SEEN WITH MEEM MEDIAL FORM" #xFCE7)
    ("ARABIC LIGATURE SEEN WITH HEH MEDIAL FORM" #xFCE8)
    ("ARABIC LIGATURE SHEEN WITH MEEM MEDIAL FORM" #xFCE9)
    ("ARABIC LIGATURE SHEEN WITH HEH MEDIAL FORM" #xFCEA)
    ("ARABIC LIGATURE KAF WITH LAM MEDIAL FORM" #xFCEB)
    ("ARABIC LIGATURE KAF WITH MEEM MEDIAL FORM" #xFCEC)
    ("ARABIC LIGATURE LAM WITH MEEM MEDIAL FORM" #xFCED)
    ("ARABIC LIGATURE NOON WITH MEEM MEDIAL FORM" #xFCEE)
    ("ARABIC LIGATURE NOON WITH HEH MEDIAL FORM" #xFCEF)
    ("ARABIC LIGATURE YEH WITH MEEM MEDIAL FORM" #xFCF0)
    ("ARABIC LIGATURE YEH WITH HEH MEDIAL FORM" #xFCF1)
    ("ARABIC LIGATURE SHADDA WITH FATHA MEDIAL FORM" #xFCF2)
    ("ARABIC LIGATURE SHADDA WITH DAMMA MEDIAL FORM" #xFCF3)
    ("ARABIC LIGATURE SHADDA WITH KASRA MEDIAL FORM" #xFCF4)
    ("ARABIC LIGATURE TAH WITH ALEF MAKSURA ISOLATED FORM" #xFCF5)
    ("ARABIC LIGATURE TAH WITH YEH ISOLATED FORM" #xFCF6)
    ("ARABIC LIGATURE AIN WITH ALEF MAKSURA ISOLATED FORM" #xFCF7)
    ("ARABIC LIGATURE AIN WITH YEH ISOLATED FORM" #xFCF8)
    ("ARABIC LIGATURE GHAIN WITH ALEF MAKSURA ISOLATED FORM" #xFCF9)
    ("ARABIC LIGATURE GHAIN WITH YEH ISOLATED FORM" #xFCFA)
    ("ARABIC LIGATURE SEEN WITH ALEF MAKSURA ISOLATED FORM" #xFCFB)
    ("ARABIC LIGATURE SEEN WITH YEH ISOLATED FORM" #xFCFC)
    ("ARABIC LIGATURE SHEEN WITH ALEF MAKSURA ISOLATED FORM" #xFCFD)
    ("ARABIC LIGATURE SHEEN WITH YEH ISOLATED FORM" #xFCFE)
    ("ARABIC LIGATURE HAH WITH ALEF MAKSURA ISOLATED FORM" #xFCFF)
    ("ARABIC LIGATURE HAH WITH YEH ISOLATED FORM" #xFD00)
    ("ARABIC LIGATURE JEEM WITH ALEF MAKSURA ISOLATED FORM" #xFD01)
    ("ARABIC LIGATURE JEEM WITH YEH ISOLATED FORM" #xFD02)
    ("ARABIC LIGATURE KHAH WITH ALEF MAKSURA ISOLATED FORM" #xFD03)
    ("ARABIC LIGATURE KHAH WITH YEH ISOLATED FORM" #xFD04)
    ("ARABIC LIGATURE SAD WITH ALEF MAKSURA ISOLATED FORM" #xFD05)
    ("ARABIC LIGATURE SAD WITH YEH ISOLATED FORM" #xFD06)
    ("ARABIC LIGATURE DAD WITH ALEF MAKSURA ISOLATED FORM" #xFD07)
    ("ARABIC LIGATURE DAD WITH YEH ISOLATED FORM" #xFD08)
    ("ARABIC LIGATURE SHEEN WITH JEEM ISOLATED FORM" #xFD09)
    ("ARABIC LIGATURE SHEEN WITH HAH ISOLATED FORM" #xFD0A)
    ("ARABIC LIGATURE SHEEN WITH KHAH ISOLATED FORM" #xFD0B)
    ("ARABIC LIGATURE SHEEN WITH MEEM ISOLATED FORM" #xFD0C)
    ("ARABIC LIGATURE SHEEN WITH REH ISOLATED FORM" #xFD0D)
    ("ARABIC LIGATURE SEEN WITH REH ISOLATED FORM" #xFD0E)
    ("ARABIC LIGATURE SAD WITH REH ISOLATED FORM" #xFD0F)
    ("ARABIC LIGATURE DAD WITH REH ISOLATED FORM" #xFD10)
    ("ARABIC LIGATURE TAH WITH ALEF MAKSURA FINAL FORM" #xFD11)
    ("ARABIC LIGATURE TAH WITH YEH FINAL FORM" #xFD12)
    ("ARABIC LIGATURE AIN WITH ALEF MAKSURA FINAL FORM" #xFD13)
    ("ARABIC LIGATURE AIN WITH YEH FINAL FORM" #xFD14)
    ("ARABIC LIGATURE GHAIN WITH ALEF MAKSURA FINAL FORM" #xFD15)
    ("ARABIC LIGATURE GHAIN WITH YEH FINAL FORM" #xFD16)
    ("ARABIC LIGATURE SEEN WITH ALEF MAKSURA FINAL FORM" #xFD17)
    ("ARABIC LIGATURE SEEN WITH YEH FINAL FORM" #xFD18)
    ("ARABIC LIGATURE SHEEN WITH ALEF MAKSURA FINAL FORM" #xFD19)
    ("ARABIC LIGATURE SHEEN WITH YEH FINAL FORM" #xFD1A)
    ("ARABIC LIGATURE HAH WITH ALEF MAKSURA FINAL FORM" #xFD1B)
    ("ARABIC LIGATURE HAH WITH YEH FINAL FORM" #xFD1C)
    ("ARABIC LIGATURE JEEM WITH ALEF MAKSURA FINAL FORM" #xFD1D)
    ("ARABIC LIGATURE JEEM WITH YEH FINAL FORM" #xFD1E)
    ("ARABIC LIGATURE KHAH WITH ALEF MAKSURA FINAL FORM" #xFD1F)
    ("ARABIC LIGATURE KHAH WITH YEH FINAL FORM" #xFD20)
    ("ARABIC LIGATURE SAD WITH ALEF MAKSURA FINAL FORM" #xFD21)
    ("ARABIC LIGATURE SAD WITH YEH FINAL FORM" #xFD22)
    ("ARABIC LIGATURE DAD WITH ALEF MAKSURA FINAL FORM" #xFD23)
    ("ARABIC LIGATURE DAD WITH YEH FINAL FORM" #xFD24)
    ("ARABIC LIGATURE SHEEN WITH JEEM FINAL FORM" #xFD25)
    ("ARABIC LIGATURE SHEEN WITH HAH FINAL FORM" #xFD26)
    ("ARABIC LIGATURE SHEEN WITH KHAH FINAL FORM" #xFD27)
    ("ARABIC LIGATURE SHEEN WITH MEEM FINAL FORM" #xFD28)
    ("ARABIC LIGATURE SHEEN WITH REH FINAL FORM" #xFD29)
    ("ARABIC LIGATURE SEEN WITH REH FINAL FORM" #xFD2A)
    ("ARABIC LIGATURE SAD WITH REH FINAL FORM" #xFD2B)
    ("ARABIC LIGATURE DAD WITH REH FINAL FORM" #xFD2C)
    ("ARABIC LIGATURE SHEEN WITH JEEM INITIAL FORM" #xFD2D)
    ("ARABIC LIGATURE SHEEN WITH HAH INITIAL FORM" #xFD2E)
    ("ARABIC LIGATURE SHEEN WITH KHAH INITIAL FORM" #xFD2F)
    ("ARABIC LIGATURE SHEEN WITH MEEM INITIAL FORM" #xFD30)
    ("ARABIC LIGATURE SEEN WITH HEH INITIAL FORM" #xFD31)
    ("ARABIC LIGATURE SHEEN WITH HEH INITIAL FORM" #xFD32)
    ("ARABIC LIGATURE TAH WITH MEEM INITIAL FORM" #xFD33)
    ("ARABIC LIGATURE SEEN WITH JEEM MEDIAL FORM" #xFD34)
    ("ARABIC LIGATURE SEEN WITH HAH MEDIAL FORM" #xFD35)
    ("ARABIC LIGATURE SEEN WITH KHAH MEDIAL FORM" #xFD36)
    ("ARABIC LIGATURE SHEEN WITH JEEM MEDIAL FORM" #xFD37)
    ("ARABIC LIGATURE SHEEN WITH HAH MEDIAL FORM" #xFD38)
    ("ARABIC LIGATURE SHEEN WITH KHAH MEDIAL FORM" #xFD39)
    ("ARABIC LIGATURE TAH WITH MEEM MEDIAL FORM" #xFD3A)
    ("ARABIC LIGATURE ZAH WITH MEEM MEDIAL FORM" #xFD3B)
    ("ARABIC LIGATURE ALEF WITH FATHATAN FINAL FORM" #xFD3C)
    ("ARABIC LIGATURE ALEF WITH FATHATAN ISOLATED FORM" #xFD3D)
    ("ORNATE LEFT PARENTHESIS" #xFD3E)
    ("ORNATE RIGHT PARENTHESIS" #xFD3F)
    ("ARABIC LIGATURE TEH WITH JEEM WITH MEEM INITIAL FORM" #xFD50)
    ("ARABIC LIGATURE TEH WITH HAH WITH JEEM FINAL FORM" #xFD51)
    ("ARABIC LIGATURE TEH WITH HAH WITH JEEM INITIAL FORM" #xFD52)
    ("ARABIC LIGATURE TEH WITH HAH WITH MEEM INITIAL FORM" #xFD53)
    ("ARABIC LIGATURE TEH WITH KHAH WITH MEEM INITIAL FORM" #xFD54)
    ("ARABIC LIGATURE TEH WITH MEEM WITH JEEM INITIAL FORM" #xFD55)
    ("ARABIC LIGATURE TEH WITH MEEM WITH HAH INITIAL FORM" #xFD56)
    ("ARABIC LIGATURE TEH WITH MEEM WITH KHAH INITIAL FORM" #xFD57)
    ("ARABIC LIGATURE JEEM WITH MEEM WITH HAH FINAL FORM" #xFD58)
    ("ARABIC LIGATURE JEEM WITH MEEM WITH HAH INITIAL FORM" #xFD59)
    ("ARABIC LIGATURE HAH WITH MEEM WITH YEH FINAL FORM" #xFD5A)
    ("ARABIC LIGATURE HAH WITH MEEM WITH ALEF MAKSURA FINAL FORM" #xFD5B)
    ("ARABIC LIGATURE SEEN WITH HAH WITH JEEM INITIAL FORM" #xFD5C)
    ("ARABIC LIGATURE SEEN WITH JEEM WITH HAH INITIAL FORM" #xFD5D)
    ("ARABIC LIGATURE SEEN WITH JEEM WITH ALEF MAKSURA FINAL FORM" #xFD5E)
    ("ARABIC LIGATURE SEEN WITH MEEM WITH HAH FINAL FORM" #xFD5F)
    ("ARABIC LIGATURE SEEN WITH MEEM WITH HAH INITIAL FORM" #xFD60)
    ("ARABIC LIGATURE SEEN WITH MEEM WITH JEEM INITIAL FORM" #xFD61)
    ("ARABIC LIGATURE SEEN WITH MEEM WITH MEEM FINAL FORM" #xFD62)
    ("ARABIC LIGATURE SEEN WITH MEEM WITH MEEM INITIAL FORM" #xFD63)
    ("ARABIC LIGATURE SAD WITH HAH WITH HAH FINAL FORM" #xFD64)
    ("ARABIC LIGATURE SAD WITH HAH WITH HAH INITIAL FORM" #xFD65)
    ("ARABIC LIGATURE SAD WITH MEEM WITH MEEM FINAL FORM" #xFD66)
    ("ARABIC LIGATURE SHEEN WITH HAH WITH MEEM FINAL FORM" #xFD67)
    ("ARABIC LIGATURE SHEEN WITH HAH WITH MEEM INITIAL FORM" #xFD68)
    ("ARABIC LIGATURE SHEEN WITH JEEM WITH YEH FINAL FORM" #xFD69)
    ("ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH FINAL FORM" #xFD6A)
    ("ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH INITIAL FORM" #xFD6B)
    ("ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM FINAL FORM" #xFD6C)
    ("ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM INITIAL FORM" #xFD6D)
    ("ARABIC LIGATURE DAD WITH HAH WITH ALEF MAKSURA FINAL FORM" #xFD6E)
    ("ARABIC LIGATURE DAD WITH KHAH WITH MEEM FINAL FORM" #xFD6F)
    ("ARABIC LIGATURE DAD WITH KHAH WITH MEEM INITIAL FORM" #xFD70)
    ("ARABIC LIGATURE TAH WITH MEEM WITH HAH FINAL FORM" #xFD71)
    ("ARABIC LIGATURE TAH WITH MEEM WITH HAH INITIAL FORM" #xFD72)
    ("ARABIC LIGATURE TAH WITH MEEM WITH MEEM INITIAL FORM" #xFD73)
    ("ARABIC LIGATURE TAH WITH MEEM WITH YEH FINAL FORM" #xFD74)
    ("ARABIC LIGATURE AIN WITH JEEM WITH MEEM FINAL FORM" #xFD75)
    ("ARABIC LIGATURE AIN WITH MEEM WITH MEEM FINAL FORM" #xFD76)
    ("ARABIC LIGATURE AIN WITH MEEM WITH MEEM INITIAL FORM" #xFD77)
    ("ARABIC LIGATURE AIN WITH MEEM WITH ALEF MAKSURA FINAL FORM" #xFD78)
    ("ARABIC LIGATURE GHAIN WITH MEEM WITH MEEM FINAL FORM" #xFD79)
    ("ARABIC LIGATURE GHAIN WITH MEEM WITH YEH FINAL FORM" #xFD7A)
    ("ARABIC LIGATURE GHAIN WITH MEEM WITH ALEF MAKSURA FINAL FORM" #xFD7B)
    ("ARABIC LIGATURE FEH WITH KHAH WITH MEEM FINAL FORM" #xFD7C)
    ("ARABIC LIGATURE FEH WITH KHAH WITH MEEM INITIAL FORM" #xFD7D)
    ("ARABIC LIGATURE QAF WITH MEEM WITH HAH FINAL FORM" #xFD7E)
    ("ARABIC LIGATURE QAF WITH MEEM WITH MEEM FINAL FORM" #xFD7F)
    ("ARABIC LIGATURE LAM WITH HAH WITH MEEM FINAL FORM" #xFD80)
    ("ARABIC LIGATURE LAM WITH HAH WITH YEH FINAL FORM" #xFD81)
    ("ARABIC LIGATURE LAM WITH HAH WITH ALEF MAKSURA FINAL FORM" #xFD82)
    ("ARABIC LIGATURE LAM WITH JEEM WITH JEEM INITIAL FORM" #xFD83)
    ("ARABIC LIGATURE LAM WITH JEEM WITH JEEM FINAL FORM" #xFD84)
    ("ARABIC LIGATURE LAM WITH KHAH WITH MEEM FINAL FORM" #xFD85)
    ("ARABIC LIGATURE LAM WITH KHAH WITH MEEM INITIAL FORM" #xFD86)
    ("ARABIC LIGATURE LAM WITH MEEM WITH HAH FINAL FORM" #xFD87)
    ("ARABIC LIGATURE LAM WITH MEEM WITH HAH INITIAL FORM" #xFD88)
    ("ARABIC LIGATURE MEEM WITH HAH WITH JEEM INITIAL FORM" #xFD89)
    ("ARABIC LIGATURE MEEM WITH HAH WITH MEEM INITIAL FORM" #xFD8A)
    ("ARABIC LIGATURE MEEM WITH HAH WITH YEH FINAL FORM" #xFD8B)
    ("ARABIC LIGATURE MEEM WITH JEEM WITH HAH INITIAL FORM" #xFD8C)
    ("ARABIC LIGATURE MEEM WITH JEEM WITH MEEM INITIAL FORM" #xFD8D)
    ("ARABIC LIGATURE MEEM WITH KHAH WITH JEEM INITIAL FORM" #xFD8E)
    ("ARABIC LIGATURE MEEM WITH KHAH WITH MEEM INITIAL FORM" #xFD8F)
    ("ARABIC LIGATURE MEEM WITH JEEM WITH KHAH INITIAL FORM" #xFD92)
    ("ARABIC LIGATURE HEH WITH MEEM WITH JEEM INITIAL FORM" #xFD93)
    ("ARABIC LIGATURE HEH WITH MEEM WITH MEEM INITIAL FORM" #xFD94)
    ("ARABIC LIGATURE NOON WITH HAH WITH MEEM INITIAL FORM" #xFD95)
    ("ARABIC LIGATURE NOON WITH HAH WITH ALEF MAKSURA FINAL FORM" #xFD96)
    ("ARABIC LIGATURE NOON WITH JEEM WITH MEEM FINAL FORM" #xFD97)
    ("ARABIC LIGATURE NOON WITH JEEM WITH MEEM INITIAL FORM" #xFD98)
    ("ARABIC LIGATURE NOON WITH JEEM WITH ALEF MAKSURA FINAL FORM" #xFD99)
    ("ARABIC LIGATURE NOON WITH MEEM WITH YEH FINAL FORM" #xFD9A)
    ("ARABIC LIGATURE NOON WITH MEEM WITH ALEF MAKSURA FINAL FORM" #xFD9B)
    ("ARABIC LIGATURE YEH WITH MEEM WITH MEEM FINAL FORM" #xFD9C)
    ("ARABIC LIGATURE YEH WITH MEEM WITH MEEM INITIAL FORM" #xFD9D)
    ("ARABIC LIGATURE BEH WITH KHAH WITH YEH FINAL FORM" #xFD9E)
    ("ARABIC LIGATURE TEH WITH JEEM WITH YEH FINAL FORM" #xFD9F)
    ("ARABIC LIGATURE TEH WITH JEEM WITH ALEF MAKSURA FINAL FORM" #xFDA0)
    ("ARABIC LIGATURE TEH WITH KHAH WITH YEH FINAL FORM" #xFDA1)
    ("ARABIC LIGATURE TEH WITH KHAH WITH ALEF MAKSURA FINAL FORM" #xFDA2)
    ("ARABIC LIGATURE TEH WITH MEEM WITH YEH FINAL FORM" #xFDA3)
    ("ARABIC LIGATURE TEH WITH MEEM WITH ALEF MAKSURA FINAL FORM" #xFDA4)
    ("ARABIC LIGATURE JEEM WITH MEEM WITH YEH FINAL FORM" #xFDA5)
    ("ARABIC LIGATURE JEEM WITH HAH WITH ALEF MAKSURA FINAL FORM" #xFDA6)
    ("ARABIC LIGATURE JEEM WITH MEEM WITH ALEF MAKSURA FINAL FORM" #xFDA7)
    ("ARABIC LIGATURE SEEN WITH KHAH WITH ALEF MAKSURA FINAL FORM" #xFDA8)
    ("ARABIC LIGATURE SAD WITH HAH WITH YEH FINAL FORM" #xFDA9)
    ("ARABIC LIGATURE SHEEN WITH HAH WITH YEH FINAL FORM" #xFDAA)
    ("ARABIC LIGATURE DAD WITH HAH WITH YEH FINAL FORM" #xFDAB)
    ("ARABIC LIGATURE LAM WITH JEEM WITH YEH FINAL FORM" #xFDAC)
    ("ARABIC LIGATURE LAM WITH MEEM WITH YEH FINAL FORM" #xFDAD)
    ("ARABIC LIGATURE YEH WITH HAH WITH YEH FINAL FORM" #xFDAE)
    ("ARABIC LIGATURE YEH WITH JEEM WITH YEH FINAL FORM" #xFDAF)
    ("ARABIC LIGATURE YEH WITH MEEM WITH YEH FINAL FORM" #xFDB0)
    ("ARABIC LIGATURE MEEM WITH MEEM WITH YEH FINAL FORM" #xFDB1)
    ("ARABIC LIGATURE QAF WITH MEEM WITH YEH FINAL FORM" #xFDB2)
    ("ARABIC LIGATURE NOON WITH HAH WITH YEH FINAL FORM" #xFDB3)
    ("ARABIC LIGATURE QAF WITH MEEM WITH HAH INITIAL FORM" #xFDB4)
    ("ARABIC LIGATURE LAM WITH HAH WITH MEEM INITIAL FORM" #xFDB5)
    ("ARABIC LIGATURE AIN WITH MEEM WITH YEH FINAL FORM" #xFDB6)
    ("ARABIC LIGATURE KAF WITH MEEM WITH YEH FINAL FORM" #xFDB7)
    ("ARABIC LIGATURE NOON WITH JEEM WITH HAH INITIAL FORM" #xFDB8)
    ("ARABIC LIGATURE MEEM WITH KHAH WITH YEH FINAL FORM" #xFDB9)
    ("ARABIC LIGATURE LAM WITH JEEM WITH MEEM INITIAL FORM" #xFDBA)
    ("ARABIC LIGATURE KAF WITH MEEM WITH MEEM FINAL FORM" #xFDBB)
    ("ARABIC LIGATURE LAM WITH JEEM WITH MEEM FINAL FORM" #xFDBC)
    ("ARABIC LIGATURE NOON WITH JEEM WITH HAH FINAL FORM" #xFDBD)
    ("ARABIC LIGATURE JEEM WITH HAH WITH YEH FINAL FORM" #xFDBE)
    ("ARABIC LIGATURE HAH WITH JEEM WITH YEH FINAL FORM" #xFDBF)
    ("ARABIC LIGATURE MEEM WITH JEEM WITH YEH FINAL FORM" #xFDC0)
    ("ARABIC LIGATURE FEH WITH MEEM WITH YEH FINAL FORM" #xFDC1)
    ("ARABIC LIGATURE BEH WITH HAH WITH YEH FINAL FORM" #xFDC2)
    ("ARABIC LIGATURE KAF WITH MEEM WITH MEEM INITIAL FORM" #xFDC3)
    ("ARABIC LIGATURE AIN WITH JEEM WITH MEEM INITIAL FORM" #xFDC4)
    ("ARABIC LIGATURE SAD WITH MEEM WITH MEEM INITIAL FORM" #xFDC5)
    ("ARABIC LIGATURE SEEN WITH KHAH WITH YEH FINAL FORM" #xFDC6)
    ("ARABIC LIGATURE NOON WITH JEEM WITH YEH FINAL FORM" #xFDC7)
    ("ARABIC LIGATURE SALLA USED AS KORANIC STOP SIGN ISOLATED FORM" #xFDF0)
    ("ARABIC LIGATURE QALA USED AS KORANIC STOP SIGN ISOLATED FORM" #xFDF1)
    ("ARABIC LIGATURE ALLAH ISOLATED FORM" #xFDF2)
    ("ARABIC LIGATURE AKBAR ISOLATED FORM" #xFDF3)
    ("ARABIC LIGATURE MOHAMMAD ISOLATED FORM" #xFDF4)
    ("ARABIC LIGATURE SALAM ISOLATED FORM" #xFDF5)
    ("ARABIC LIGATURE RASOUL ISOLATED FORM" #xFDF6)
    ("ARABIC LIGATURE ALAYHE ISOLATED FORM" #xFDF7)
    ("ARABIC LIGATURE WASALLAM ISOLATED FORM" #xFDF8)
    ("ARABIC LIGATURE SALLA ISOLATED FORM" #xFDF9)
    ("ARABIC LIGATURE SALLALLAHOU ALAYHE WASALLAM" #xFDFA)
    ("ARABIC LIGATURE JALLAJALALOUHOU" #xFDFB)
    ("RIAL SIGN" #xFDFC)
    ))

;; arch-tag: 1fbfd997-096a-43c8-88a1-9d438ddabb57
