(nxml-define-char-name-set 'tibetan
  '(("TIBETAN SYLLABLE OM" #x0F00)
    ("TIBETAN MARK GTER YIG MGO TRUNCATED A" #x0F01)
    ("TIBETAN MARK GTER YIG MGO -UM RNAM BCAD MA" #x0F02)
    ("TIBETAN MARK GTER YIG MGO -UM GTER TSHEG MA" #x0F03)
    ("TIBETAN MARK INITIAL YIG MGO MDUN MA" #x0F04)
    ("TIBETAN MARK CLOSING YIG MGO SGAB MA" #x0F05)
    ("TIBETAN MARK CARET YIG MGO PHUR SHAD MA" #x0F06)
    ("TIBETAN MARK YIG MGO TSHEG SHAD MA" #x0F07)
    ("TIBETAN MARK SBRUL SHAD" #x0F08)
    ("TIBETAN MARK BSKUR YIG MGO" #x0F09)
    ("TIBETAN MARK BKA- SHOG YIG MGO" #x0F0A)
    ("TIBETAN MARK INTERSYLLABIC TSHEG" #x0F0B)
    ("TIBETAN MARK DELIMITER TSHEG BSTAR" #x0F0C)
    ("TIBETAN MARK SHAD" #x0F0D)
    ("TIBETAN MARK NYIS SHAD" #x0F0E)
    ("TIBETAN MARK TSHEG SHAD" #x0F0F)
    ("TIBETAN MARK NYIS TSHEG SHAD" #x0F10)
    ("TIBETAN MARK RIN CHEN SPUNGS SHAD" #x0F11)
    ("TIBETAN MARK RGYA GRAM SHAD" #x0F12)
    ("TIBETAN MARK CARET -DZUD RTAGS ME LONG CAN" #x0F13)
    ("TIBETAN MARK GTER TSHEG" #x0F14)
    ("TIBETAN LOGOTYPE SIGN CHAD RTAGS" #x0F15)
    ("TIBETAN LOGOTYPE SIGN LHAG RTAGS" #x0F16)
    ("TIBETAN ASTROLOGICAL SIGN SGRA GCAN -CHAR RTAGS" #x0F17)
    ("TIBETAN ASTROLOGICAL SIGN -KHYUD PA" #x0F18)
    ("TIBETAN ASTROLOGICAL SIGN SDONG TSHUGS" #x0F19)
    ("TIBETAN SIGN RDEL DKAR GCIG" #x0F1A)
    ("TIBETAN SIGN RDEL DKAR GNYIS" #x0F1B)
    ("TIBETAN SIGN RDEL DKAR GSUM" #x0F1C)
    ("TIBETAN SIGN RDEL NAG GCIG" #x0F1D)
    ("TIBETAN SIGN RDEL NAG GNYIS" #x0F1E)
    ("TIBETAN SIGN RDEL DKAR RDEL NAG" #x0F1F)
    ("TIBETAN DIGIT ZERO" #x0F20)
    ("TIBETAN DIGIT ONE" #x0F21)
    ("TIBETAN DIGIT TWO" #x0F22)
    ("TIBETAN DIGIT THREE" #x0F23)
    ("TIBETAN DIGIT FOUR" #x0F24)
    ("TIBETAN DIGIT FIVE" #x0F25)
    ("TIBETAN DIGIT SIX" #x0F26)
    ("TIBETAN DIGIT SEVEN" #x0F27)
    ("TIBETAN DIGIT EIGHT" #x0F28)
    ("TIBETAN DIGIT NINE" #x0F29)
    ("TIBETAN DIGIT HALF ONE" #x0F2A)
    ("TIBETAN DIGIT HALF TWO" #x0F2B)
    ("TIBETAN DIGIT HALF THREE" #x0F2C)
    ("TIBETAN DIGIT HALF FOUR" #x0F2D)
    ("TIBETAN DIGIT HALF FIVE" #x0F2E)
    ("TIBETAN DIGIT HALF SIX" #x0F2F)
    ("TIBETAN DIGIT HALF SEVEN" #x0F30)
    ("TIBETAN DIGIT HALF EIGHT" #x0F31)
    ("TIBETAN DIGIT HALF NINE" #x0F32)
    ("TIBETAN DIGIT HALF ZERO" #x0F33)
    ("TIBETAN MARK BSDUS RTAGS" #x0F34)
    ("TIBETAN MARK NGAS BZUNG NYI ZLA" #x0F35)
    ("TIBETAN MARK CARET -DZUD RTAGS BZHI MIG CAN" #x0F36)
    ("TIBETAN MARK NGAS BZUNG SGOR RTAGS" #x0F37)
    ("TIBETAN MARK CHE MGO" #x0F38)
    ("TIBETAN MARK TSA -PHRU" #x0F39)
    ("TIBETAN MARK GUG RTAGS GYON" #x0F3A)
    ("TIBETAN MARK GUG RTAGS GYAS" #x0F3B)
    ("TIBETAN MARK ANG KHANG GYON" #x0F3C)
    ("TIBETAN MARK ANG KHANG GYAS" #x0F3D)
    ("TIBETAN SIGN YAR TSHES" #x0F3E)
    ("TIBETAN SIGN MAR TSHES" #x0F3F)
    ("TIBETAN LETTER KA" #x0F40)
    ("TIBETAN LETTER KHA" #x0F41)
    ("TIBETAN LETTER GA" #x0F42)
    ("TIBETAN LETTER GHA" #x0F43)
    ("TIBETAN LETTER NGA" #x0F44)
    ("TIBETAN LETTER CA" #x0F45)
    ("TIBETAN LETTER CHA" #x0F46)
    ("TIBETAN LETTER JA" #x0F47)
    ("TIBETAN LETTER NYA" #x0F49)
    ("TIBETAN LETTER TTA" #x0F4A)
    ("TIBETAN LETTER TTHA" #x0F4B)
    ("TIBETAN LETTER DDA" #x0F4C)
    ("TIBETAN LETTER DDHA" #x0F4D)
    ("TIBETAN LETTER NNA" #x0F4E)
    ("TIBETAN LETTER TA" #x0F4F)
    ("TIBETAN LETTER THA" #x0F50)
    ("TIBETAN LETTER DA" #x0F51)
    ("TIBETAN LETTER DHA" #x0F52)
    ("TIBETAN LETTER NA" #x0F53)
    ("TIBETAN LETTER PA" #x0F54)
    ("TIBETAN LETTER PHA" #x0F55)
    ("TIBETAN LETTER BA" #x0F56)
    ("TIBETAN LETTER BHA" #x0F57)
    ("TIBETAN LETTER MA" #x0F58)
    ("TIBETAN LETTER TSA" #x0F59)
    ("TIBETAN LETTER TSHA" #x0F5A)
    ("TIBETAN LETTER DZA" #x0F5B)
    ("TIBETAN LETTER DZHA" #x0F5C)
    ("TIBETAN LETTER WA" #x0F5D)
    ("TIBETAN LETTER ZHA" #x0F5E)
    ("TIBETAN LETTER ZA" #x0F5F)
    ("TIBETAN LETTER -A" #x0F60)
    ("TIBETAN LETTER YA" #x0F61)
    ("TIBETAN LETTER RA" #x0F62)
    ("TIBETAN LETTER LA" #x0F63)
    ("TIBETAN LETTER SHA" #x0F64)
    ("TIBETAN LETTER SSA" #x0F65)
    ("TIBETAN LETTER SA" #x0F66)
    ("TIBETAN LETTER HA" #x0F67)
    ("TIBETAN LETTER A" #x0F68)
    ("TIBETAN LETTER KSSA" #x0F69)
    ("TIBETAN LETTER FIXED-FORM RA" #x0F6A)
    ("TIBETAN VOWEL SIGN AA" #x0F71)
    ("TIBETAN VOWEL SIGN I" #x0F72)
    ("TIBETAN VOWEL SIGN II" #x0F73)
    ("TIBETAN VOWEL SIGN U" #x0F74)
    ("TIBETAN VOWEL SIGN UU" #x0F75)
    ("TIBETAN VOWEL SIGN VOCALIC R" #x0F76)
    ("TIBETAN VOWEL SIGN VOCALIC RR" #x0F77)
    ("TIBETAN VOWEL SIGN VOCALIC L" #x0F78)
    ("TIBETAN VOWEL SIGN VOCALIC LL" #x0F79)
    ("TIBETAN VOWEL SIGN E" #x0F7A)
    ("TIBETAN VOWEL SIGN EE" #x0F7B)
    ("TIBETAN VOWEL SIGN O" #x0F7C)
    ("TIBETAN VOWEL SIGN OO" #x0F7D)
    ("TIBETAN SIGN RJES SU NGA RO" #x0F7E)
    ("TIBETAN SIGN RNAM BCAD" #x0F7F)
    ("TIBETAN VOWEL SIGN REVERSED I" #x0F80)
    ("TIBETAN VOWEL SIGN REVERSED II" #x0F81)
    ("TIBETAN SIGN NYI ZLA NAA DA" #x0F82)
    ("TIBETAN SIGN SNA LDAN" #x0F83)
    ("TIBETAN MARK HALANTA" #x0F84)
    ("TIBETAN MARK PALUTA" #x0F85)
    ("TIBETAN SIGN LCI RTAGS" #x0F86)
    ("TIBETAN SIGN YANG RTAGS" #x0F87)
    ("TIBETAN SIGN LCE TSA CAN" #x0F88)
    ("TIBETAN SIGN MCHU CAN" #x0F89)
    ("TIBETAN SIGN GRU CAN RGYINGS" #x0F8A)
    ("TIBETAN SIGN GRU MED RGYINGS" #x0F8B)
    ("TIBETAN SUBJOINED LETTER KA" #x0F90)
    ("TIBETAN SUBJOINED LETTER KHA" #x0F91)
    ("TIBETAN SUBJOINED LETTER GA" #x0F92)
    ("TIBETAN SUBJOINED LETTER GHA" #x0F93)
    ("TIBETAN SUBJOINED LETTER NGA" #x0F94)
    ("TIBETAN SUBJOINED LETTER CA" #x0F95)
    ("TIBETAN SUBJOINED LETTER CHA" #x0F96)
    ("TIBETAN SUBJOINED LETTER JA" #x0F97)
    ("TIBETAN SUBJOINED LETTER NYA" #x0F99)
    ("TIBETAN SUBJOINED LETTER TTA" #x0F9A)
    ("TIBETAN SUBJOINED LETTER TTHA" #x0F9B)
    ("TIBETAN SUBJOINED LETTER DDA" #x0F9C)
    ("TIBETAN SUBJOINED LETTER DDHA" #x0F9D)
    ("TIBETAN SUBJOINED LETTER NNA" #x0F9E)
    ("TIBETAN SUBJOINED LETTER TA" #x0F9F)
    ("TIBETAN SUBJOINED LETTER THA" #x0FA0)
    ("TIBETAN SUBJOINED LETTER DA" #x0FA1)
    ("TIBETAN SUBJOINED LETTER DHA" #x0FA2)
    ("TIBETAN SUBJOINED LETTER NA" #x0FA3)
    ("TIBETAN SUBJOINED LETTER PA" #x0FA4)
    ("TIBETAN SUBJOINED LETTER PHA" #x0FA5)
    ("TIBETAN SUBJOINED LETTER BA" #x0FA6)
    ("TIBETAN SUBJOINED LETTER BHA" #x0FA7)
    ("TIBETAN SUBJOINED LETTER MA" #x0FA8)
    ("TIBETAN SUBJOINED LETTER TSA" #x0FA9)
    ("TIBETAN SUBJOINED LETTER TSHA" #x0FAA)
    ("TIBETAN SUBJOINED LETTER DZA" #x0FAB)
    ("TIBETAN SUBJOINED LETTER DZHA" #x0FAC)
    ("TIBETAN SUBJOINED LETTER WA" #x0FAD)
    ("TIBETAN SUBJOINED LETTER ZHA" #x0FAE)
    ("TIBETAN SUBJOINED LETTER ZA" #x0FAF)
    ("TIBETAN SUBJOINED LETTER -A" #x0FB0)
    ("TIBETAN SUBJOINED LETTER YA" #x0FB1)
    ("TIBETAN SUBJOINED LETTER RA" #x0FB2)
    ("TIBETAN SUBJOINED LETTER LA" #x0FB3)
    ("TIBETAN SUBJOINED LETTER SHA" #x0FB4)
    ("TIBETAN SUBJOINED LETTER SSA" #x0FB5)
    ("TIBETAN SUBJOINED LETTER SA" #x0FB6)
    ("TIBETAN SUBJOINED LETTER HA" #x0FB7)
    ("TIBETAN SUBJOINED LETTER A" #x0FB8)
    ("TIBETAN SUBJOINED LETTER KSSA" #x0FB9)
    ("TIBETAN SUBJOINED LETTER FIXED-FORM WA" #x0FBA)
    ("TIBETAN SUBJOINED LETTER FIXED-FORM YA" #x0FBB)
    ("TIBETAN SUBJOINED LETTER FIXED-FORM RA" #x0FBC)
    ("TIBETAN KU RU KHA" #x0FBE)
    ("TIBETAN KU RU KHA BZHI MIG CAN" #x0FBF)
    ("TIBETAN CANTILLATION SIGN HEAVY BEAT" #x0FC0)
    ("TIBETAN CANTILLATION SIGN LIGHT BEAT" #x0FC1)
    ("TIBETAN CANTILLATION SIGN CANG TE-U" #x0FC2)
    ("TIBETAN CANTILLATION SIGN SBUB -CHAL" #x0FC3)
    ("TIBETAN SYMBOL DRIL BU" #x0FC4)
    ("TIBETAN SYMBOL RDO RJE" #x0FC5)
    ("TIBETAN SYMBOL PADMA GDAN" #x0FC6)
    ("TIBETAN SYMBOL RDO RJE RGYA GRAM" #x0FC7)
    ("TIBETAN SYMBOL PHUR PA" #x0FC8)
    ("TIBETAN SYMBOL NOR BU" #x0FC9)
    ("TIBETAN SYMBOL NOR BU NYIS -KHYIL" #x0FCA)
    ("TIBETAN SYMBOL NOR BU GSUM -KHYIL" #x0FCB)
    ("TIBETAN SYMBOL NOR BU BZHI -KHYIL" #x0FCC)
    ("TIBETAN SIGN RDEL NAG GSUM" #x0FCF)
    ))

;; arch-tag: 547e48c6-3ded-4dc7-be69-4ee324c604c3
