(nxml-define-char-name-set 'hiragana
  '(("HIRAGANA LETTER SMALL A" #x3041)
    ("HIRAGANA LETTER A" #x3042)
    ("HIRAGANA LETTER SMALL I" #x3043)
    ("HIRAGANA LETTER I" #x3044)
    ("HIRAGANA LETTER SMALL U" #x3045)
    ("HIRAGANA LETTER U" #x3046)
    ("HIRAGANA LETTER SMALL E" #x3047)
    ("HIRAGANA LETTER E" #x3048)
    ("HIRAGANA LETTER SMALL O" #x3049)
    ("HIRAGANA LETTER O" #x304A)
    ("HIRAGANA LETTER KA" #x304B)
    ("HIRAGANA LETTER GA" #x304C)
    ("HIRAGANA LETTER KI" #x304D)
    ("HIRAGANA LETTER GI" #x304E)
    ("HIRAGANA LETTER KU" #x304F)
    ("HIRAGANA LETTER GU" #x3050)
    ("HIRAGANA LETTER KE" #x3051)
    ("HIRAGANA LETTER GE" #x3052)
    ("HIRAGANA LETTER KO" #x3053)
    ("HIRAGANA LETTER GO" #x3054)
    ("HIRAGANA LETTER SA" #x3055)
    ("HIRAGANA LETTER ZA" #x3056)
    ("HIRAGANA LETTER SI" #x3057)
    ("HIRAGANA LETTER ZI" #x3058)
    ("HIRAGANA LETTER SU" #x3059)
    ("HIRAGANA LETTER ZU" #x305A)
    ("HIRAGANA LETTER SE" #x305B)
    ("HIRAGANA LETTER ZE" #x305C)
    ("HIRAGANA LETTER SO" #x305D)
    ("HIRAGANA LETTER ZO" #x305E)
    ("HIRAGANA LETTER TA" #x305F)
    ("HIRAGANA LETTER DA" #x3060)
    ("HIRAGANA LETTER TI" #x3061)
    ("HIRAGANA LETTER DI" #x3062)
    ("HIRAGANA LETTER SMALL TU" #x3063)
    ("HIRAGANA LETTER TU" #x3064)
    ("HIRAGANA LETTER DU" #x3065)
    ("HIRAGANA LETTER TE" #x3066)
    ("HIRAGANA LETTER DE" #x3067)
    ("HIRAGANA LETTER TO" #x3068)
    ("HIRAGANA LETTER DO" #x3069)
    ("HIRAGANA LETTER NA" #x306A)
    ("HIRAGANA LETTER NI" #x306B)
    ("HIRAGANA LETTER NU" #x306C)
    ("HIRAGANA LETTER NE" #x306D)
    ("HIRAGANA LETTER NO" #x306E)
    ("HIRAGANA LETTER HA" #x306F)
    ("HIRAGANA LETTER BA" #x3070)
    ("HIRAGANA LETTER PA" #x3071)
    ("HIRAGANA LETTER HI" #x3072)
    ("HIRAGANA LETTER BI" #x3073)
    ("HIRAGANA LETTER PI" #x3074)
    ("HIRAGANA LETTER HU" #x3075)
    ("HIRAGANA LETTER BU" #x3076)
    ("HIRAGANA LETTER PU" #x3077)
    ("HIRAGANA LETTER HE" #x3078)
    ("HIRAGANA LETTER BE" #x3079)
    ("HIRAGANA LETTER PE" #x307A)
    ("HIRAGANA LETTER HO" #x307B)
    ("HIRAGANA LETTER BO" #x307C)
    ("HIRAGANA LETTER PO" #x307D)
    ("HIRAGANA LETTER MA" #x307E)
    ("HIRAGANA LETTER MI" #x307F)
    ("HIRAGANA LETTER MU" #x3080)
    ("HIRAGANA LETTER ME" #x3081)
    ("HIRAGANA LETTER MO" #x3082)
    ("HIRAGANA LETTER SMALL YA" #x3083)
    ("HIRAGANA LETTER YA" #x3084)
    ("HIRAGANA LETTER SMALL YU" #x3085)
    ("HIRAGANA LETTER YU" #x3086)
    ("HIRAGANA LETTER SMALL YO" #x3087)
    ("HIRAGANA LETTER YO" #x3088)
    ("HIRAGANA LETTER RA" #x3089)
    ("HIRAGANA LETTER RI" #x308A)
    ("HIRAGANA LETTER RU" #x308B)
    ("HIRAGANA LETTER RE" #x308C)
    ("HIRAGANA LETTER RO" #x308D)
    ("HIRAGANA LETTER SMALL WA" #x308E)
    ("HIRAGANA LETTER WA" #x308F)
    ("HIRAGANA LETTER WI" #x3090)
    ("HIRAGANA LETTER WE" #x3091)
    ("HIRAGANA LETTER WO" #x3092)
    ("HIRAGANA LETTER N" #x3093)
    ("HIRAGANA LETTER VU" #x3094)
    ("HIRAGANA LETTER SMALL KA" #x3095)
    ("HIRAGANA LETTER SMALL KE" #x3096)
    ("COMBINING KATAKANA-HIRAGANA VOICED SOUND MARK" #x3099)
    ("COMBINING KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK" #x309A)
    ("KATAKANA-HIRAGANA VOICED SOUND MARK" #x309B)
    ("KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK" #x309C)
    ("HIRAGANA ITERATION MARK" #x309D)
    ("HIRAGANA VOICED ITERATION MARK" #x309E)
    ("HIRAGANA DIGRAPH YORI" #x309F)
    ))

;; arch-tag: dc96b2bf-74f6-4885-adbb-4a81bd2884be
