(nxml-define-char-name-set 'cyrillic-supplementary
  '(("CYRILLIC CAPITAL LETTER KOMI DE" #x0500)
    ("CYRILLIC SMALL LETTER KOMI DE" #x0501)
    ("CYRILLIC CAPITAL LETTER KOMI DJE" #x0502)
    ("CYRILLIC SMALL LETTER KOMI DJE" #x0503)
    ("CYRILLIC CAPITAL LETTER KOMI ZJE" #x0504)
    ("CYRILLIC SMALL LETTER KOMI ZJE" #x0505)
    ("CYRILLIC CAPITAL LETTER KOMI DZJE" #x0506)
    ("CYRILLIC SMALL LETTER KOMI DZJE" #x0507)
    ("CYRILLIC CAPITAL LETTER KOMI LJE" #x0508)
    ("CYRILLIC SMALL LETTER KOMI LJE" #x0509)
    ("CYRILLIC CAPITAL LETTER KOMI NJE" #x050A)
    ("CYRILLIC SMALL LETTER KOMI NJE" #x050B)
    ("CYRILLIC CAPITAL LETTER KOMI SJE" #x050C)
    ("CYRILLIC SMALL LETTER KOMI SJE" #x050D)
    ("CYRILLIC CAPITAL LETTER KOMI TJE" #x050E)
    ("CYRILLIC SMALL LETTER KOMI TJE" #x050F)
    ))

;; arch-tag: 3ae51d36-3921-4ec5-b6be-3e9c895be9f3
