(nxml-define-char-name-set 'hangul-jamo
  '(("HANGUL CHOSEONG KIYEOK" #x1100)
    ("HANGUL CHOSEONG SSANGKIYEOK" #x1101)
    ("HANGUL CHOSEONG NIEUN" #x1102)
    ("HANGUL CHOSEONG TIKEUT" #x1103)
    ("HANGUL CHOSEONG SSANGTIKEUT" #x1104)
    ("HANGUL CHOSEONG RIEUL" #x1105)
    ("HANGUL CHOSEONG MIEUM" #x1106)
    ("HANGUL CHOSEONG PIEUP" #x1107)
    ("HANGUL CHOSEONG SSANGPIEUP" #x1108)
    ("HANGUL CHOSEONG SIOS" #x1109)
    ("HANGUL CHOSEONG SSANGSIOS" #x110A)
    ("HANGUL CHOSEONG IEUNG" #x110B)
    ("HANGUL CHOSEONG CIEUC" #x110C)
    ("HANGUL CHOSEONG SSANGCIEUC" #x110D)
    ("HANGUL CHOSEONG CHIEUCH" #x110E)
    ("HANGUL CHOSEONG KHIEUKH" #x110F)
    ("HANGUL CHOSEONG THIEUTH" #x1110)
    ("HANGUL CHOSEONG PHIEUPH" #x1111)
    ("HANGUL CHOSEONG HIEUH" #x1112)
    ("HANGUL CHOSEONG NIEUN-KIYEOK" #x1113)
    ("HANGUL CHOSEONG SSANGNIEUN" #x1114)
    ("HANGUL CHOSEONG NIEUN-TIKEUT" #x1115)
    ("HANGUL CHOSEONG NIEUN-PIEUP" #x1116)
    ("HANGUL CHOSEONG TIKEUT-KIYEOK" #x1117)
    ("HANGUL CHOSEONG RIEUL-NIEUN" #x1118)
    ("HANGUL CHOSEONG SSANGRIEUL" #x1119)
    ("HANGUL CHOSEONG RIEUL-HIEUH" #x111A)
    ("HANGUL CHOSEONG KAPYEOUNRIEUL" #x111B)
    ("HANGUL CHOSEONG MIEUM-PIEUP" #x111C)
    ("HANGUL CHOSEONG KAPYEOUNMIEUM" #x111D)
    ("HANGUL CHOSEONG PIEUP-KIYEOK" #x111E)
    ("HANGUL CHOSEONG PIEUP-NIEUN" #x111F)
    ("HANGUL CHOSEONG PIEUP-TIKEUT" #x1120)
    ("HANGUL CHOSEONG PIEUP-SIOS" #x1121)
    ("HANGUL CHOSEONG PIEUP-SIOS-KIYEOK" #x1122)
    ("HANGUL CHOSEONG PIEUP-SIOS-TIKEUT" #x1123)
    ("HANGUL CHOSEONG PIEUP-SIOS-PIEUP" #x1124)
    ("HANGUL CHOSEONG PIEUP-SSANGSIOS" #x1125)
    ("HANGUL CHOSEONG PIEUP-SIOS-CIEUC" #x1126)
    ("HANGUL CHOSEONG PIEUP-CIEUC" #x1127)
    ("HANGUL CHOSEONG PIEUP-CHIEUCH" #x1128)
    ("HANGUL CHOSEONG PIEUP-THIEUTH" #x1129)
    ("HANGUL CHOSEONG PIEUP-PHIEUPH" #x112A)
    ("HANGUL CHOSEONG KAPYEOUNPIEUP" #x112B)
    ("HANGUL CHOSEONG KAPYEOUNSSANGPIEUP" #x112C)
    ("HANGUL CHOSEONG SIOS-KIYEOK" #x112D)
    ("HANGUL CHOSEONG SIOS-NIEUN" #x112E)
    ("HANGUL CHOSEONG SIOS-TIKEUT" #x112F)
    ("HANGUL CHOSEONG SIOS-RIEUL" #x1130)
    ("HANGUL CHOSEONG SIOS-MIEUM" #x1131)
    ("HANGUL CHOSEONG SIOS-PIEUP" #x1132)
    ("HANGUL CHOSEONG SIOS-PIEUP-KIYEOK" #x1133)
    ("HANGUL CHOSEONG SIOS-SSANGSIOS" #x1134)
    ("HANGUL CHOSEONG SIOS-IEUNG" #x1135)
    ("HANGUL CHOSEONG SIOS-CIEUC" #x1136)
    ("HANGUL CHOSEONG SIOS-CHIEUCH" #x1137)
    ("HANGUL CHOSEONG SIOS-KHIEUKH" #x1138)
    ("HANGUL CHOSEONG SIOS-THIEUTH" #x1139)
    ("HANGUL CHOSEONG SIOS-PHIEUPH" #x113A)
    ("HANGUL CHOSEONG SIOS-HIEUH" #x113B)
    ("HANGUL CHOSEONG CHITUEUMSIOS" #x113C)
    ("HANGUL CHOSEONG CHITUEUMSSANGSIOS" #x113D)
    ("HANGUL CHOSEONG CEONGCHIEUMSIOS" #x113E)
    ("HANGUL CHOSEONG CEONGCHIEUMSSANGSIOS" #x113F)
    ("HANGUL CHOSEONG PANSIOS" #x1140)
    ("HANGUL CHOSEONG IEUNG-KIYEOK" #x1141)
    ("HANGUL CHOSEONG IEUNG-TIKEUT" #x1142)
    ("HANGUL CHOSEONG IEUNG-MIEUM" #x1143)
    ("HANGUL CHOSEONG IEUNG-PIEUP" #x1144)
    ("HANGUL CHOSEONG IEUNG-SIOS" #x1145)
    ("HANGUL CHOSEONG IEUNG-PANSIOS" #x1146)
    ("HANGUL CHOSEONG SSANGIEUNG" #x1147)
    ("HANGUL CHOSEONG IEUNG-CIEUC" #x1148)
    ("HANGUL CHOSEONG IEUNG-CHIEUCH" #x1149)
    ("HANGUL CHOSEONG IEUNG-THIEUTH" #x114A)
    ("HANGUL CHOSEONG IEUNG-PHIEUPH" #x114B)
    ("HANGUL CHOSEONG YESIEUNG" #x114C)
    ("HANGUL CHOSEONG CIEUC-IEUNG" #x114D)
    ("HANGUL CHOSEONG CHITUEUMCIEUC" #x114E)
    ("HANGUL CHOSEONG CHITUEUMSSANGCIEUC" #x114F)
    ("HANGUL CHOSEONG CEONGCHIEUMCIEUC" #x1150)
    ("HANGUL CHOSEONG CEONGCHIEUMSSANGCIEUC" #x1151)
    ("HANGUL CHOSEONG CHIEUCH-KHIEUKH" #x1152)
    ("HANGUL CHOSEONG CHIEUCH-HIEUH" #x1153)
    ("HANGUL CHOSEONG CHITUEUMCHIEUCH" #x1154)
    ("HANGUL CHOSEONG CEONGCHIEUMCHIEUCH" #x1155)
    ("HANGUL CHOSEONG PHIEUPH-PIEUP" #x1156)
    ("HANGUL CHOSEONG KAPYEOUNPHIEUPH" #x1157)
    ("HANGUL CHOSEONG SSANGHIEUH" #x1158)
    ("HANGUL CHOSEONG YEORINHIEUH" #x1159)
    ("HANGUL CHOSEONG FILLER" #x115F)
    ("HANGUL JUNGSEONG FILLER" #x1160)
    ("HANGUL JUNGSEONG A" #x1161)
    ("HANGUL JUNGSEONG AE" #x1162)
    ("HANGUL JUNGSEONG YA" #x1163)
    ("HANGUL JUNGSEONG YAE" #x1164)
    ("HANGUL JUNGSEONG EO" #x1165)
    ("HANGUL JUNGSEONG E" #x1166)
    ("HANGUL JUNGSEONG YEO" #x1167)
    ("HANGUL JUNGSEONG YE" #x1168)
    ("HANGUL JUNGSEONG O" #x1169)
    ("HANGUL JUNGSEONG WA" #x116A)
    ("HANGUL JUNGSEONG WAE" #x116B)
    ("HANGUL JUNGSEONG OE" #x116C)
    ("HANGUL JUNGSEONG YO" #x116D)
    ("HANGUL JUNGSEONG U" #x116E)
    ("HANGUL JUNGSEONG WEO" #x116F)
    ("HANGUL JUNGSEONG WE" #x1170)
    ("HANGUL JUNGSEONG WI" #x1171)
    ("HANGUL JUNGSEONG YU" #x1172)
    ("HANGUL JUNGSEONG EU" #x1173)
    ("HANGUL JUNGSEONG YI" #x1174)
    ("HANGUL JUNGSEONG I" #x1175)
    ("HANGUL JUNGSEONG A-O" #x1176)
    ("HANGUL JUNGSEONG A-U" #x1177)
    ("HANGUL JUNGSEONG YA-O" #x1178)
    ("HANGUL JUNGSEONG YA-YO" #x1179)
    ("HANGUL JUNGSEONG EO-O" #x117A)
    ("HANGUL JUNGSEONG EO-U" #x117B)
    ("HANGUL JUNGSEONG EO-EU" #x117C)
    ("HANGUL JUNGSEONG YEO-O" #x117D)
    ("HANGUL JUNGSEONG YEO-U" #x117E)
    ("HANGUL JUNGSEONG O-EO" #x117F)
    ("HANGUL JUNGSEONG O-E" #x1180)
    ("HANGUL JUNGSEONG O-YE" #x1181)
    ("HANGUL JUNGSEONG O-O" #x1182)
    ("HANGUL JUNGSEONG O-U" #x1183)
    ("HANGUL JUNGSEONG YO-YA" #x1184)
    ("HANGUL JUNGSEONG YO-YAE" #x1185)
    ("HANGUL JUNGSEONG YO-YEO" #x1186)
    ("HANGUL JUNGSEONG YO-O" #x1187)
    ("HANGUL JUNGSEONG YO-I" #x1188)
    ("HANGUL JUNGSEONG U-A" #x1189)
    ("HANGUL JUNGSEONG U-AE" #x118A)
    ("HANGUL JUNGSEONG U-EO-EU" #x118B)
    ("HANGUL JUNGSEONG U-YE" #x118C)
    ("HANGUL JUNGSEONG U-U" #x118D)
    ("HANGUL JUNGSEONG YU-A" #x118E)
    ("HANGUL JUNGSEONG YU-EO" #x118F)
    ("HANGUL JUNGSEONG YU-E" #x1190)
    ("HANGUL JUNGSEONG YU-YEO" #x1191)
    ("HANGUL JUNGSEONG YU-YE" #x1192)
    ("HANGUL JUNGSEONG YU-U" #x1193)
    ("HANGUL JUNGSEONG YU-I" #x1194)
    ("HANGUL JUNGSEONG EU-U" #x1195)
    ("HANGUL JUNGSEONG EU-EU" #x1196)
    ("HANGUL JUNGSEONG YI-U" #x1197)
    ("HANGUL JUNGSEONG I-A" #x1198)
    ("HANGUL JUNGSEONG I-YA" #x1199)
    ("HANGUL JUNGSEONG I-O" #x119A)
    ("HANGUL JUNGSEONG I-U" #x119B)
    ("HANGUL JUNGSEONG I-EU" #x119C)
    ("HANGUL JUNGSEONG I-ARAEA" #x119D)
    ("HANGUL JUNGSEONG ARAEA" #x119E)
    ("HANGUL JUNGSEONG ARAEA-EO" #x119F)
    ("HANGUL JUNGSEONG ARAEA-U" #x11A0)
    ("HANGUL JUNGSEONG ARAEA-I" #x11A1)
    ("HANGUL JUNGSEONG SSANGARAEA" #x11A2)
    ("HANGUL JONGSEONG KIYEOK" #x11A8)
    ("HANGUL JONGSEONG SSANGKIYEOK" #x11A9)
    ("HANGUL JONGSEONG KIYEOK-SIOS" #x11AA)
    ("HANGUL JONGSEONG NIEUN" #x11AB)
    ("HANGUL JONGSEONG NIEUN-CIEUC" #x11AC)
    ("HANGUL JONGSEONG NIEUN-HIEUH" #x11AD)
    ("HANGUL JONGSEONG TIKEUT" #x11AE)
    ("HANGUL JONGSEONG RIEUL" #x11AF)
    ("HANGUL JONGSEONG RIEUL-KIYEOK" #x11B0)
    ("HANGUL JONGSEONG RIEUL-MIEUM" #x11B1)
    ("HANGUL JONGSEONG RIEUL-PIEUP" #x11B2)
    ("HANGUL JONGSEONG RIEUL-SIOS" #x11B3)
    ("HANGUL JONGSEONG RIEUL-THIEUTH" #x11B4)
    ("HANGUL JONGSEONG RIEUL-PHIEUPH" #x11B5)
    ("HANGUL JONGSEONG RIEUL-HIEUH" #x11B6)
    ("HANGUL JONGSEONG MIEUM" #x11B7)
    ("HANGUL JONGSEONG PIEUP" #x11B8)
    ("HANGUL JONGSEONG PIEUP-SIOS" #x11B9)
    ("HANGUL JONGSEONG SIOS" #x11BA)
    ("HANGUL JONGSEONG SSANGSIOS" #x11BB)
    ("HANGUL JONGSEONG IEUNG" #x11BC)
    ("HANGUL JONGSEONG CIEUC" #x11BD)
    ("HANGUL JONGSEONG CHIEUCH" #x11BE)
    ("HANGUL JONGSEONG KHIEUKH" #x11BF)
    ("HANGUL JONGSEONG THIEUTH" #x11C0)
    ("HANGUL JONGSEONG PHIEUPH" #x11C1)
    ("HANGUL JONGSEONG HIEUH" #x11C2)
    ("HANGUL JONGSEONG KIYEOK-RIEUL" #x11C3)
    ("HANGUL JONGSEONG KIYEOK-SIOS-KIYEOK" #x11C4)
    ("HANGUL JONGSEONG NIEUN-KIYEOK" #x11C5)
    ("HANGUL JONGSEONG NIEUN-TIKEUT" #x11C6)
    ("HANGUL JONGSEONG NIEUN-SIOS" #x11C7)
    ("HANGUL JONGSEONG NIEUN-PANSIOS" #x11C8)
    ("HANGUL JONGSEONG NIEUN-THIEUTH" #x11C9)
    ("HANGUL JONGSEONG TIKEUT-KIYEOK" #x11CA)
    ("HANGUL JONGSEONG TIKEUT-RIEUL" #x11CB)
    ("HANGUL JONGSEONG RIEUL-KIYEOK-SIOS" #x11CC)
    ("HANGUL JONGSEONG RIEUL-NIEUN" #x11CD)
    ("HANGUL JONGSEONG RIEUL-TIKEUT" #x11CE)
    ("HANGUL JONGSEONG RIEUL-TIKEUT-HIEUH" #x11CF)
    ("HANGUL JONGSEONG SSANGRIEUL" #x11D0)
    ("HANGUL JONGSEONG RIEUL-MIEUM-KIYEOK" #x11D1)
    ("HANGUL JONGSEONG RIEUL-MIEUM-SIOS" #x11D2)
    ("HANGUL JONGSEONG RIEUL-PIEUP-SIOS" #x11D3)
    ("HANGUL JONGSEONG RIEUL-PIEUP-HIEUH" #x11D4)
    ("HANGUL JONGSEONG RIEUL-KAPYEOUNPIEUP" #x11D5)
    ("HANGUL JONGSEONG RIEUL-SSANGSIOS" #x11D6)
    ("HANGUL JONGSEONG RIEUL-PANSIOS" #x11D7)
    ("HANGUL JONGSEONG RIEUL-KHIEUKH" #x11D8)
    ("HANGUL JONGSEONG RIEUL-YEORINHIEUH" #x11D9)
    ("HANGUL JONGSEONG MIEUM-KIYEOK" #x11DA)
    ("HANGUL JONGSEONG MIEUM-RIEUL" #x11DB)
    ("HANGUL JONGSEONG MIEUM-PIEUP" #x11DC)
    ("HANGUL JONGSEONG MIEUM-SIOS" #x11DD)
    ("HANGUL JONGSEONG MIEUM-SSANGSIOS" #x11DE)
    ("HANGUL JONGSEONG MIEUM-PANSIOS" #x11DF)
    ("HANGUL JONGSEONG MIEUM-CHIEUCH" #x11E0)
    ("HANGUL JONGSEONG MIEUM-HIEUH" #x11E1)
    ("HANGUL JONGSEONG KAPYEOUNMIEUM" #x11E2)
    ("HANGUL JONGSEONG PIEUP-RIEUL" #x11E3)
    ("HANGUL JONGSEONG PIEUP-PHIEUPH" #x11E4)
    ("HANGUL JONGSEONG PIEUP-HIEUH" #x11E5)
    ("HANGUL JONGSEONG KAPYEOUNPIEUP" #x11E6)
    ("HANGUL JONGSEONG SIOS-KIYEOK" #x11E7)
    ("HANGUL JONGSEONG SIOS-TIKEUT" #x11E8)
    ("HANGUL JONGSEONG SIOS-RIEUL" #x11E9)
    ("HANGUL JONGSEONG SIOS-PIEUP" #x11EA)
    ("HANGUL JONGSEONG PANSIOS" #x11EB)
    ("HANGUL JONGSEONG IEUNG-KIYEOK" #x11EC)
    ("HANGUL JONGSEONG IEUNG-SSANGKIYEOK" #x11ED)
    ("HANGUL JONGSEONG SSANGIEUNG" #x11EE)
    ("HANGUL JONGSEONG IEUNG-KHIEUKH" #x11EF)
    ("HANGUL JONGSEONG YESIEUNG" #x11F0)
    ("HANGUL JONGSEONG YESIEUNG-SIOS" #x11F1)
    ("HANGUL JONGSEONG YESIEUNG-PANSIOS" #x11F2)
    ("HANGUL JONGSEONG PHIEUPH-PIEUP" #x11F3)
    ("HANGUL JONGSEONG KAPYEOUNPHIEUPH" #x11F4)
    ("HANGUL JONGSEONG HIEUH-NIEUN" #x11F5)
    ("HANGUL JONGSEONG HIEUH-RIEUL" #x11F6)
    ("HANGUL JONGSEONG HIEUH-MIEUM" #x11F7)
    ("HANGUL JONGSEONG HIEUH-PIEUP" #x11F8)
    ("HANGUL JONGSEONG YEORINHIEUH" #x11F9)
    ))

;; arch-tag: e631a2ad-7d7e-4a86-89f0-e6e4f7db5f39
