(nxml-define-char-name-set 'cherokee
  '(("CHEROKEE LETTER A" #x13A0)
    ("CHEROKEE LETTER E" #x13A1)
    ("CHEROKEE LETTER I" #x13A2)
    ("CHEROKEE LETTER O" #x13A3)
    ("CHEROKEE LETTER U" #x13A4)
    ("CHEROKEE LETTER V" #x13A5)
    ("CHEROKEE LETTER GA" #x13A6)
    ("CHEROKEE LETTER KA" #x13A7)
    ("CHEROKEE LETTER GE" #x13A8)
    ("CHEROKEE LETTER GI" #x13A9)
    ("CHEROKEE LETTER GO" #x13AA)
    ("CHEROKEE LETTER GU" #x13AB)
    ("CHEROKEE LETTER GV" #x13AC)
    ("CHEROKEE LETTER HA" #x13AD)
    ("CHEROKEE LETTER HE" #x13AE)
    ("CHEROKEE LETTER HI" #x13AF)
    ("CHEROKEE LETTER HO" #x13B0)
    ("CHEROKEE LETTER HU" #x13B1)
    ("CHEROKEE LETTER HV" #x13B2)
    ("CHEROKEE LETTER LA" #x13B3)
    ("CHEROKEE LETTER LE" #x13B4)
    ("CHEROKEE LETTER LI" #x13B5)
    ("CHEROKEE LETTER LO" #x13B6)
    ("CHEROKEE LETTER LU" #x13B7)
    ("CHEROKEE LETTER LV" #x13B8)
    ("CHEROKEE LETTER MA" #x13B9)
    ("CHEROKEE LETTER ME" #x13BA)
    ("CHEROKEE LETTER MI" #x13BB)
    ("CHEROKEE LETTER MO" #x13BC)
    ("CHEROKEE LETTER MU" #x13BD)
    ("CHEROKEE LETTER NA" #x13BE)
    ("CHEROKEE LETTER HNA" #x13BF)
    ("CHEROKEE LETTER NAH" #x13C0)
    ("CHEROKEE LETTER NE" #x13C1)
    ("CHEROKEE LETTER NI" #x13C2)
    ("CHEROKEE LETTER NO" #x13C3)
    ("CHEROKEE LETTER NU" #x13C4)
    ("CHEROKEE LETTER NV" #x13C5)
    ("CHEROKEE LETTER QUA" #x13C6)
    ("CHEROKEE LETTER QUE" #x13C7)
    ("CHEROKEE LETTER QUI" #x13C8)
    ("CHEROKEE LETTER QUO" #x13C9)
    ("CHEROKEE LETTER QUU" #x13CA)
    ("CHEROKEE LETTER QUV" #x13CB)
    ("CHEROKEE LETTER SA" #x13CC)
    ("CHEROKEE LETTER S" #x13CD)
    ("CHEROKEE LETTER SE" #x13CE)
    ("CHEROKEE LETTER SI" #x13CF)
    ("CHEROKEE LETTER SO" #x13D0)
    ("CHEROKEE LETTER SU" #x13D1)
    ("CHEROKEE LETTER SV" #x13D2)
    ("CHEROKEE LETTER DA" #x13D3)
    ("CHEROKEE LETTER TA" #x13D4)
    ("CHEROKEE LETTER DE" #x13D5)
    ("CHEROKEE LETTER TE" #x13D6)
    ("CHEROKEE LETTER DI" #x13D7)
    ("CHEROKEE LETTER TI" #x13D8)
    ("CHEROKEE LETTER DO" #x13D9)
    ("CHEROKEE LETTER DU" #x13DA)
    ("CHEROKEE LETTER DV" #x13DB)
    ("CHEROKEE LETTER DLA" #x13DC)
    ("CHEROKEE LETTER TLA" #x13DD)
    ("CHEROKEE LETTER TLE" #x13DE)
    ("CHEROKEE LETTER TLI" #x13DF)
    ("CHEROKEE LETTER TLO" #x13E0)
    ("CHEROKEE LETTER TLU" #x13E1)
    ("CHEROKEE LETTER TLV" #x13E2)
    ("CHEROKEE LETTER TSA" #x13E3)
    ("CHEROKEE LETTER TSE" #x13E4)
    ("CHEROKEE LETTER TSI" #x13E5)
    ("CHEROKEE LETTER TSO" #x13E6)
    ("CHEROKEE LETTER TSU" #x13E7)
    ("CHEROKEE LETTER TSV" #x13E8)
    ("CHEROKEE LETTER WA" #x13E9)
    ("CHEROKEE LETTER WE" #x13EA)
    ("CHEROKEE LETTER WI" #x13EB)
    ("CHEROKEE LETTER WO" #x13EC)
    ("CHEROKEE LETTER WU" #x13ED)
    ("CHEROKEE LETTER WV" #x13EE)
    ("CHEROKEE LETTER YA" #x13EF)
    ("CHEROKEE LETTER YE" #x13F0)
    ("CHEROKEE LETTER YI" #x13F1)
    ("CHEROKEE LETTER YO" #x13F2)
    ("CHEROKEE LETTER YU" #x13F3)
    ("CHEROKEE LETTER YV" #x13F4)
    ))

;; arch-tag: 42d669f3-aa0a-4b38-8f60-c1bf05b0d8be
