(nxml-define-char-name-set 'tagbanwa
  '(("TAGBANWA LETTER A" #x1760)
    ("TAGBANWA LETTER I" #x1761)
    ("TAGBANWA LETTER U" #x1762)
    ("TAGBANWA LETTER KA" #x1763)
    ("TAGBANWA LETTER GA" #x1764)
    ("TAGBANWA LETTER NGA" #x1765)
    ("TAGBANWA LETTER TA" #x1766)
    ("TAGBANWA LETTER DA" #x1767)
    ("TAGBANWA LETTER NA" #x1768)
    ("TAGBANWA LETTER PA" #x1769)
    ("TAGBANWA LETTER BA" #x176A)
    ("TAGBANWA LETTER MA" #x176B)
    ("TAGBANWA LETTER YA" #x176C)
    ("TAGBANWA LETTER LA" #x176E)
    ("TAGBANWA LETTER WA" #x176F)
    ("TAGBANWA LETTER SA" #x1770)
    ("TAGBANWA VOWEL SIGN I" #x1772)
    ("TAGBANWA VOWEL SIGN U" #x1773)
    ))

;; arch-tag: 7ec84a4f-8077-400f-9b1a-f31b6d75fb65
