(nxml-define-char-name-set 'miscellaneous-mathematical-symbols-a
  '(("WHITE DIAMOND WITH CENTRED DOT" #x27D0)
    ("AND WITH DOT" #x27D1)
    ("ELEMENT OF OPENING UPWARDS" #x27D2)
    ("LOWER RIGHT CORNER WITH DOT" #x27D3)
    ("UPPER LEFT CORNER WITH DOT" #x27D4)
    ("LEFT OUTER JOIN" #x27D5)
    ("RIGHT OUTER JOIN" #x27D6)
    ("FULL OUTER JOIN" #x27D7)
    ("LARGE UP TACK" #x27D8)
    ("LARGE DOWN TACK" #x27D9)
    ("LEFT AND RIGHT DOUBLE TURNSTILE" #x27DA)
    ("LEFT AND RIGHT TACK" #x27DB)
    ("LEFT MULTIMAP" #x27DC)
    ("LONG RIGHT TACK" #x27DD)
    ("LONG LEFT TACK" #x27DE)
    ("UP TACK WITH CIRCLE ABOVE" #x27DF)
    ("LOZENGE DIVIDED BY HORIZONTAL RULE" #x27E0)
    ("WHITE CONCAVE-SIDED DIAMOND" #x27E1)
    ("WHITE CONCAVE-SIDED DIAMOND WITH LEFTWARDS TICK" #x27E2)
    ("WHITE CONCAVE-SIDED DIAMOND WITH RIGHTWARDS TICK" #x27E3)
    ("WHITE SQUARE WITH LEFTWARDS TICK" #x27E4)
    ("WHITE SQUARE WITH RIGHTWARDS TICK" #x27E5)
    ("MATHEMATICAL LEFT WHITE SQUARE BRACKET" #x27E6)
    ("MATHEMATICAL RIGHT WHITE SQUARE BRACKET" #x27E7)
    ("MATHEMATICAL LEFT ANGLE BRACKET" #x27E8)
    ("MATHEMATICAL RIGHT ANGLE BRACKET" #x27E9)
    ("MATHEMATICAL LEFT DOUBLE ANGLE BRACKET" #x27EA)
    ("MATHEMATICAL RIGHT DOUBLE ANGLE BRACKET" #x27EB)
    ))

;; arch-tag: f66b9857-6560-4ad9-9adb-1f3b5fb079de
