(nxml-define-char-name-set 'cjk-compatibility
  '(("SQUARE APAATO" #x3300)
    ("SQUARE ARUHUA" #x3301)
    ("SQUARE ANPEA" #x3302)
    ("SQUARE AARU" #x3303)
    ("SQUARE ININGU" #x3304)
    ("SQUARE INTI" #x3305)
    ("SQUARE UON" #x3306)
    ("SQUARE ESUKUUDO" #x3307)
    ("SQUARE EEKAA" #x3308)
    ("SQUARE ONSU" #x3309)
    ("SQUARE OOMU" #x330A)
    ("SQUARE KAIRI" #x330B)
    ("SQUARE KARATTO" #x330C)
    ("SQUARE KARORII" #x330D)
    ("SQUARE GARON" #x330E)
    ("SQUARE GANMA" #x330F)
    ("SQUARE GIGA" #x3310)
    ("SQUARE GINII" #x3311)
    ("SQUARE KYURII" #x3312)
    ("SQUARE GIRUDAA" #x3313)
    ("SQUARE KIRO" #x3314)
    ("SQUARE KIROGURAMU" #x3315)
    ("SQUARE KIROMEETORU" #x3316)
    ("SQUARE KIROWATTO" #x3317)
    ("SQUARE GURAMU" #x3318)
    ("SQUARE GURAMUTON" #x3319)
    ("SQUARE KURUZEIRO" #x331A)
    ("SQUARE KUROONE" #x331B)
    ("SQUARE KEESU" #x331C)
    ("SQUARE KORUNA" #x331D)
    ("SQUARE KOOPO" #x331E)
    ("SQUARE SAIKURU" #x331F)
    ("SQUARE SANTIIMU" #x3320)
    ("SQUARE SIRINGU" #x3321)
    ("SQUARE SENTI" #x3322)
    ("SQUARE SENTO" #x3323)
    ("SQUARE DAASU" #x3324)
    ("SQUARE DESI" #x3325)
    ("SQUARE DORU" #x3326)
    ("SQUARE TON" #x3327)
    ("SQUARE NANO" #x3328)
    ("SQUARE NOTTO" #x3329)
    ("SQUARE HAITU" #x332A)
    ("SQUARE PAASENTO" #x332B)
    ("SQUARE PAATU" #x332C)
    ("SQUARE BAARERU" #x332D)
    ("SQUARE PIASUTORU" #x332E)
    ("SQUARE PIKURU" #x332F)
    ("SQUARE PIKO" #x3330)
    ("SQUARE BIRU" #x3331)
    ("SQUARE HUARADDO" #x3332)
    ("SQUARE HUIITO" #x3333)
    ("SQUARE BUSSYERU" #x3334)
    ("SQUARE HURAN" #x3335)
    ("SQUARE HEKUTAARU" #x3336)
    ("SQUARE PESO" #x3337)
    ("SQUARE PENIHI" #x3338)
    ("SQUARE HERUTU" #x3339)
    ("SQUARE PENSU" #x333A)
    ("SQUARE PEEZI" #x333B)
    ("SQUARE BEETA" #x333C)
    ("SQUARE POINTO" #x333D)
    ("SQUARE BORUTO" #x333E)
    ("SQUARE HON" #x333F)
    ("SQUARE PONDO" #x3340)
    ("SQUARE HOORU" #x3341)
    ("SQUARE HOON" #x3342)
    ("SQUARE MAIKURO" #x3343)
    ("SQUARE MAIRU" #x3344)
    ("SQUARE MAHHA" #x3345)
    ("SQUARE MARUKU" #x3346)
    ("SQUARE MANSYON" #x3347)
    ("SQUARE MIKURON" #x3348)
    ("SQUARE MIRI" #x3349)
    ("SQUARE MIRIBAARU" #x334A)
    ("SQUARE MEGA" #x334B)
    ("SQUARE MEGATON" #x334C)
    ("SQUARE MEETORU" #x334D)
    ("SQUARE YAADO" #x334E)
    ("SQUARE YAARU" #x334F)
    ("SQUARE YUAN" #x3350)
    ("SQUARE RITTORU" #x3351)
    ("SQUARE RIRA" #x3352)
    ("SQUARE RUPII" #x3353)
    ("SQUARE RUUBURU" #x3354)
    ("SQUARE REMU" #x3355)
    ("SQUARE RENTOGEN" #x3356)
    ("SQUARE WATTO" #x3357)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ZERO" #x3358)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ONE" #x3359)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWO" #x335A)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THREE" #x335B)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOUR" #x335C)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIVE" #x335D)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIX" #x335E)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVEN" #x335F)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHT" #x3360)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINE" #x3361)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TEN" #x3362)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ELEVEN" #x3363)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWELVE" #x3364)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THIRTEEN" #x3365)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOURTEEN" #x3366)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIFTEEN" #x3367)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIXTEEN" #x3368)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVENTEEN" #x3369)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHTEEN" #x336A)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINETEEN" #x336B)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY" #x336C)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-ONE" #x336D)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-TWO" #x336E)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-THREE" #x336F)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-FOUR" #x3370)
    ("SQUARE HPA" #x3371)
    ("SQUARE DA" #x3372)
    ("SQUARE AU" #x3373)
    ("SQUARE BAR" #x3374)
    ("SQUARE OV" #x3375)
    ("SQUARE PC" #x3376)
    ("SQUARE ERA NAME HEISEI" #x337B)
    ("SQUARE ERA NAME SYOUWA" #x337C)
    ("SQUARE ERA NAME TAISYOU" #x337D)
    ("SQUARE ERA NAME MEIZI" #x337E)
    ("SQUARE CORPORATION" #x337F)
    ("SQUARE PA AMPS" #x3380)
    ("SQUARE NA" #x3381)
    ("SQUARE MU A" #x3382)
    ("SQUARE MA" #x3383)
    ("SQUARE KA" #x3384)
    ("SQUARE KB" #x3385)
    ("SQUARE MB" #x3386)
    ("SQUARE GB" #x3387)
    ("SQUARE CAL" #x3388)
    ("SQUARE KCAL" #x3389)
    ("SQUARE PF" #x338A)
    ("SQUARE NF" #x338B)
    ("SQUARE MU F" #x338C)
    ("SQUARE MU G" #x338D)
    ("SQUARE MG" #x338E)
    ("SQUARE KG" #x338F)
    ("SQUARE HZ" #x3390)
    ("SQUARE KHZ" #x3391)
    ("SQUARE MHZ" #x3392)
    ("SQUARE GHZ" #x3393)
    ("SQUARE THZ" #x3394)
    ("SQUARE MU L" #x3395)
    ("SQUARE ML" #x3396)
    ("SQUARE DL" #x3397)
    ("SQUARE KL" #x3398)
    ("SQUARE FM" #x3399)
    ("SQUARE NM" #x339A)
    ("SQUARE MU M" #x339B)
    ("SQUARE MM" #x339C)
    ("SQUARE CM" #x339D)
    ("SQUARE KM" #x339E)
    ("SQUARE MM SQUARED" #x339F)
    ("SQUARE CM SQUARED" #x33A0)
    ("SQUARE M SQUARED" #x33A1)
    ("SQUARE KM SQUARED" #x33A2)
    ("SQUARE MM CUBED" #x33A3)
    ("SQUARE CM CUBED" #x33A4)
    ("SQUARE M CUBED" #x33A5)
    ("SQUARE KM CUBED" #x33A6)
    ("SQUARE M OVER S" #x33A7)
    ("SQUARE M OVER S SQUARED" #x33A8)
    ("SQUARE PA" #x33A9)
    ("SQUARE KPA" #x33AA)
    ("SQUARE MPA" #x33AB)
    ("SQUARE GPA" #x33AC)
    ("SQUARE RAD" #x33AD)
    ("SQUARE RAD OVER S" #x33AE)
    ("SQUARE RAD OVER S SQUARED" #x33AF)
    ("SQUARE PS" #x33B0)
    ("SQUARE NS" #x33B1)
    ("SQUARE MU S" #x33B2)
    ("SQUARE MS" #x33B3)
    ("SQUARE PV" #x33B4)
    ("SQUARE NV" #x33B5)
    ("SQUARE MU V" #x33B6)
    ("SQUARE MV" #x33B7)
    ("SQUARE KV" #x33B8)
    ("SQUARE MV MEGA" #x33B9)
    ("SQUARE PW" #x33BA)
    ("SQUARE NW" #x33BB)
    ("SQUARE MU W" #x33BC)
    ("SQUARE MW" #x33BD)
    ("SQUARE KW" #x33BE)
    ("SQUARE MW MEGA" #x33BF)
    ("SQUARE K OHM" #x33C0)
    ("SQUARE M OHM" #x33C1)
    ("SQUARE AM" #x33C2)
    ("SQUARE BQ" #x33C3)
    ("SQUARE CC" #x33C4)
    ("SQUARE CD" #x33C5)
    ("SQUARE C OVER KG" #x33C6)
    ("SQUARE CO" #x33C7)
    ("SQUARE DB" #x33C8)
    ("SQUARE GY" #x33C9)
    ("SQUARE HA" #x33CA)
    ("SQUARE HP" #x33CB)
    ("SQUARE IN" #x33CC)
    ("SQUARE KK" #x33CD)
    ("SQUARE KM CAPITAL" #x33CE)
    ("SQUARE KT" #x33CF)
    ("SQUARE LM" #x33D0)
    ("SQUARE LN" #x33D1)
    ("SQUARE LOG" #x33D2)
    ("SQUARE LX" #x33D3)
    ("SQUARE MB SMALL" #x33D4)
    ("SQUARE MIL" #x33D5)
    ("SQUARE MOL" #x33D6)
    ("SQUARE PH" #x33D7)
    ("SQUARE PM" #x33D8)
    ("SQUARE PPM" #x33D9)
    ("SQUARE PR" #x33DA)
    ("SQUARE SR" #x33DB)
    ("SQUARE SV" #x33DC)
    ("SQUARE WB" #x33DD)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ONE" #x33E0)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWO" #x33E1)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THREE" #x33E2)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOUR" #x33E3)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIVE" #x33E4)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIX" #x33E5)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVEN" #x33E6)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHT" #x33E7)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINE" #x33E8)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TEN" #x33E9)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ELEVEN" #x33EA)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWELVE" #x33EB)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTEEN" #x33EC)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOURTEEN" #x33ED)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIFTEEN" #x33EE)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIXTEEN" #x33EF)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVENTEEN" #x33F0)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHTEEN" #x33F1)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINETEEN" #x33F2)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY" #x33F3)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-ONE" #x33F4)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-TWO" #x33F5)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-THREE" #x33F6)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FOUR" #x33F7)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FIVE" #x33F8)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SIX" #x33F9)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SEVEN" #x33FA)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-EIGHT" #x33FB)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-NINE" #x33FC)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY" #x33FD)
    ("IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY-ONE" #x33FE)
    ))

;; arch-tag: 818090a1-7854-485f-80bc-97e89f733e75
