(nxml-define-char-name-set 'combining-diacritical-marks-for-symbols
  '(("COMBINING LEFT HARPOON ABOVE" #x20D0)
    ("COMBINING RIGHT HARPOON ABOVE" #x20D1)
    ("COMBINING LONG VERTICAL LINE OVERLAY" #x20D2)
    ("COMBINING SHORT VERTICAL LINE OVERLAY" #x20D3)
    ("COMBINING ANTICLOCKWISE ARROW ABOVE" #x20D4)
    ("COMBINING CLOCKWISE ARROW ABOVE" #x20D5)
    ("COMBINING LEFT ARROW ABOVE" #x20D6)
    ("COMBINING RIGHT ARROW ABOVE" #x20D7)
    ("COMBINING RING OVERLAY" #x20D8)
    ("COMBINING CLOCKWISE RING OVERLAY" #x20D9)
    ("COMBINING ANTICLOCKWISE RING OVERLAY" #x20DA)
    ("COMBINING THREE DOTS ABOVE" #x20DB)
    ("COMBINING FOUR DOTS ABOVE" #x20DC)
    ("COMBINING ENCLOSING CIRCLE" #x20DD)
    ("COMBINING ENCLOSING SQUARE" #x20DE)
    ("COMBINING ENCLOSING DIAMOND" #x20DF)
    ("COMBINING ENCLOSING CIRCLE BACKSLASH" #x20E0)
    ("COMBINING LEFT RIGHT ARROW ABOVE" #x20E1)
    ("COMBINING ENCLOSING SCREEN" #x20E2)
    ("COMBINING ENCLOSING KEYCAP" #x20E3)
    ("COMBINING ENCLOSING UPWARD POINTING TRIANGLE" #x20E4)
    ("COMBINING REVERSE SOLIDUS OVERLAY" #x20E5)
    ("COMBINING DOUBLE VERTICAL STROKE OVERLAY" #x20E6)
    ("COMBINING ANNUITY SYMBOL" #x20E7)
    ("COMBINING TRIPLE UNDERDOT" #x20E8)
    ("COMBINING WIDE BRIDGE ABOVE" #x20E9)
    ("COMBINING LEFTWARDS ARROW OVERLAY" #x20EA)
    ))

;; arch-tag: 4b1f7ef0-287b-49a6-a9f2-910752aac517
