(nxml-define-char-name-set 'supplemental-mathematical-operators
  '(("N-ARY CIRCLED DOT OPERATOR" #x2A00)
    ("N-ARY CIRCLED PLUS OPERATOR" #x2A01)
    ("N-ARY CIRCLED TIMES OPERATOR" #x2A02)
    ("N-ARY UNION OPERATOR WITH DOT" #x2A03)
    ("N-ARY UNION OPERATOR WITH PLUS" #x2A04)
    ("N-ARY SQUARE INTERSECTION OPERATOR" #x2A05)
    ("N-ARY SQUARE UNION OPERATOR" #x2A06)
    ("TWO LOGICAL AND OPERATOR" #x2A07)
    ("TWO LOGICAL OR OPERATOR" #x2A08)
    ("N-ARY TIMES OPERATOR" #x2A09)
    ("MODULO TWO SUM" #x2A0A)
    ("SUMMATION WITH INTEGRAL" #x2A0B)
    ("QUADRUPLE INTEGRAL OPERATOR" #x2A0C)
    ("FINITE PART INTEGRAL" #x2A0D)
    ("INTEGRAL WITH DOUBLE STROKE" #x2A0E)
    ("INTEGRAL AVERAGE WITH SLASH" #x2A0F)
    ("CIRCULATION FUNCTION" #x2A10)
    ("ANTICLOCKWISE INTEGRATION" #x2A11)
    ("LINE INTEGRATION WITH RECTANGULAR PATH AROUND POLE" #x2A12)
    ("LINE INTEGRATION WITH SEMICIRCULAR PATH AROUND POLE" #x2A13)
    ("LINE INTEGRATION NOT INCLUDING THE POLE" #x2A14)
    ("INTEGRAL AROUND A POINT OPERATOR" #x2A15)
    ("QUATERNION INTEGRAL OPERATOR" #x2A16)
    ("INTEGRAL WITH LEFTWARDS ARROW WITH HOOK" #x2A17)
    ("INTEGRAL WITH TIMES SIGN" #x2A18)
    ("INTEGRAL WITH INTERSECTION" #x2A19)
    ("INTEGRAL WITH UNION" #x2A1A)
    ("INTEGRAL WITH OVERBAR" #x2A1B)
    ("INTEGRAL WITH UNDERBAR" #x2A1C)
    ("JOIN" #x2A1D)
    ("LARGE LEFT TRIANGLE OPERATOR" #x2A1E)
    ("Z NOTATION SCHEMA COMPOSITION" #x2A1F)
    ("Z NOTATION SCHEMA PIPING" #x2A20)
    ("Z NOTATION SCHEMA PROJECTION" #x2A21)
    ("PLUS SIGN WITH SMALL CIRCLE ABOVE" #x2A22)
    ("PLUS SIGN WITH CIRCUMFLEX ACCENT ABOVE" #x2A23)
    ("PLUS SIGN WITH TILDE ABOVE" #x2A24)
    ("PLUS SIGN WITH DOT BELOW" #x2A25)
    ("PLUS SIGN WITH TILDE BELOW" #x2A26)
    ("PLUS SIGN WITH SUBSCRIPT TWO" #x2A27)
    ("PLUS SIGN WITH BLACK TRIANGLE" #x2A28)
    ("MINUS SIGN WITH COMMA ABOVE" #x2A29)
    ("MINUS SIGN WITH DOT BELOW" #x2A2A)
    ("MINUS SIGN WITH FALLING DOTS" #x2A2B)
    ("MINUS SIGN WITH RISING DOTS" #x2A2C)
    ("PLUS SIGN IN LEFT HALF CIRCLE" #x2A2D)
    ("PLUS SIGN IN RIGHT HALF CIRCLE" #x2A2E)
    ("VECTOR OR CROSS PRODUCT" #x2A2F)
    ("MULTIPLICATION SIGN WITH DOT ABOVE" #x2A30)
    ("MULTIPLICATION SIGN WITH UNDERBAR" #x2A31)
    ("SEMIDIRECT PRODUCT WITH BOTTOM CLOSED" #x2A32)
    ("SMASH PRODUCT" #x2A33)
    ("MULTIPLICATION SIGN IN LEFT HALF CIRCLE" #x2A34)
    ("MULTIPLICATION SIGN IN RIGHT HALF CIRCLE" #x2A35)
    ("CIRCLED MULTIPLICATION SIGN WITH CIRCUMFLEX ACCENT" #x2A36)
    ("MULTIPLICATION SIGN IN DOUBLE CIRCLE" #x2A37)
    ("CIRCLED DIVISION SIGN" #x2A38)
    ("PLUS SIGN IN TRIANGLE" #x2A39)
    ("MINUS SIGN IN TRIANGLE" #x2A3A)
    ("MULTIPLICATION SIGN IN TRIANGLE" #x2A3B)
    ("INTERIOR PRODUCT" #x2A3C)
    ("RIGHTHAND INTERIOR PRODUCT" #x2A3D)
    ("Z NOTATION RELATIONAL COMPOSITION" #x2A3E)
    ("AMALGAMATION OR COPRODUCT" #x2A3F)
    ("INTERSECTION WITH DOT" #x2A40)
    ("UNION WITH MINUS SIGN" #x2A41)
    ("UNION WITH OVERBAR" #x2A42)
    ("INTERSECTION WITH OVERBAR" #x2A43)
    ("INTERSECTION WITH LOGICAL AND" #x2A44)
    ("UNION WITH LOGICAL OR" #x2A45)
    ("UNION ABOVE INTERSECTION" #x2A46)
    ("INTERSECTION ABOVE UNION" #x2A47)
    ("UNION ABOVE BAR ABOVE INTERSECTION" #x2A48)
    ("INTERSECTION ABOVE BAR ABOVE UNION" #x2A49)
    ("UNION BESIDE AND JOINED WITH UNION" #x2A4A)
    ("INTERSECTION BESIDE AND JOINED WITH INTERSECTION" #x2A4B)
    ("CLOSED UNION WITH SERIFS" #x2A4C)
    ("CLOSED INTERSECTION WITH SERIFS" #x2A4D)
    ("DOUBLE SQUARE INTERSECTION" #x2A4E)
    ("DOUBLE SQUARE UNION" #x2A4F)
    ("CLOSED UNION WITH SERIFS AND SMASH PRODUCT" #x2A50)
    ("LOGICAL AND WITH DOT ABOVE" #x2A51)
    ("LOGICAL OR WITH DOT ABOVE" #x2A52)
    ("DOUBLE LOGICAL AND" #x2A53)
    ("DOUBLE LOGICAL OR" #x2A54)
    ("TWO INTERSECTING LOGICAL AND" #x2A55)
    ("TWO INTERSECTING LOGICAL OR" #x2A56)
    ("SLOPING LARGE OR" #x2A57)
    ("SLOPING LARGE AND" #x2A58)
    ("LOGICAL OR OVERLAPPING LOGICAL AND" #x2A59)
    ("LOGICAL AND WITH MIDDLE STEM" #x2A5A)
    ("LOGICAL OR WITH MIDDLE STEM" #x2A5B)
    ("LOGICAL AND WITH HORIZONTAL DASH" #x2A5C)
    ("LOGICAL OR WITH HORIZONTAL DASH" #x2A5D)
    ("LOGICAL AND WITH DOUBLE OVERBAR" #x2A5E)
    ("LOGICAL AND WITH UNDERBAR" #x2A5F)
    ("LOGICAL AND WITH DOUBLE UNDERBAR" #x2A60)
    ("SMALL VEE WITH UNDERBAR" #x2A61)
    ("LOGICAL OR WITH DOUBLE OVERBAR" #x2A62)
    ("LOGICAL OR WITH DOUBLE UNDERBAR" #x2A63)
    ("Z NOTATION DOMAIN ANTIRESTRICTION" #x2A64)
    ("Z NOTATION RANGE ANTIRESTRICTION" #x2A65)
    ("EQUALS SIGN WITH DOT BELOW" #x2A66)
    ("IDENTICAL WITH DOT ABOVE" #x2A67)
    ("TRIPLE HORIZONTAL BAR WITH DOUBLE VERTICAL STROKE" #x2A68)
    ("TRIPLE HORIZONTAL BAR WITH TRIPLE VERTICAL STROKE" #x2A69)
    ("TILDE OPERATOR WITH DOT ABOVE" #x2A6A)
    ("TILDE OPERATOR WITH RISING DOTS" #x2A6B)
    ("SIMILAR MINUS SIMILAR" #x2A6C)
    ("CONGRUENT WITH DOT ABOVE" #x2A6D)
    ("EQUALS WITH ASTERISK" #x2A6E)
    ("ALMOST EQUAL TO WITH CIRCUMFLEX ACCENT" #x2A6F)
    ("APPROXIMATELY EQUAL OR EQUAL TO" #x2A70)
    ("EQUALS SIGN ABOVE PLUS SIGN" #x2A71)
    ("PLUS SIGN ABOVE EQUALS SIGN" #x2A72)
    ("EQUALS SIGN ABOVE TILDE OPERATOR" #x2A73)
    ("DOUBLE COLON EQUAL" #x2A74)
    ("TWO CONSECUTIVE EQUALS SIGNS" #x2A75)
    ("THREE CONSECUTIVE EQUALS SIGNS" #x2A76)
    ("EQUALS SIGN WITH TWO DOTS ABOVE AND TWO DOTS BELOW" #x2A77)
    ("EQUIVALENT WITH FOUR DOTS ABOVE" #x2A78)
    ("LESS-THAN WITH CIRCLE INSIDE" #x2A79)
    ("GREATER-THAN WITH CIRCLE INSIDE" #x2A7A)
    ("LESS-THAN WITH QUESTION MARK ABOVE" #x2A7B)
    ("GREATER-THAN WITH QUESTION MARK ABOVE" #x2A7C)
    ("LESS-THAN OR SLANTED EQUAL TO" #x2A7D)
    ("GREATER-THAN OR SLANTED EQUAL TO" #x2A7E)
    ("LESS-THAN OR SLANTED EQUAL TO WITH DOT INSIDE" #x2A7F)
    ("GREATER-THAN OR SLANTED EQUAL TO WITH DOT INSIDE" #x2A80)
    ("LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE" #x2A81)
    ("GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE" #x2A82)
    ("LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE RIGHT" #x2A83)
    ("GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE LEFT" #x2A84)
    ("LESS-THAN OR APPROXIMATE" #x2A85)
    ("GREATER-THAN OR APPROXIMATE" #x2A86)
    ("LESS-THAN AND SINGLE-LINE NOT EQUAL TO" #x2A87)
    ("GREATER-THAN AND SINGLE-LINE NOT EQUAL TO" #x2A88)
    ("LESS-THAN AND NOT APPROXIMATE" #x2A89)
    ("GREATER-THAN AND NOT APPROXIMATE" #x2A8A)
    ("LESS-THAN ABOVE DOUBLE-LINE EQUAL ABOVE GREATER-THAN" #x2A8B)
    ("GREATER-THAN ABOVE DOUBLE-LINE EQUAL ABOVE LESS-THAN" #x2A8C)
    ("LESS-THAN ABOVE SIMILAR OR EQUAL" #x2A8D)
    ("GREATER-THAN ABOVE SIMILAR OR EQUAL" #x2A8E)
    ("LESS-THAN ABOVE SIMILAR ABOVE GREATER-THAN" #x2A8F)
    ("GREATER-THAN ABOVE SIMILAR ABOVE LESS-THAN" #x2A90)
    ("LESS-THAN ABOVE GREATER-THAN ABOVE DOUBLE-LINE EQUAL" #x2A91)
    ("GREATER-THAN ABOVE LESS-THAN ABOVE DOUBLE-LINE EQUAL" #x2A92)
    ("LESS-THAN ABOVE SLANTED EQUAL ABOVE GREATER-THAN ABOVE SLANTED EQUAL" #x2A93)
    ("GREATER-THAN ABOVE SLANTED EQUAL ABOVE LESS-THAN ABOVE SLANTED EQUAL" #x2A94)
    ("SLANTED EQUAL TO OR LESS-THAN" #x2A95)
    ("SLANTED EQUAL TO OR GREATER-THAN" #x2A96)
    ("SLANTED EQUAL TO OR LESS-THAN WITH DOT INSIDE" #x2A97)
    ("SLANTED EQUAL TO OR GREATER-THAN WITH DOT INSIDE" #x2A98)
    ("DOUBLE-LINE EQUAL TO OR LESS-THAN" #x2A99)
    ("DOUBLE-LINE EQUAL TO OR GREATER-THAN" #x2A9A)
    ("DOUBLE-LINE SLANTED EQUAL TO OR LESS-THAN" #x2A9B)
    ("DOUBLE-LINE SLANTED EQUAL TO OR GREATER-THAN" #x2A9C)
    ("SIMILAR OR LESS-THAN" #x2A9D)
    ("SIMILAR OR GREATER-THAN" #x2A9E)
    ("SIMILAR ABOVE LESS-THAN ABOVE EQUALS SIGN" #x2A9F)
    ("SIMILAR ABOVE GREATER-THAN ABOVE EQUALS SIGN" #x2AA0)
    ("DOUBLE NESTED LESS-THAN" #x2AA1)
    ("DOUBLE NESTED GREATER-THAN" #x2AA2)
    ("DOUBLE NESTED LESS-THAN WITH UNDERBAR" #x2AA3)
    ("GREATER-THAN OVERLAPPING LESS-THAN" #x2AA4)
    ("GREATER-THAN BESIDE LESS-THAN" #x2AA5)
    ("LESS-THAN CLOSED BY CURVE" #x2AA6)
    ("GREATER-THAN CLOSED BY CURVE" #x2AA7)
    ("LESS-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL" #x2AA8)
    ("GREATER-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL" #x2AA9)
    ("SMALLER THAN" #x2AAA)
    ("LARGER THAN" #x2AAB)
    ("SMALLER THAN OR EQUAL TO" #x2AAC)
    ("LARGER THAN OR EQUAL TO" #x2AAD)
    ("EQUALS SIGN WITH BUMPY ABOVE" #x2AAE)
    ("PRECEDES ABOVE SINGLE-LINE EQUALS SIGN" #x2AAF)
    ("SUCCEEDS ABOVE SINGLE-LINE EQUALS SIGN" #x2AB0)
    ("PRECEDES ABOVE SINGLE-LINE NOT EQUAL TO" #x2AB1)
    ("SUCCEEDS ABOVE SINGLE-LINE NOT EQUAL TO" #x2AB2)
    ("PRECEDES ABOVE EQUALS SIGN" #x2AB3)
    ("SUCCEEDS ABOVE EQUALS SIGN" #x2AB4)
    ("PRECEDES ABOVE NOT EQUAL TO" #x2AB5)
    ("SUCCEEDS ABOVE NOT EQUAL TO" #x2AB6)
    ("PRECEDES ABOVE ALMOST EQUAL TO" #x2AB7)
    ("SUCCEEDS ABOVE ALMOST EQUAL TO" #x2AB8)
    ("PRECEDES ABOVE NOT ALMOST EQUAL TO" #x2AB9)
    ("SUCCEEDS ABOVE NOT ALMOST EQUAL TO" #x2ABA)
    ("DOUBLE PRECEDES" #x2ABB)
    ("DOUBLE SUCCEEDS" #x2ABC)
    ("SUBSET WITH DOT" #x2ABD)
    ("SUPERSET WITH DOT" #x2ABE)
    ("SUBSET WITH PLUS SIGN BELOW" #x2ABF)
    ("SUPERSET WITH PLUS SIGN BELOW" #x2AC0)
    ("SUBSET WITH MULTIPLICATION SIGN BELOW" #x2AC1)
    ("SUPERSET WITH MULTIPLICATION SIGN BELOW" #x2AC2)
    ("SUBSET OF OR EQUAL TO WITH DOT ABOVE" #x2AC3)
    ("SUPERSET OF OR EQUAL TO WITH DOT ABOVE" #x2AC4)
    ("SUBSET OF ABOVE EQUALS SIGN" #x2AC5)
    ("SUPERSET OF ABOVE EQUALS SIGN" #x2AC6)
    ("SUBSET OF ABOVE TILDE OPERATOR" #x2AC7)
    ("SUPERSET OF ABOVE TILDE OPERATOR" #x2AC8)
    ("SUBSET OF ABOVE ALMOST EQUAL TO" #x2AC9)
    ("SUPERSET OF ABOVE ALMOST EQUAL TO" #x2ACA)
    ("SUBSET OF ABOVE NOT EQUAL TO" #x2ACB)
    ("SUPERSET OF ABOVE NOT EQUAL TO" #x2ACC)
    ("SQUARE LEFT OPEN BOX OPERATOR" #x2ACD)
    ("SQUARE RIGHT OPEN BOX OPERATOR" #x2ACE)
    ("CLOSED SUBSET" #x2ACF)
    ("CLOSED SUPERSET" #x2AD0)
    ("CLOSED SUBSET OR EQUAL TO" #x2AD1)
    ("CLOSED SUPERSET OR EQUAL TO" #x2AD2)
    ("SUBSET ABOVE SUPERSET" #x2AD3)
    ("SUPERSET ABOVE SUBSET" #x2AD4)
    ("SUBSET ABOVE SUBSET" #x2AD5)
    ("SUPERSET ABOVE SUPERSET" #x2AD6)
    ("SUPERSET BESIDE SUBSET" #x2AD7)
    ("SUPERSET BESIDE AND JOINED BY DASH WITH SUBSET" #x2AD8)
    ("ELEMENT OF OPENING DOWNWARDS" #x2AD9)
    ("PITCHFORK WITH TEE TOP" #x2ADA)
    ("TRANSVERSAL INTERSECTION" #x2ADB)
    ("FORKING" #x2ADC)
    ("NONFORKING" #x2ADD)
    ("SHORT LEFT TACK" #x2ADE)
    ("SHORT DOWN TACK" #x2ADF)
    ("SHORT UP TACK" #x2AE0)
    ("PERPENDICULAR WITH S" #x2AE1)
    ("VERTICAL BAR TRIPLE RIGHT TURNSTILE" #x2AE2)
    ("DOUBLE VERTICAL BAR LEFT TURNSTILE" #x2AE3)
    ("VERTICAL BAR DOUBLE LEFT TURNSTILE" #x2AE4)
    ("DOUBLE VERTICAL BAR DOUBLE LEFT TURNSTILE" #x2AE5)
    ("LONG DASH FROM LEFT MEMBER OF DOUBLE VERTICAL" #x2AE6)
    ("SHORT DOWN TACK WITH OVERBAR" #x2AE7)
    ("SHORT UP TACK WITH UNDERBAR" #x2AE8)
    ("SHORT UP TACK ABOVE SHORT DOWN TACK" #x2AE9)
    ("DOUBLE DOWN TACK" #x2AEA)
    ("DOUBLE UP TACK" #x2AEB)
    ("DOUBLE STROKE NOT SIGN" #x2AEC)
    ("REVERSED DOUBLE STROKE NOT SIGN" #x2AED)
    ("DOES NOT DIVIDE WITH REVERSED NEGATION SLASH" #x2AEE)
    ("VERTICAL LINE WITH CIRCLE ABOVE" #x2AEF)
    ("VERTICAL LINE WITH CIRCLE BELOW" #x2AF0)
    ("DOWN TACK WITH CIRCLE BELOW" #x2AF1)
    ("PARALLEL WITH HORIZONTAL STROKE" #x2AF2)
    ("PARALLEL WITH TILDE OPERATOR" #x2AF3)
    ("TRIPLE VERTICAL BAR BINARY RELATION" #x2AF4)
    ("TRIPLE VERTICAL BAR WITH HORIZONTAL STROKE" #x2AF5)
    ("TRIPLE COLON OPERATOR" #x2AF6)
    ("TRIPLE NESTED LESS-THAN" #x2AF7)
    ("TRIPLE NESTED GREATER-THAN" #x2AF8)
    ("DOUBLE-LINE SLANTED LESS-THAN OR EQUAL TO" #x2AF9)
    ("DOUBLE-LINE SLANTED GREATER-THAN OR EQUAL TO" #x2AFA)
    ("TRIPLE SOLIDUS BINARY RELATION" #x2AFB)
    ("LARGE TRIPLE VERTICAL BAR OPERATOR" #x2AFC)
    ("DOUBLE SOLIDUS OPERATOR" #x2AFD)
    ("WHITE VERTICAL BAR" #x2AFE)
    ("N-ARY WHITE VERTICAL BAR" #x2AFF)
    ))

;; arch-tag: 672a8fbf-fb4a-4862-8b8e-2379b799484c
