(nxml-define-char-name-set 'cjk-symbols-and-punctuation
  '(("IDEOGRAPHIC SPACE" #x3000)
    ("IDEOGRAPHIC COMMA" #x3001)
    ("IDEOGRAPHIC FULL STOP" #x3002)
    ("DITTO MARK" #x3003)
    ("JAPANESE INDUSTRIAL STANDARD SYMBOL" #x3004)
    ("IDEOGRAPHIC ITERATION MARK" #x3005)
    ("IDEOGRAPHIC CLOSING MARK" #x3006)
    ("IDEOGRAPHIC NUMBER ZERO" #x3007)
    ("LEFT ANGLE BRACKET" #x3008)
    ("RIGHT ANGLE BRACKET" #x3009)
    ("LEFT DOUBLE ANGLE BRACKET" #x300A)
    ("RIGHT DOUBLE ANGLE BRACKET" #x300B)
    ("LEFT CORNER BRACKET" #x300C)
    ("RIGHT CORNER BRACKET" #x300D)
    ("LEFT WHITE CORNER BRACKET" #x300E)
    ("RIGHT WHITE CORNER BRACKET" #x300F)
    ("LEFT BLACK LENTICULAR BRACKET" #x3010)
    ("RIGHT BLACK LENTICULAR BRACKET" #x3011)
    ("POSTAL MARK" #x3012)
    ("GETA MARK" #x3013)
    ("LEFT TORTOISE SHELL BRACKET" #x3014)
    ("RIGHT TORTOISE SHELL BRACKET" #x3015)
    ("LEFT WHITE LENTICULAR BRACKET" #x3016)
    ("RIGHT WHITE LENTICULAR BRACKET" #x3017)
    ("LEFT WHITE TORTOISE SHELL BRACKET" #x3018)
    ("RIGHT WHITE TORTOISE SHELL BRACKET" #x3019)
    ("LEFT WHITE SQUARE BRACKET" #x301A)
    ("RIGHT WHITE SQUARE BRACKET" #x301B)
    ("WAVE DASH" #x301C)
    ("REVERSED DOUBLE PRIME QUOTATION MARK" #x301D)
    ("DOUBLE PRIME QUOTATION MARK" #x301E)
    ("LOW DOUBLE PRIME QUOTATION MARK" #x301F)
    ("POSTAL MARK FACE" #x3020)
    ("HANGZHOU NUMERAL ONE" #x3021)
    ("HANGZHOU NUMERAL TWO" #x3022)
    ("HANGZHOU NUMERAL THREE" #x3023)
    ("HANGZHOU NUMERAL FOUR" #x3024)
    ("HANGZHOU NUMERAL FIVE" #x3025)
    ("HANGZHOU NUMERAL SIX" #x3026)
    ("HANGZHOU NUMERAL SEVEN" #x3027)
    ("HANGZHOU NUMERAL EIGHT" #x3028)
    ("HANGZHOU NUMERAL NINE" #x3029)
    ("IDEOGRAPHIC LEVEL TONE MARK" #x302A)
    ("IDEOGRAPHIC RISING TONE MARK" #x302B)
    ("IDEOGRAPHIC DEPARTING TONE MARK" #x302C)
    ("IDEOGRAPHIC ENTERING TONE MARK" #x302D)
    ("HANGUL SINGLE DOT TONE MARK" #x302E)
    ("HANGUL DOUBLE DOT TONE MARK" #x302F)
    ("WAVY DASH" #x3030)
    ("VERTICAL KANA REPEAT MARK" #x3031)
    ("VERTICAL KANA REPEAT WITH VOICED SOUND MARK" #x3032)
    ("VERTICAL KANA REPEAT MARK UPPER HALF" #x3033)
    ("VERTICAL KANA REPEAT WITH VOICED SOUND MARK UPPER HALF" #x3034)
    ("VERTICAL KANA REPEAT MARK LOWER HALF" #x3035)
    ("CIRCLED POSTAL MARK" #x3036)
    ("IDEOGRAPHIC TELEGRAPH LINE FEED SEPARATOR SYMBOL" #x3037)
    ("HANGZHOU NUMERAL TEN" #x3038)
    ("HANGZHOU NUMERAL TWENTY" #x3039)
    ("HANGZHOU NUMERAL THIRTY" #x303A)
    ("VERTICAL IDEOGRAPHIC ITERATION MARK" #x303B)
    ("MASU MARK" #x303C)
    ("PART ALTERNATION MARK" #x303D)
    ("IDEOGRAPHIC VARIATION INDICATOR" #x303E)
    ("IDEOGRAPHIC HALF FILL SPACE" #x303F)
    ))

;; arch-tag: 05689acc-1d48-4a36-9b60-75d5abf8a065
