(nxml-define-char-name-set 'latin-extended-b
  '(("LATIN SMALL LETTER B WITH STROKE" #x0180)
    ("LATIN CAPITAL LETTER B WITH HOOK" #x0181)
    ("LATIN CAPITAL LETTER B WITH TOPBAR" #x0182)
    ("LATIN SMALL LETTER B WITH TOPBAR" #x0183)
    ("LATIN CAPITAL LETTER TONE SIX" #x0184)
    ("LATIN SMALL LETTER TONE SIX" #x0185)
    ("LATIN CAPITAL LETTER OPEN O" #x0186)
    ("LATIN CAPITAL LETTER C WITH HOOK" #x0187)
    ("LATIN SMALL LETTER C WITH HOOK" #x0188)
    ("LATIN CAPITAL LETTER AFRICAN D" #x0189)
    ("LATIN CAPITAL LETTER D WITH HOOK" #x018A)
    ("LATIN CAPITAL LETTER D WITH TOPBAR" #x018B)
    ("LATIN SMALL LETTER D WITH TOPBAR" #x018C)
    ("LATIN SMALL LETTER TURNED DELTA" #x018D)
    ("LATIN CAPITAL LETTER REVERSED E" #x018E)
    ("LATIN CAPITAL LETTER SCHWA" #x018F)
    ("LATIN CAPITAL LETTER OPEN E" #x0190)
    ("LATIN CAPITAL LETTER F WITH HOOK" #x0191)
    ("LATIN SMALL LETTER F WITH HOOK" #x0192)
    ("LATIN CAPITAL LETTER G WITH HOOK" #x0193)
    ("LATIN CAPITAL LETTER GAMMA" #x0194)
    ("LATIN SMALL LETTER HV" #x0195)
    ("LATIN CAPITAL LETTER IOTA" #x0196)
    ("LATIN CAPITAL LETTER I WITH STROKE" #x0197)
    ("LATIN CAPITAL LETTER K WITH HOOK" #x0198)
    ("LATIN SMALL LETTER K WITH HOOK" #x0199)
    ("LATIN SMALL LETTER L WITH BAR" #x019A)
    ("LATIN SMALL LETTER LAMBDA WITH STROKE" #x019B)
    ("LATIN CAPITAL LETTER TURNED M" #x019C)
    ("LATIN CAPITAL LETTER N WITH LEFT HOOK" #x019D)
    ("LATIN SMALL LETTER N WITH LONG RIGHT LEG" #x019E)
    ("LATIN CAPITAL LETTER O WITH MIDDLE TILDE" #x019F)
    ("LATIN CAPITAL LETTER O WITH HORN" #x01A0)
    ("LATIN SMALL LETTER O WITH HORN" #x01A1)
    ("LATIN CAPITAL LETTER OI" #x01A2)
    ("LATIN SMALL LETTER OI" #x01A3)
    ("LATIN CAPITAL LETTER P WITH HOOK" #x01A4)
    ("LATIN SMALL LETTER P WITH HOOK" #x01A5)
    ("LATIN LETTER YR" #x01A6)
    ("LATIN CAPITAL LETTER TONE TWO" #x01A7)
    ("LATIN SMALL LETTER TONE TWO" #x01A8)
    ("LATIN CAPITAL LETTER ESH" #x01A9)
    ("LATIN LETTER REVERSED ESH LOOP" #x01AA)
    ("LATIN SMALL LETTER T WITH PALATAL HOOK" #x01AB)
    ("LATIN CAPITAL LETTER T WITH HOOK" #x01AC)
    ("LATIN SMALL LETTER T WITH HOOK" #x01AD)
    ("LATIN CAPITAL LETTER T WITH RETROFLEX HOOK" #x01AE)
    ("LATIN CAPITAL LETTER U WITH HORN" #x01AF)
    ("LATIN SMALL LETTER U WITH HORN" #x01B0)
    ("LATIN CAPITAL LETTER UPSILON" #x01B1)
    ("LATIN CAPITAL LETTER V WITH HOOK" #x01B2)
    ("LATIN CAPITAL LETTER Y WITH HOOK" #x01B3)
    ("LATIN SMALL LETTER Y WITH HOOK" #x01B4)
    ("LATIN CAPITAL LETTER Z WITH STROKE" #x01B5)
    ("LATIN SMALL LETTER Z WITH STROKE" #x01B6)
    ("LATIN CAPITAL LETTER EZH" #x01B7)
    ("LATIN CAPITAL LETTER EZH REVERSED" #x01B8)
    ("LATIN SMALL LETTER EZH REVERSED" #x01B9)
    ("LATIN SMALL LETTER EZH WITH TAIL" #x01BA)
    ("LATIN LETTER TWO WITH STROKE" #x01BB)
    ("LATIN CAPITAL LETTER TONE FIVE" #x01BC)
    ("LATIN SMALL LETTER TONE FIVE" #x01BD)
    ("LATIN LETTER INVERTED GLOTTAL STOP WITH STROKE" #x01BE)
    ("LATIN LETTER WYNN" #x01BF)
    ("LATIN LETTER DENTAL CLICK" #x01C0)
    ("LATIN LETTER LATERAL CLICK" #x01C1)
    ("LATIN LETTER ALVEOLAR CLICK" #x01C2)
    ("LATIN LETTER RETROFLEX CLICK" #x01C3)
    ("LATIN CAPITAL LETTER DZ WITH CARON" #x01C4)
    ("LATIN CAPITAL LETTER D WITH SMALL LETTER Z WITH CARON" #x01C5)
    ("LATIN SMALL LETTER DZ WITH CARON" #x01C6)
    ("LATIN CAPITAL LETTER LJ" #x01C7)
    ("LATIN CAPITAL LETTER L WITH SMALL LETTER J" #x01C8)
    ("LATIN SMALL LETTER LJ" #x01C9)
    ("LATIN CAPITAL LETTER NJ" #x01CA)
    ("LATIN CAPITAL LETTER N WITH SMALL LETTER J" #x01CB)
    ("LATIN SMALL LETTER NJ" #x01CC)
    ("LATIN CAPITAL LETTER A WITH CARON" #x01CD)
    ("LATIN SMALL LETTER A WITH CARON" #x01CE)
    ("LATIN CAPITAL LETTER I WITH CARON" #x01CF)
    ("LATIN SMALL LETTER I WITH CARON" #x01D0)
    ("LATIN CAPITAL LETTER O WITH CARON" #x01D1)
    ("LATIN SMALL LETTER O WITH CARON" #x01D2)
    ("LATIN CAPITAL LETTER U WITH CARON" #x01D3)
    ("LATIN SMALL LETTER U WITH CARON" #x01D4)
    ("LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON" #x01D5)
    ("LATIN SMALL LETTER U WITH DIAERESIS AND MACRON" #x01D6)
    ("LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE" #x01D7)
    ("LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE" #x01D8)
    ("LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON" #x01D9)
    ("LATIN SMALL LETTER U WITH DIAERESIS AND CARON" #x01DA)
    ("LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE" #x01DB)
    ("LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE" #x01DC)
    ("LATIN SMALL LETTER TURNED E" #x01DD)
    ("LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON" #x01DE)
    ("LATIN SMALL LETTER A WITH DIAERESIS AND MACRON" #x01DF)
    ("LATIN CAPITAL LETTER A WITH DOT ABOVE AND MACRON" #x01E0)
    ("LATIN SMALL LETTER A WITH DOT ABOVE AND MACRON" #x01E1)
    ("LATIN CAPITAL LETTER AE WITH MACRON" #x01E2)
    ("LATIN SMALL LETTER AE WITH MACRON" #x01E3)
    ("LATIN CAPITAL LETTER G WITH STROKE" #x01E4)
    ("LATIN SMALL LETTER G WITH STROKE" #x01E5)
    ("LATIN CAPITAL LETTER G WITH CARON" #x01E6)
    ("LATIN SMALL LETTER G WITH CARON" #x01E7)
    ("LATIN CAPITAL LETTER K WITH CARON" #x01E8)
    ("LATIN SMALL LETTER K WITH CARON" #x01E9)
    ("LATIN CAPITAL LETTER O WITH OGONEK" #x01EA)
    ("LATIN SMALL LETTER O WITH OGONEK" #x01EB)
    ("LATIN CAPITAL LETTER O WITH OGONEK AND MACRON" #x01EC)
    ("LATIN SMALL LETTER O WITH OGONEK AND MACRON" #x01ED)
    ("LATIN CAPITAL LETTER EZH WITH CARON" #x01EE)
    ("LATIN SMALL LETTER EZH WITH CARON" #x01EF)
    ("LATIN SMALL LETTER J WITH CARON" #x01F0)
    ("LATIN CAPITAL LETTER DZ" #x01F1)
    ("LATIN CAPITAL LETTER D WITH SMALL LETTER Z" #x01F2)
    ("LATIN SMALL LETTER DZ" #x01F3)
    ("LATIN CAPITAL LETTER G WITH ACUTE" #x01F4)
    ("LATIN SMALL LETTER G WITH ACUTE" #x01F5)
    ("LATIN CAPITAL LETTER HWAIR" #x01F6)
    ("LATIN CAPITAL LETTER WYNN" #x01F7)
    ("LATIN CAPITAL LETTER N WITH GRAVE" #x01F8)
    ("LATIN SMALL LETTER N WITH GRAVE" #x01F9)
    ("LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE" #x01FA)
    ("LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE" #x01FB)
    ("LATIN CAPITAL LETTER AE WITH ACUTE" #x01FC)
    ("LATIN SMALL LETTER AE WITH ACUTE" #x01FD)
    ("LATIN CAPITAL LETTER O WITH STROKE AND ACUTE" #x01FE)
    ("LATIN SMALL LETTER O WITH STROKE AND ACUTE" #x01FF)
    ("LATIN CAPITAL LETTER A WITH DOUBLE GRAVE" #x0200)
    ("LATIN SMALL LETTER A WITH DOUBLE GRAVE" #x0201)
    ("LATIN CAPITAL LETTER A WITH INVERTED BREVE" #x0202)
    ("LATIN SMALL LETTER A WITH INVERTED BREVE" #x0203)
    ("LATIN CAPITAL LETTER E WITH DOUBLE GRAVE" #x0204)
    ("LATIN SMALL LETTER E WITH DOUBLE GRAVE" #x0205)
    ("LATIN CAPITAL LETTER E WITH INVERTED BREVE" #x0206)
    ("LATIN SMALL LETTER E WITH INVERTED BREVE" #x0207)
    ("LATIN CAPITAL LETTER I WITH DOUBLE GRAVE" #x0208)
    ("LATIN SMALL LETTER I WITH DOUBLE GRAVE" #x0209)
    ("LATIN CAPITAL LETTER I WITH INVERTED BREVE" #x020A)
    ("LATIN SMALL LETTER I WITH INVERTED BREVE" #x020B)
    ("LATIN CAPITAL LETTER O WITH DOUBLE GRAVE" #x020C)
    ("LATIN SMALL LETTER O WITH DOUBLE GRAVE" #x020D)
    ("LATIN CAPITAL LETTER O WITH INVERTED BREVE" #x020E)
    ("LATIN SMALL LETTER O WITH INVERTED BREVE" #x020F)
    ("LATIN CAPITAL LETTER R WITH DOUBLE GRAVE" #x0210)
    ("LATIN SMALL LETTER R WITH DOUBLE GRAVE" #x0211)
    ("LATIN CAPITAL LETTER R WITH INVERTED BREVE" #x0212)
    ("LATIN SMALL LETTER R WITH INVERTED BREVE" #x0213)
    ("LATIN CAPITAL LETTER U WITH DOUBLE GRAVE" #x0214)
    ("LATIN SMALL LETTER U WITH DOUBLE GRAVE" #x0215)
    ("LATIN CAPITAL LETTER U WITH INVERTED BREVE" #x0216)
    ("LATIN SMALL LETTER U WITH INVERTED BREVE" #x0217)
    ("LATIN CAPITAL LETTER S WITH COMMA BELOW" #x0218)
    ("LATIN SMALL LETTER S WITH COMMA BELOW" #x0219)
    ("LATIN CAPITAL LETTER T WITH COMMA BELOW" #x021A)
    ("LATIN SMALL LETTER T WITH COMMA BELOW" #x021B)
    ("LATIN CAPITAL LETTER YOGH" #x021C)
    ("LATIN SMALL LETTER YOGH" #x021D)
    ("LATIN CAPITAL LETTER H WITH CARON" #x021E)
    ("LATIN SMALL LETTER H WITH CARON" #x021F)
    ("LATIN CAPITAL LETTER N WITH LONG RIGHT LEG" #x0220)
    ("LATIN CAPITAL LETTER OU" #x0222)
    ("LATIN SMALL LETTER OU" #x0223)
    ("LATIN CAPITAL LETTER Z WITH HOOK" #x0224)
    ("LATIN SMALL LETTER Z WITH HOOK" #x0225)
    ("LATIN CAPITAL LETTER A WITH DOT ABOVE" #x0226)
    ("LATIN SMALL LETTER A WITH DOT ABOVE" #x0227)
    ("LATIN CAPITAL LETTER E WITH CEDILLA" #x0228)
    ("LATIN SMALL LETTER E WITH CEDILLA" #x0229)
    ("LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON" #x022A)
    ("LATIN SMALL LETTER O WITH DIAERESIS AND MACRON" #x022B)
    ("LATIN CAPITAL LETTER O WITH TILDE AND MACRON" #x022C)
    ("LATIN SMALL LETTER O WITH TILDE AND MACRON" #x022D)
    ("LATIN CAPITAL LETTER O WITH DOT ABOVE" #x022E)
    ("LATIN SMALL LETTER O WITH DOT ABOVE" #x022F)
    ("LATIN CAPITAL LETTER O WITH DOT ABOVE AND MACRON" #x0230)
    ("LATIN SMALL LETTER O WITH DOT ABOVE AND MACRON" #x0231)
    ("LATIN CAPITAL LETTER Y WITH MACRON" #x0232)
    ("LATIN SMALL LETTER Y WITH MACRON" #x0233)
    ))

;; arch-tag: d2a4e7e0-0d3a-4a29-994e-f5d0ed62cea2
