(nxml-define-char-name-set 'specials
  '(("INTERLINEAR ANNOTATION ANCHOR" #xFFF9)
    ("INTERLINEAR ANNOTATION SEPARATOR" #xFFFA)
    ("INTERLINEAR ANNOTATION TERMINATOR" #xFFFB)
    ("OBJECT REPLACEMENT CHARACTER" #xFFFC)
    ("REPLACEMENT CHARACTER" #xFFFD)
    ))

;; arch-tag: 0bef6e81-4857-4032-a18a-18784233cc81
