(nxml-define-char-name-set 'cjk-radicals-supplement
  '(("CJK RADICAL REPEAT" #x2E80)
    ("CJK RADICAL CLIFF" #x2E81)
    ("CJK RADICAL SECOND ONE" #x2E82)
    ("CJK RADICAL SECOND TWO" #x2E83)
    ("CJK RADICAL SECOND THREE" #x2E84)
    ("CJK RADICAL PERSON" #x2E85)
    ("CJK RADICAL BOX" #x2E86)
    ("CJK RADICAL TABLE" #x2E87)
    ("CJK RADICAL KNIFE ONE" #x2E88)
    ("CJK RADICAL KNIFE TWO" #x2E89)
    ("CJK RADICAL DIVINATION" #x2E8A)
    ("CJK RADICAL SEAL" #x2E8B)
    ("CJK RADICAL SMALL ONE" #x2E8C)
    ("CJK RADICAL SMALL TWO" #x2E8D)
    ("CJK RADICAL LAME ONE" #x2E8E)
    ("CJK RADICAL LAME TWO" #x2E8F)
    ("CJK RADICAL LAME THREE" #x2E90)
    ("CJK RADICAL LAME FOUR" #x2E91)
    ("CJK RADICAL SNAKE" #x2E92)
    ("CJK RADICAL THREAD" #x2E93)
    ("CJK RADICAL SNOUT ONE" #x2E94)
    ("CJK RADICAL SNOUT TWO" #x2E95)
    ("CJK RADICAL HEART ONE" #x2E96)
    ("CJK RADICAL HEART TWO" #x2E97)
    ("CJK RADICAL HAND" #x2E98)
    ("CJK RADICAL RAP" #x2E99)
    ("CJK RADICAL CHOKE" #x2E9B)
    ("CJK RADICAL SUN" #x2E9C)
    ("CJK RADICAL MOON" #x2E9D)
    ("CJK RADICAL DEATH" #x2E9E)
    ("CJK RADICAL MOTHER" #x2E9F)
    ("CJK RADICAL CIVILIAN" #x2EA0)
    ("CJK RADICAL WATER ONE" #x2EA1)
    ("CJK RADICAL WATER TWO" #x2EA2)
    ("CJK RADICAL FIRE" #x2EA3)
    ("CJK RADICAL PAW ONE" #x2EA4)
    ("CJK RADICAL PAW TWO" #x2EA5)
    ("CJK RADICAL SIMPLIFIED HALF TREE TRUNK" #x2EA6)
    ("CJK RADICAL COW" #x2EA7)
    ("CJK RADICAL DOG" #x2EA8)
    ("CJK RADICAL JADE" #x2EA9)
    ("CJK RADICAL BOLT OF CLOTH" #x2EAA)
    ("CJK RADICAL EYE" #x2EAB)
    ("CJK RADICAL SPIRIT ONE" #x2EAC)
    ("CJK RADICAL SPIRIT TWO" #x2EAD)
    ("CJK RADICAL BAMBOO" #x2EAE)
    ("CJK RADICAL SILK" #x2EAF)
    ("CJK RADICAL C-SIMPLIFIED SILK" #x2EB0)
    ("CJK RADICAL NET ONE" #x2EB1)
    ("CJK RADICAL NET TWO" #x2EB2)
    ("CJK RADICAL NET THREE" #x2EB3)
    ("CJK RADICAL NET FOUR" #x2EB4)
    ("CJK RADICAL MESH" #x2EB5)
    ("CJK RADICAL SHEEP" #x2EB6)
    ("CJK RADICAL RAM" #x2EB7)
    ("CJK RADICAL EWE" #x2EB8)
    ("CJK RADICAL OLD" #x2EB9)
    ("CJK RADICAL BRUSH ONE" #x2EBA)
    ("CJK RADICAL BRUSH TWO" #x2EBB)
    ("CJK RADICAL MEAT" #x2EBC)
    ("CJK RADICAL MORTAR" #x2EBD)
    ("CJK RADICAL GRASS ONE" #x2EBE)
    ("CJK RADICAL GRASS TWO" #x2EBF)
    ("CJK RADICAL GRASS THREE" #x2EC0)
    ("CJK RADICAL TIGER" #x2EC1)
    ("CJK RADICAL CLOTHES" #x2EC2)
    ("CJK RADICAL WEST ONE" #x2EC3)
    ("CJK RADICAL WEST TWO" #x2EC4)
    ("CJK RADICAL C-SIMPLIFIED SEE" #x2EC5)
    ("CJK RADICAL SIMPLIFIED HORN" #x2EC6)
    ("CJK RADICAL HORN" #x2EC7)
    ("CJK RADICAL C-SIMPLIFIED SPEECH" #x2EC8)
    ("CJK RADICAL C-SIMPLIFIED SHELL" #x2EC9)
    ("CJK RADICAL FOOT" #x2ECA)
    ("CJK RADICAL C-SIMPLIFIED CART" #x2ECB)
    ("CJK RADICAL SIMPLIFIED WALK" #x2ECC)
    ("CJK RADICAL WALK ONE" #x2ECD)
    ("CJK RADICAL WALK TWO" #x2ECE)
    ("CJK RADICAL CITY" #x2ECF)
    ("CJK RADICAL C-SIMPLIFIED GOLD" #x2ED0)
    ("CJK RADICAL LONG ONE" #x2ED1)
    ("CJK RADICAL LONG TWO" #x2ED2)
    ("CJK RADICAL C-SIMPLIFIED LONG" #x2ED3)
    ("CJK RADICAL C-SIMPLIFIED GATE" #x2ED4)
    ("CJK RADICAL MOUND ONE" #x2ED5)
    ("CJK RADICAL MOUND TWO" #x2ED6)
    ("CJK RADICAL RAIN" #x2ED7)
    ("CJK RADICAL BLUE" #x2ED8)
    ("CJK RADICAL C-SIMPLIFIED TANNED LEATHER" #x2ED9)
    ("CJK RADICAL C-SIMPLIFIED LEAF" #x2EDA)
    ("CJK RADICAL C-SIMPLIFIED WIND" #x2EDB)
    ("CJK RADICAL C-SIMPLIFIED FLY" #x2EDC)
    ("CJK RADICAL EAT ONE" #x2EDD)
    ("CJK RADICAL EAT TWO" #x2EDE)
    ("CJK RADICAL EAT THREE" #x2EDF)
    ("CJK RADICAL C-SIMPLIFIED EAT" #x2EE0)
    ("CJK RADICAL HEAD" #x2EE1)
    ("CJK RADICAL C-SIMPLIFIED HORSE" #x2EE2)
    ("CJK RADICAL BONE" #x2EE3)
    ("CJK RADICAL GHOST" #x2EE4)
    ("CJK RADICAL C-SIMPLIFIED FISH" #x2EE5)
    ("CJK RADICAL C-SIMPLIFIED BIRD" #x2EE6)
    ("CJK RADICAL C-SIMPLIFIED SALT" #x2EE7)
    ("CJK RADICAL SIMPLIFIED WHEAT" #x2EE8)
    ("CJK RADICAL SIMPLIFIED YELLOW" #x2EE9)
    ("CJK RADICAL C-SIMPLIFIED FROG" #x2EEA)
    ("CJK RADICAL J-SIMPLIFIED EVEN" #x2EEB)
    ("CJK RADICAL C-SIMPLIFIED EVEN" #x2EEC)
    ("CJK RADICAL J-SIMPLIFIED TOOTH" #x2EED)
    ("CJK RADICAL C-SIMPLIFIED TOOTH" #x2EEE)
    ("CJK RADICAL J-SIMPLIFIED DRAGON" #x2EEF)
    ("CJK RADICAL C-SIMPLIFIED DRAGON" #x2EF0)
    ("CJK RADICAL TURTLE" #x2EF1)
    ("CJK RADICAL J-SIMPLIFIED TURTLE" #x2EF2)
    ("CJK RADICAL C-SIMPLIFIED TURTLE" #x2EF3)
    ))

;; arch-tag: 59a2c3dd-63ee-4cba-b427-1e8eb782634f
