(nxml-define-char-name-set 'telugu
  '(("TELUGU SIGN CANDRABINDU" #x0C01)
    ("TELUGU SIGN ANUSVARA" #x0C02)
    ("TELUGU SIGN VISARGA" #x0C03)
    ("TELUGU LETTER A" #x0C05)
    ("TELUGU LETTER AA" #x0C06)
    ("TELUGU LETTER I" #x0C07)
    ("TELUGU LETTER II" #x0C08)
    ("TELUGU LETTER U" #x0C09)
    ("TELUGU LETTER UU" #x0C0A)
    ("TELUGU LETTER VOCALIC R" #x0C0B)
    ("TELUGU LETTER VOCALIC L" #x0C0C)
    ("TELUGU LETTER E" #x0C0E)
    ("TELUGU LETTER EE" #x0C0F)
    ("TELUGU LETTER AI" #x0C10)
    ("TELUGU LETTER O" #x0C12)
    ("TELUGU LETTER OO" #x0C13)
    ("TELUGU LETTER AU" #x0C14)
    ("TELUGU LETTER KA" #x0C15)
    ("TELUGU LETTER KHA" #x0C16)
    ("TELUGU LETTER GA" #x0C17)
    ("TELUGU LETTER GHA" #x0C18)
    ("TELUGU LETTER NGA" #x0C19)
    ("TELUGU LETTER CA" #x0C1A)
    ("TELUGU LETTER CHA" #x0C1B)
    ("TELUGU LETTER JA" #x0C1C)
    ("TELUGU LETTER JHA" #x0C1D)
    ("TELUGU LETTER NYA" #x0C1E)
    ("TELUGU LETTER TTA" #x0C1F)
    ("TELUGU LETTER TTHA" #x0C20)
    ("TELUGU LETTER DDA" #x0C21)
    ("TELUGU LETTER DDHA" #x0C22)
    ("TELUGU LETTER NNA" #x0C23)
    ("TELUGU LETTER TA" #x0C24)
    ("TELUGU LETTER THA" #x0C25)
    ("TELUGU LETTER DA" #x0C26)
    ("TELUGU LETTER DHA" #x0C27)
    ("TELUGU LETTER NA" #x0C28)
    ("TELUGU LETTER PA" #x0C2A)
    ("TELUGU LETTER PHA" #x0C2B)
    ("TELUGU LETTER BA" #x0C2C)
    ("TELUGU LETTER BHA" #x0C2D)
    ("TELUGU LETTER MA" #x0C2E)
    ("TELUGU LETTER YA" #x0C2F)
    ("TELUGU LETTER RA" #x0C30)
    ("TELUGU LETTER RRA" #x0C31)
    ("TELUGU LETTER LA" #x0C32)
    ("TELUGU LETTER LLA" #x0C33)
    ("TELUGU LETTER VA" #x0C35)
    ("TELUGU LETTER SHA" #x0C36)
    ("TELUGU LETTER SSA" #x0C37)
    ("TELUGU LETTER SA" #x0C38)
    ("TELUGU LETTER HA" #x0C39)
    ("TELUGU VOWEL SIGN AA" #x0C3E)
    ("TELUGU VOWEL SIGN I" #x0C3F)
    ("TELUGU VOWEL SIGN II" #x0C40)
    ("TELUGU VOWEL SIGN U" #x0C41)
    ("TELUGU VOWEL SIGN UU" #x0C42)
    ("TELUGU VOWEL SIGN VOCALIC R" #x0C43)
    ("TELUGU VOWEL SIGN VOCALIC RR" #x0C44)
    ("TELUGU VOWEL SIGN E" #x0C46)
    ("TELUGU VOWEL SIGN EE" #x0C47)
    ("TELUGU VOWEL SIGN AI" #x0C48)
    ("TELUGU VOWEL SIGN O" #x0C4A)
    ("TELUGU VOWEL SIGN OO" #x0C4B)
    ("TELUGU VOWEL SIGN AU" #x0C4C)
    ("TELUGU SIGN VIRAMA" #x0C4D)
    ("TELUGU LENGTH MARK" #x0C55)
    ("TELUGU AI LENGTH MARK" #x0C56)
    ("TELUGU LETTER VOCALIC RR" #x0C60)
    ("TELUGU LETTER VOCALIC LL" #x0C61)
    ("TELUGU DIGIT ZERO" #x0C66)
    ("TELUGU DIGIT ONE" #x0C67)
    ("TELUGU DIGIT TWO" #x0C68)
    ("TELUGU DIGIT THREE" #x0C69)
    ("TELUGU DIGIT FOUR" #x0C6A)
    ("TELUGU DIGIT FIVE" #x0C6B)
    ("TELUGU DIGIT SIX" #x0C6C)
    ("TELUGU DIGIT SEVEN" #x0C6D)
    ("TELUGU DIGIT EIGHT" #x0C6E)
    ("TELUGU DIGIT NINE" #x0C6F)
    ))

;; arch-tag: 12e2c9a2-fa3a-400a-9adb-82a89e0a639b
