(nxml-define-char-name-set 'miscellaneous-mathematical-symbols-b
  '(("TRIPLE VERTICAL BAR DELIMITER" #x2980)
    ("Z NOTATION SPOT" #x2981)
    ("Z NOTATION TYPE COLON" #x2982)
    ("LEFT WHITE CURLY BRACKET" #x2983)
    ("RIGHT WHITE CURLY BRACKET" #x2984)
    ("LEFT WHITE PARENTHESIS" #x2985)
    ("RIGHT WHITE PARENTHESIS" #x2986)
    ("Z NOTATION LEFT IMAGE BRACKET" #x2987)
    ("Z NOTATION RIGHT IMAGE BRACKET" #x2988)
    ("Z NOTATION LEFT BINDING BRACKET" #x2989)
    ("Z NOTATION RIGHT BINDING BRACKET" #x298A)
    ("LEFT SQUARE BRACKET WITH UNDERBAR" #x298B)
    ("RIGHT SQUARE BRACKET WITH UNDERBAR" #x298C)
    ("LEFT SQUARE BRACKET WITH TICK IN TOP CORNER" #x298D)
    ("RIGHT SQUARE BRACKET WITH TICK IN BOTTOM CORNER" #x298E)
    ("LEFT SQUARE BRACKET WITH TICK IN BOTTOM CORNER" #x298F)
    ("RIGHT SQUARE BRACKET WITH TICK IN TOP CORNER" #x2990)
    ("LEFT ANGLE BRACKET WITH DOT" #x2991)
    ("RIGHT ANGLE BRACKET WITH DOT" #x2992)
    ("LEFT ARC LESS-THAN BRACKET" #x2993)
    ("RIGHT ARC GREATER-THAN BRACKET" #x2994)
    ("DOUBLE LEFT ARC GREATER-THAN BRACKET" #x2995)
    ("DOUBLE RIGHT ARC LESS-THAN BRACKET" #x2996)
    ("LEFT BLACK TORTOISE SHELL BRACKET" #x2997)
    ("RIGHT BLACK TORTOISE SHELL BRACKET" #x2998)
    ("DOTTED FENCE" #x2999)
    ("VERTICAL ZIGZAG LINE" #x299A)
    ("MEASURED ANGLE OPENING LEFT" #x299B)
    ("RIGHT ANGLE VARIANT WITH SQUARE" #x299C)
    ("MEASURED RIGHT ANGLE WITH DOT" #x299D)
    ("ANGLE WITH S INSIDE" #x299E)
    ("ACUTE ANGLE" #x299F)
    ("SPHERICAL ANGLE OPENING LEFT" #x29A0)
    ("SPHERICAL ANGLE OPENING UP" #x29A1)
    ("TURNED ANGLE" #x29A2)
    ("REVERSED ANGLE" #x29A3)
    ("ANGLE WITH UNDERBAR" #x29A4)
    ("REVERSED ANGLE WITH UNDERBAR" #x29A5)
    ("OBLIQUE ANGLE OPENING UP" #x29A6)
    ("OBLIQUE ANGLE OPENING DOWN" #x29A7)
    ("MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND RIGHT" #x29A8)
    ("MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND LEFT" #x29A9)
    ("MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND RIGHT" #x29AA)
    ("MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND LEFT" #x29AB)
    ("MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND UP" #x29AC)
    ("MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND UP" #x29AD)
    ("MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND DOWN" #x29AE)
    ("MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND DOWN" #x29AF)
    ("REVERSED EMPTY SET" #x29B0)
    ("EMPTY SET WITH OVERBAR" #x29B1)
    ("EMPTY SET WITH SMALL CIRCLE ABOVE" #x29B2)
    ("EMPTY SET WITH RIGHT ARROW ABOVE" #x29B3)
    ("EMPTY SET WITH LEFT ARROW ABOVE" #x29B4)
    ("CIRCLE WITH HORIZONTAL BAR" #x29B5)
    ("CIRCLED VERTICAL BAR" #x29B6)
    ("CIRCLED PARALLEL" #x29B7)
    ("CIRCLED REVERSE SOLIDUS" #x29B8)
    ("CIRCLED PERPENDICULAR" #x29B9)
    ("CIRCLE DIVIDED BY HORIZONTAL BAR AND TOP HALF DIVIDED BY VERTICAL BAR" #x29BA)
    ("CIRCLE WITH SUPERIMPOSED X" #x29BB)
    ("CIRCLED ANTICLOCKWISE-ROTATED DIVISION SIGN" #x29BC)
    ("UP ARROW THROUGH CIRCLE" #x29BD)
    ("CIRCLED WHITE BULLET" #x29BE)
    ("CIRCLED BULLET" #x29BF)
    ("CIRCLED LESS-THAN" #x29C0)
    ("CIRCLED GREATER-THAN" #x29C1)
    ("CIRCLE WITH SMALL CIRCLE TO THE RIGHT" #x29C2)
    ("CIRCLE WITH TWO HORIZONTAL STROKES TO THE RIGHT" #x29C3)
    ("SQUARED RISING DIAGONAL SLASH" #x29C4)
    ("SQUARED FALLING DIAGONAL SLASH" #x29C5)
    ("SQUARED ASTERISK" #x29C6)
    ("SQUARED SMALL CIRCLE" #x29C7)
    ("SQUARED SQUARE" #x29C8)
    ("TWO JOINED SQUARES" #x29C9)
    ("TRIANGLE WITH DOT ABOVE" #x29CA)
    ("TRIANGLE WITH UNDERBAR" #x29CB)
    ("S IN TRIANGLE" #x29CC)
    ("TRIANGLE WITH SERIFS AT BOTTOM" #x29CD)
    ("RIGHT TRIANGLE ABOVE LEFT TRIANGLE" #x29CE)
    ("LEFT TRIANGLE BESIDE VERTICAL BAR" #x29CF)
    ("VERTICAL BAR BESIDE RIGHT TRIANGLE" #x29D0)
    ("BOWTIE WITH LEFT HALF BLACK" #x29D1)
    ("BOWTIE WITH RIGHT HALF BLACK" #x29D2)
    ("BLACK BOWTIE" #x29D3)
    ("TIMES WITH LEFT HALF BLACK" #x29D4)
    ("TIMES WITH RIGHT HALF BLACK" #x29D5)
    ("WHITE HOURGLASS" #x29D6)
    ("BLACK HOURGLASS" #x29D7)
    ("LEFT WIGGLY FENCE" #x29D8)
    ("RIGHT WIGGLY FENCE" #x29D9)
    ("LEFT DOUBLE WIGGLY FENCE" #x29DA)
    ("RIGHT DOUBLE WIGGLY FENCE" #x29DB)
    ("INCOMPLETE INFINITY" #x29DC)
    ("TIE OVER INFINITY" #x29DD)
    ("INFINITY NEGATED WITH VERTICAL BAR" #x29DE)
    ("DOUBLE-ENDED MULTIMAP" #x29DF)
    ("SQUARE WITH CONTOURED OUTLINE" #x29E0)
    ("INCREASES AS" #x29E1)
    ("SHUFFLE PRODUCT" #x29E2)
    ("EQUALS SIGN AND SLANTED PARALLEL" #x29E3)
    ("EQUALS SIGN AND SLANTED PARALLEL WITH TILDE ABOVE" #x29E4)
    ("IDENTICAL TO AND SLANTED PARALLEL" #x29E5)
    ("GLEICH STARK" #x29E6)
    ("THERMODYNAMIC" #x29E7)
    ("DOWN-POINTING TRIANGLE WITH LEFT HALF BLACK" #x29E8)
    ("DOWN-POINTING TRIANGLE WITH RIGHT HALF BLACK" #x29E9)
    ("BLACK DIAMOND WITH DOWN ARROW" #x29EA)
    ("BLACK LOZENGE" #x29EB)
    ("WHITE CIRCLE WITH DOWN ARROW" #x29EC)
    ("BLACK CIRCLE WITH DOWN ARROW" #x29ED)
    ("ERROR-BARRED WHITE SQUARE" #x29EE)
    ("ERROR-BARRED BLACK SQUARE" #x29EF)
    ("ERROR-BARRED WHITE DIAMOND" #x29F0)
    ("ERROR-BARRED BLACK DIAMOND" #x29F1)
    ("ERROR-BARRED WHITE CIRCLE" #x29F2)
    ("ERROR-BARRED BLACK CIRCLE" #x29F3)
    ("RULE-DELAYED" #x29F4)
    ("REVERSE SOLIDUS OPERATOR" #x29F5)
    ("SOLIDUS WITH OVERBAR" #x29F6)
    ("REVERSE SOLIDUS WITH HORIZONTAL STROKE" #x29F7)
    ("BIG SOLIDUS" #x29F8)
    ("BIG REVERSE SOLIDUS" #x29F9)
    ("DOUBLE PLUS" #x29FA)
    ("TRIPLE PLUS" #x29FB)
    ("LEFT-POINTING CURVED ANGLE BRACKET" #x29FC)
    ("RIGHT-POINTING CURVED ANGLE BRACKET" #x29FD)
    ("TINY" #x29FE)
    ("MINY" #x29FF)
    ))

;; arch-tag: 4ae3dcff-8f56-4f7e-b9f3-07108e543969
