(nxml-define-char-name-set 'supplemental-arrows-a
  '(("UPWARDS QUADRUPLE ARROW" #x27F0)
    ("DOWNWARDS QUADRUPLE ARROW" #x27F1)
    ("ANTICLOCKWISE GAPPED CIRCLE ARROW" #x27F2)
    ("CLOCKWISE GAPPED CIRCLE ARROW" #x27F3)
    ("RIGHT ARROW WITH CIRCLED PLUS" #x27F4)
    ("LONG LEFTWARDS ARROW" #x27F5)
    ("LONG RIGHTWARDS ARROW" #x27F6)
    ("LONG LEFT RIGHT ARROW" #x27F7)
    ("LONG LEFTWARDS DOUBLE ARROW" #x27F8)
    ("LONG RIGHTWARDS DOUBLE ARROW" #x27F9)
    ("LONG LEFT RIGHT DOUBLE ARROW" #x27FA)
    ("LONG LEFTWARDS ARROW FROM BAR" #x27FB)
    ("LONG RIGHTWARDS ARROW FROM BAR" #x27FC)
    ("LONG LEFTWARDS DOUBLE ARROW FROM BAR" #x27FD)
    ("LONG RIGHTWARDS DOUBLE ARROW FROM BAR" #x27FE)
    ("LONG RIGHTWARDS SQUIGGLE ARROW" #x27FF)
    ))

;; arch-tag: 64623112-13d2-4bd6-8768-5508e3a7f8a9
