(nxml-define-char-name-set 'byzantine-musical-symbols
  '(("BYZANTINE MUSICAL SYMBOL PSILI" #x1D000)
    ("BYZANTINE MUSICAL SYMBOL DASEIA" #x1D001)
    ("BYZANTINE MUSICAL SYMBOL PERISPOMENI" #x1D002)
    ("BYZANTINE MUSICAL SYMBOL OXEIA EKFONITIKON" #x1D003)
    ("BYZANTINE MUSICAL SYMBOL OXEIA DIPLI" #x1D004)
    ("BYZANTINE MUSICAL SYMBOL VAREIA EKFONITIKON" #x1D005)
    ("BYZANTINE MUSICAL SYMBOL VAREIA DIPLI" #x1D006)
    ("BYZANTINE MUSICAL SYMBOL KATHISTI" #x1D007)
    ("BYZANTINE MUSICAL SYMBOL SYRMATIKI" #x1D008)
    ("BYZANTINE MUSICAL SYMBOL PARAKLITIKI" #x1D009)
    ("BYZANTINE MUSICAL SYMBOL YPOKRISIS" #x1D00A)
    ("BYZANTINE MUSICAL SYMBOL YPOKRISIS DIPLI" #x1D00B)
    ("BYZANTINE MUSICAL SYMBOL KREMASTI" #x1D00C)
    ("BYZANTINE MUSICAL SYMBOL APESO EKFONITIKON" #x1D00D)
    ("BYZANTINE MUSICAL SYMBOL EXO EKFONITIKON" #x1D00E)
    ("BYZANTINE MUSICAL SYMBOL TELEIA" #x1D00F)
    ("BYZANTINE MUSICAL SYMBOL KENTIMATA" #x1D010)
    ("BYZANTINE MUSICAL SYMBOL APOSTROFOS" #x1D011)
    ("BYZANTINE MUSICAL SYMBOL APOSTROFOS DIPLI" #x1D012)
    ("BYZANTINE MUSICAL SYMBOL SYNEVMA" #x1D013)
    ("BYZANTINE MUSICAL SYMBOL THITA" #x1D014)
    ("BYZANTINE MUSICAL SYMBOL OLIGON ARCHAION" #x1D015)
    ("BYZANTINE MUSICAL SYMBOL GORGON ARCHAION" #x1D016)
    ("BYZANTINE MUSICAL SYMBOL PSILON" #x1D017)
    ("BYZANTINE MUSICAL SYMBOL CHAMILON" #x1D018)
    ("BYZANTINE MUSICAL SYMBOL VATHY" #x1D019)
    ("BYZANTINE MUSICAL SYMBOL ISON ARCHAION" #x1D01A)
    ("BYZANTINE MUSICAL SYMBOL KENTIMA ARCHAION" #x1D01B)
    ("BYZANTINE MUSICAL SYMBOL KENTIMATA ARCHAION" #x1D01C)
    ("BYZANTINE MUSICAL SYMBOL SAXIMATA" #x1D01D)
    ("BYZANTINE MUSICAL SYMBOL PARICHON" #x1D01E)
    ("BYZANTINE MUSICAL SYMBOL STAVROS APODEXIA" #x1D01F)
    ("BYZANTINE MUSICAL SYMBOL OXEIAI ARCHAION" #x1D020)
    ("BYZANTINE MUSICAL SYMBOL VAREIAI ARCHAION" #x1D021)
    ("BYZANTINE MUSICAL SYMBOL APODERMA ARCHAION" #x1D022)
    ("BYZANTINE MUSICAL SYMBOL APOTHEMA" #x1D023)
    ("BYZANTINE MUSICAL SYMBOL KLASMA" #x1D024)
    ("BYZANTINE MUSICAL SYMBOL REVMA" #x1D025)
    ("BYZANTINE MUSICAL SYMBOL PIASMA ARCHAION" #x1D026)
    ("BYZANTINE MUSICAL SYMBOL TINAGMA" #x1D027)
    ("BYZANTINE MUSICAL SYMBOL ANATRICHISMA" #x1D028)
    ("BYZANTINE MUSICAL SYMBOL SEISMA" #x1D029)
    ("BYZANTINE MUSICAL SYMBOL SYNAGMA ARCHAION" #x1D02A)
    ("BYZANTINE MUSICAL SYMBOL SYNAGMA META STAVROU" #x1D02B)
    ("BYZANTINE MUSICAL SYMBOL OYRANISMA ARCHAION" #x1D02C)
    ("BYZANTINE MUSICAL SYMBOL THEMA" #x1D02D)
    ("BYZANTINE MUSICAL SYMBOL LEMOI" #x1D02E)
    ("BYZANTINE MUSICAL SYMBOL DYO" #x1D02F)
    ("BYZANTINE MUSICAL SYMBOL TRIA" #x1D030)
    ("BYZANTINE MUSICAL SYMBOL TESSERA" #x1D031)
    ("BYZANTINE MUSICAL SYMBOL KRATIMATA" #x1D032)
    ("BYZANTINE MUSICAL SYMBOL APESO EXO NEO" #x1D033)
    ("BYZANTINE MUSICAL SYMBOL FTHORA ARCHAION" #x1D034)
    ("BYZANTINE MUSICAL SYMBOL IMIFTHORA" #x1D035)
    ("BYZANTINE MUSICAL SYMBOL TROMIKON ARCHAION" #x1D036)
    ("BYZANTINE MUSICAL SYMBOL KATAVA TROMIKON" #x1D037)
    ("BYZANTINE MUSICAL SYMBOL PELASTON" #x1D038)
    ("BYZANTINE MUSICAL SYMBOL PSIFISTON" #x1D039)
    ("BYZANTINE MUSICAL SYMBOL KONTEVMA" #x1D03A)
    ("BYZANTINE MUSICAL SYMBOL CHOREVMA ARCHAION" #x1D03B)
    ("BYZANTINE MUSICAL SYMBOL RAPISMA" #x1D03C)
    ("BYZANTINE MUSICAL SYMBOL PARAKALESMA ARCHAION" #x1D03D)
    ("BYZANTINE MUSICAL SYMBOL PARAKLITIKI ARCHAION" #x1D03E)
    ("BYZANTINE MUSICAL SYMBOL ICHADIN" #x1D03F)
    ("BYZANTINE MUSICAL SYMBOL NANA" #x1D040)
    ("BYZANTINE MUSICAL SYMBOL PETASMA" #x1D041)
    ("BYZANTINE MUSICAL SYMBOL KONTEVMA ALLO" #x1D042)
    ("BYZANTINE MUSICAL SYMBOL TROMIKON ALLO" #x1D043)
    ("BYZANTINE MUSICAL SYMBOL STRAGGISMATA" #x1D044)
    ("BYZANTINE MUSICAL SYMBOL GRONTHISMATA" #x1D045)
    ("BYZANTINE MUSICAL SYMBOL ISON NEO" #x1D046)
    ("BYZANTINE MUSICAL SYMBOL OLIGON NEO" #x1D047)
    ("BYZANTINE MUSICAL SYMBOL OXEIA NEO" #x1D048)
    ("BYZANTINE MUSICAL SYMBOL PETASTI" #x1D049)
    ("BYZANTINE MUSICAL SYMBOL KOUFISMA" #x1D04A)
    ("BYZANTINE MUSICAL SYMBOL PETASTOKOUFISMA" #x1D04B)
    ("BYZANTINE MUSICAL SYMBOL KRATIMOKOUFISMA" #x1D04C)
    ("BYZANTINE MUSICAL SYMBOL PELASTON NEO" #x1D04D)
    ("BYZANTINE MUSICAL SYMBOL KENTIMATA NEO ANO" #x1D04E)
    ("BYZANTINE MUSICAL SYMBOL KENTIMA NEO ANO" #x1D04F)
    ("BYZANTINE MUSICAL SYMBOL YPSILI" #x1D050)
    ("BYZANTINE MUSICAL SYMBOL APOSTROFOS NEO" #x1D051)
    ("BYZANTINE MUSICAL SYMBOL APOSTROFOI SYNDESMOS NEO" #x1D052)
    ("BYZANTINE MUSICAL SYMBOL YPORROI" #x1D053)
    ("BYZANTINE MUSICAL SYMBOL KRATIMOYPORROON" #x1D054)
    ("BYZANTINE MUSICAL SYMBOL ELAFRON" #x1D055)
    ("BYZANTINE MUSICAL SYMBOL CHAMILI" #x1D056)
    ("BYZANTINE MUSICAL SYMBOL MIKRON ISON" #x1D057)
    ("BYZANTINE MUSICAL SYMBOL VAREIA NEO" #x1D058)
    ("BYZANTINE MUSICAL SYMBOL PIASMA NEO" #x1D059)
    ("BYZANTINE MUSICAL SYMBOL PSIFISTON NEO" #x1D05A)
    ("BYZANTINE MUSICAL SYMBOL OMALON" #x1D05B)
    ("BYZANTINE MUSICAL SYMBOL ANTIKENOMA" #x1D05C)
    ("BYZANTINE MUSICAL SYMBOL LYGISMA" #x1D05D)
    ("BYZANTINE MUSICAL SYMBOL PARAKLITIKI NEO" #x1D05E)
    ("BYZANTINE MUSICAL SYMBOL PARAKALESMA NEO" #x1D05F)
    ("BYZANTINE MUSICAL SYMBOL ETERON PARAKALESMA" #x1D060)
    ("BYZANTINE MUSICAL SYMBOL KYLISMA" #x1D061)
    ("BYZANTINE MUSICAL SYMBOL ANTIKENOKYLISMA" #x1D062)
    ("BYZANTINE MUSICAL SYMBOL TROMIKON NEO" #x1D063)
    ("BYZANTINE MUSICAL SYMBOL EKSTREPTON" #x1D064)
    ("BYZANTINE MUSICAL SYMBOL SYNAGMA NEO" #x1D065)
    ("BYZANTINE MUSICAL SYMBOL SYRMA" #x1D066)
    ("BYZANTINE MUSICAL SYMBOL CHOREVMA NEO" #x1D067)
    ("BYZANTINE MUSICAL SYMBOL EPEGERMA" #x1D068)
    ("BYZANTINE MUSICAL SYMBOL SEISMA NEO" #x1D069)
    ("BYZANTINE MUSICAL SYMBOL XIRON KLASMA" #x1D06A)
    ("BYZANTINE MUSICAL SYMBOL TROMIKOPSIFISTON" #x1D06B)
    ("BYZANTINE MUSICAL SYMBOL PSIFISTOLYGISMA" #x1D06C)
    ("BYZANTINE MUSICAL SYMBOL TROMIKOLYGISMA" #x1D06D)
    ("BYZANTINE MUSICAL SYMBOL TROMIKOPARAKALESMA" #x1D06E)
    ("BYZANTINE MUSICAL SYMBOL PSIFISTOPARAKALESMA" #x1D06F)
    ("BYZANTINE MUSICAL SYMBOL TROMIKOSYNAGMA" #x1D070)
    ("BYZANTINE MUSICAL SYMBOL PSIFISTOSYNAGMA" #x1D071)
    ("BYZANTINE MUSICAL SYMBOL GORGOSYNTHETON" #x1D072)
    ("BYZANTINE MUSICAL SYMBOL ARGOSYNTHETON" #x1D073)
    ("BYZANTINE MUSICAL SYMBOL ETERON ARGOSYNTHETON" #x1D074)
    ("BYZANTINE MUSICAL SYMBOL OYRANISMA NEO" #x1D075)
    ("BYZANTINE MUSICAL SYMBOL THEMATISMOS ESO" #x1D076)
    ("BYZANTINE MUSICAL SYMBOL THEMATISMOS EXO" #x1D077)
    ("BYZANTINE MUSICAL SYMBOL THEMA APLOUN" #x1D078)
    ("BYZANTINE MUSICAL SYMBOL THES KAI APOTHES" #x1D079)
    ("BYZANTINE MUSICAL SYMBOL KATAVASMA" #x1D07A)
    ("BYZANTINE MUSICAL SYMBOL ENDOFONON" #x1D07B)
    ("BYZANTINE MUSICAL SYMBOL YFEN KATO" #x1D07C)
    ("BYZANTINE MUSICAL SYMBOL YFEN ANO" #x1D07D)
    ("BYZANTINE MUSICAL SYMBOL STAVROS" #x1D07E)
    ("BYZANTINE MUSICAL SYMBOL KLASMA ANO" #x1D07F)
    ("BYZANTINE MUSICAL SYMBOL DIPLI ARCHAION" #x1D080)
    ("BYZANTINE MUSICAL SYMBOL KRATIMA ARCHAION" #x1D081)
    ("BYZANTINE MUSICAL SYMBOL KRATIMA ALLO" #x1D082)
    ("BYZANTINE MUSICAL SYMBOL KRATIMA NEO" #x1D083)
    ("BYZANTINE MUSICAL SYMBOL APODERMA NEO" #x1D084)
    ("BYZANTINE MUSICAL SYMBOL APLI" #x1D085)
    ("BYZANTINE MUSICAL SYMBOL DIPLI" #x1D086)
    ("BYZANTINE MUSICAL SYMBOL TRIPLI" #x1D087)
    ("BYZANTINE MUSICAL SYMBOL TETRAPLI" #x1D088)
    ("BYZANTINE MUSICAL SYMBOL KORONIS" #x1D089)
    ("BYZANTINE MUSICAL SYMBOL LEIMMA ENOS CHRONOU" #x1D08A)
    ("BYZANTINE MUSICAL SYMBOL LEIMMA DYO CHRONON" #x1D08B)
    ("BYZANTINE MUSICAL SYMBOL LEIMMA TRION CHRONON" #x1D08C)
    ("BYZANTINE MUSICAL SYMBOL LEIMMA TESSARON CHRONON" #x1D08D)
    ("BYZANTINE MUSICAL SYMBOL LEIMMA IMISEOS CHRONOU" #x1D08E)
    ("BYZANTINE MUSICAL SYMBOL GORGON NEO ANO" #x1D08F)
    ("BYZANTINE MUSICAL SYMBOL GORGON PARESTIGMENON ARISTERA" #x1D090)
    ("BYZANTINE MUSICAL SYMBOL GORGON PARESTIGMENON DEXIA" #x1D091)
    ("BYZANTINE MUSICAL SYMBOL DIGORGON" #x1D092)
    ("BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON ARISTERA KATO" #x1D093)
    ("BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON ARISTERA ANO" #x1D094)
    ("BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON DEXIA" #x1D095)
    ("BYZANTINE MUSICAL SYMBOL TRIGORGON" #x1D096)
    ("BYZANTINE MUSICAL SYMBOL ARGON" #x1D097)
    ("BYZANTINE MUSICAL SYMBOL IMIDIARGON" #x1D098)
    ("BYZANTINE MUSICAL SYMBOL DIARGON" #x1D099)
    ("BYZANTINE MUSICAL SYMBOL AGOGI POLI ARGI" #x1D09A)
    ("BYZANTINE MUSICAL SYMBOL AGOGI ARGOTERI" #x1D09B)
    ("BYZANTINE MUSICAL SYMBOL AGOGI ARGI" #x1D09C)
    ("BYZANTINE MUSICAL SYMBOL AGOGI METRIA" #x1D09D)
    ("BYZANTINE MUSICAL SYMBOL AGOGI MESI" #x1D09E)
    ("BYZANTINE MUSICAL SYMBOL AGOGI GORGI" #x1D09F)
    ("BYZANTINE MUSICAL SYMBOL AGOGI GORGOTERI" #x1D0A0)
    ("BYZANTINE MUSICAL SYMBOL AGOGI POLI GORGI" #x1D0A1)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA PROTOS ICHOS" #x1D0A2)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA ALLI PROTOS ICHOS" #x1D0A3)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA DEYTEROS ICHOS" #x1D0A4)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA ALLI DEYTEROS ICHOS" #x1D0A5)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA TRITOS ICHOS" #x1D0A6)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA TRIFONIAS" #x1D0A7)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA TETARTOS ICHOS" #x1D0A8)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA TETARTOS LEGETOS ICHOS" #x1D0A9)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA LEGETOS ICHOS" #x1D0AA)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA PLAGIOS ICHOS" #x1D0AB)
    ("BYZANTINE MUSICAL SYMBOL ISAKIA TELOUS ICHIMATOS" #x1D0AC)
    ("BYZANTINE MUSICAL SYMBOL APOSTROFOI TELOUS ICHIMATOS" #x1D0AD)
    ("BYZANTINE MUSICAL SYMBOL FANEROSIS TETRAFONIAS" #x1D0AE)
    ("BYZANTINE MUSICAL SYMBOL FANEROSIS MONOFONIAS" #x1D0AF)
    ("BYZANTINE MUSICAL SYMBOL FANEROSIS DIFONIAS" #x1D0B0)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA VARYS ICHOS" #x1D0B1)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA PROTOVARYS ICHOS" #x1D0B2)
    ("BYZANTINE MUSICAL SYMBOL MARTYRIA PLAGIOS TETARTOS ICHOS" #x1D0B3)
    ("BYZANTINE MUSICAL SYMBOL GORTHMIKON N APLOUN" #x1D0B4)
    ("BYZANTINE MUSICAL SYMBOL GORTHMIKON N DIPLOUN" #x1D0B5)
    ("BYZANTINE MUSICAL SYMBOL ENARXIS KAI FTHORA VOU" #x1D0B6)
    ("BYZANTINE MUSICAL SYMBOL IMIFONON" #x1D0B7)
    ("BYZANTINE MUSICAL SYMBOL IMIFTHORON" #x1D0B8)
    ("BYZANTINE MUSICAL SYMBOL FTHORA ARCHAION DEYTEROU ICHOU" #x1D0B9)
    ("BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI PA" #x1D0BA)
    ("BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NANA" #x1D0BB)
    ("BYZANTINE MUSICAL SYMBOL FTHORA NAOS ICHOS" #x1D0BC)
    ("BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI DI" #x1D0BD)
    ("BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON DIATONON DI" #x1D0BE)
    ("BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI KE" #x1D0BF)
    ("BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI ZO" #x1D0C0)
    ("BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NI KATO" #x1D0C1)
    ("BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NI ANO" #x1D0C2)
    ("BYZANTINE MUSICAL SYMBOL FTHORA MALAKON CHROMA DIFONIAS" #x1D0C3)
    ("BYZANTINE MUSICAL SYMBOL FTHORA MALAKON CHROMA MONOFONIAS" #x1D0C4)
    ("BYZANTINE MUSICAL SYMBOL FHTORA SKLIRON CHROMA VASIS" #x1D0C5)
    ("BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON CHROMA SYNAFI" #x1D0C6)
    ("BYZANTINE MUSICAL SYMBOL FTHORA NENANO" #x1D0C7)
    ("BYZANTINE MUSICAL SYMBOL CHROA ZYGOS" #x1D0C8)
    ("BYZANTINE MUSICAL SYMBOL CHROA KLITON" #x1D0C9)
    ("BYZANTINE MUSICAL SYMBOL CHROA SPATHI" #x1D0CA)
    ("BYZANTINE MUSICAL SYMBOL FTHORA I YFESIS TETARTIMORION" #x1D0CB)
    ("BYZANTINE MUSICAL SYMBOL FTHORA ENARMONIOS ANTIFONIA" #x1D0CC)
    ("BYZANTINE MUSICAL SYMBOL YFESIS TRITIMORION" #x1D0CD)
    ("BYZANTINE MUSICAL SYMBOL DIESIS TRITIMORION" #x1D0CE)
    ("BYZANTINE MUSICAL SYMBOL DIESIS TETARTIMORION" #x1D0CF)
    ("BYZANTINE MUSICAL SYMBOL DIESIS APLI DYO DODEKATA" #x1D0D0)
    ("BYZANTINE MUSICAL SYMBOL DIESIS MONOGRAMMOS TESSERA DODEKATA" #x1D0D1)
    ("BYZANTINE MUSICAL SYMBOL DIESIS DIGRAMMOS EX DODEKATA" #x1D0D2)
    ("BYZANTINE MUSICAL SYMBOL DIESIS TRIGRAMMOS OKTO DODEKATA" #x1D0D3)
    ("BYZANTINE MUSICAL SYMBOL YFESIS APLI DYO DODEKATA" #x1D0D4)
    ("BYZANTINE MUSICAL SYMBOL YFESIS MONOGRAMMOS TESSERA DODEKATA" #x1D0D5)
    ("BYZANTINE MUSICAL SYMBOL YFESIS DIGRAMMOS EX DODEKATA" #x1D0D6)
    ("BYZANTINE MUSICAL SYMBOL YFESIS TRIGRAMMOS OKTO DODEKATA" #x1D0D7)
    ("BYZANTINE MUSICAL SYMBOL GENIKI DIESIS" #x1D0D8)
    ("BYZANTINE MUSICAL SYMBOL GENIKI YFESIS" #x1D0D9)
    ("BYZANTINE MUSICAL SYMBOL DIASTOLI APLI MIKRI" #x1D0DA)
    ("BYZANTINE MUSICAL SYMBOL DIASTOLI APLI MEGALI" #x1D0DB)
    ("BYZANTINE MUSICAL SYMBOL DIASTOLI DIPLI" #x1D0DC)
    ("BYZANTINE MUSICAL SYMBOL DIASTOLI THESEOS" #x1D0DD)
    ("BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS" #x1D0DE)
    ("BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS DISIMOU" #x1D0DF)
    ("BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS TRISIMOU" #x1D0E0)
    ("BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS TETRASIMOU" #x1D0E1)
    ("BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS" #x1D0E2)
    ("BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS DISIMOU" #x1D0E3)
    ("BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS TRISIMOU" #x1D0E4)
    ("BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS TETRASIMOU" #x1D0E5)
    ("BYZANTINE MUSICAL SYMBOL DIGRAMMA GG" #x1D0E6)
    ("BYZANTINE MUSICAL SYMBOL DIFTOGGOS OU" #x1D0E7)
    ("BYZANTINE MUSICAL SYMBOL STIGMA" #x1D0E8)
    ("BYZANTINE MUSICAL SYMBOL ARKTIKO PA" #x1D0E9)
    ("BYZANTINE MUSICAL SYMBOL ARKTIKO VOU" #x1D0EA)
    ("BYZANTINE MUSICAL SYMBOL ARKTIKO GA" #x1D0EB)
    ("BYZANTINE MUSICAL SYMBOL ARKTIKO DI" #x1D0EC)
    ("BYZANTINE MUSICAL SYMBOL ARKTIKO KE" #x1D0ED)
    ("BYZANTINE MUSICAL SYMBOL ARKTIKO ZO" #x1D0EE)
    ("BYZANTINE MUSICAL SYMBOL ARKTIKO NI" #x1D0EF)
    ("BYZANTINE MUSICAL SYMBOL KENTIMATA NEO MESO" #x1D0F0)
    ("BYZANTINE MUSICAL SYMBOL KENTIMA NEO MESO" #x1D0F1)
    ("BYZANTINE MUSICAL SYMBOL KENTIMATA NEO KATO" #x1D0F2)
    ("BYZANTINE MUSICAL SYMBOL KENTIMA NEO KATO" #x1D0F3)
    ("BYZANTINE MUSICAL SYMBOL KLASMA KATO" #x1D0F4)
    ("BYZANTINE MUSICAL SYMBOL GORGON NEO KATO" #x1D0F5)
    ))

;; arch-tag: 2b360fc3-2760-43c9-9d3e-2ef3fd8452b5
