(nxml-define-char-name-set 'bopomofo
  '(("BOPOMOFO LETTER B" #x3105)
    ("BOPOMOFO LETTER P" #x3106)
    ("BOPOMOFO LETTER M" #x3107)
    ("BOPOMOFO LETTER F" #x3108)
    ("BOPOMOFO LETTER D" #x3109)
    ("BOPOMOFO LETTER T" #x310A)
    ("BOPOMOFO LETTER N" #x310B)
    ("BOPOMOFO LETTER L" #x310C)
    ("BOPOMOFO LETTER G" #x310D)
    ("BOPOMOFO LETTER K" #x310E)
    ("BOPOMOFO LETTER H" #x310F)
    ("BOPOMOFO LETTER J" #x3110)
    ("BOPOMOFO LETTER Q" #x3111)
    ("BOPOMOFO LETTER X" #x3112)
    ("BOPOMOFO LETTER ZH" #x3113)
    ("BOPOMOFO LETTER CH" #x3114)
    ("BOPOMOFO LETTER SH" #x3115)
    ("BOPOMOFO LETTER R" #x3116)
    ("BOPOMOFO LETTER Z" #x3117)
    ("BOPOMOFO LETTER C" #x3118)
    ("BOPOMOFO LETTER S" #x3119)
    ("BOPOMOFO LETTER A" #x311A)
    ("BOPOMOFO LETTER O" #x311B)
    ("BOPOMOFO LETTER E" #x311C)
    ("BOPOMOFO LETTER EH" #x311D)
    ("BOPOMOFO LETTER AI" #x311E)
    ("BOPOMOFO LETTER EI" #x311F)
    ("BOPOMOFO LETTER AU" #x3120)
    ("BOPOMOFO LETTER OU" #x3121)
    ("BOPOMOFO LETTER AN" #x3122)
    ("BOPOMOFO LETTER EN" #x3123)
    ("BOPOMOFO LETTER ANG" #x3124)
    ("BOPOMOFO LETTER ENG" #x3125)
    ("BOPOMOFO LETTER ER" #x3126)
    ("BOPOMOFO LETTER I" #x3127)
    ("BOPOMOFO LETTER U" #x3128)
    ("BOPOMOFO LETTER IU" #x3129)
    ("BOPOMOFO LETTER V" #x312A)
    ("BOPOMOFO LETTER NG" #x312B)
    ("BOPOMOFO LETTER GN" #x312C)
    ))

;; arch-tag: 0aeb6bb4-ac93-4014-a836-eb793636a168
