(nxml-define-char-name-set 'optical-character-recognition
  '(("OCR HOOK" #x2440)
    ("OCR CHAIR" #x2441)
    ("OCR FORK" #x2442)
    ("OCR INVERTED FORK" #x2443)
    ("OCR BELT BUCKLE" #x2444)
    ("OCR BOW TIE" #x2445)
    ("OCR BRANCH BANK IDENTIFICATION" #x2446)
    ("OCR AMOUNT OF CHECK" #x2447)
    ("OCR DASH" #x2448)
    ("OCR CUSTOMER ACCOUNT NUMBER" #x2449)
    ("OCR DOUBLE BACKSLASH" #x244A)
    ))

;; arch-tag: e532db31-38fd-4eee-8e47-974316fb6dd5
